#ifndef __DEFINES__
#define __DEFINES__

#include <unistd.h>
#include <stdio.h>
#include <pthread.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <pcap.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>

#ifdef _FLOW_ADDER
#include <statistics.h>
#endif


#ifdef _SOLARIS
 #include <sys/ethernet.h>
 #include <inttypes.h>
#else
 #include <net/ethernet.h>
#endif

#define __FAVOR_BSD
 #include <netinet/tcp.h>
 #include <netinet/udp.h>
#undef __FAVOR_BSD

#ifndef ETHER_HDR_LEN
 #define ETHER_HDR_LEN sizeof(struct ether_header)
#endif

#define VERSION "2.0"

#define MAX_BUFFER 2048
#define MAX_LINE 512

#define MAX_LENGTH_IFACE 10

#define MAX_PROTOCOL 4
#define HASH_TCP 0
#define HASH_UDP 1
#define HASH_ICMP 2
#define HASH_OTHERS 3

#define GIGA 1073741824L
#define GIGA_MODIF 1000000000L

#define GET_HASH(source_ip, source_port, destination_ip, destination_port, max)	( (((source_ip.s_addr*953) % max) + 			\
																				   ((source_port*2677) % max) + 				\
																				   ((destination_ip.s_addr*953) % max) + 		\
																				   ((destination_port*2677) % max)) % max)
#define GET_HASH_TIME(seconds, max) (seconds % max)

#ifdef _LISTENER
#define NUMBER_OF_HASHES	1
#endif
#ifdef _FLOW_ADDER
#define NUMBER_OF_HASHES	((MAX_PROTOCOL*10) + 10)
#endif

typedef struct RAW_PACKAGE{
	struct ip ip_h;
	union {
		struct icmp icmp_h;
		struct tcphdr tcp_h;
		struct udphdr udp_h;
	};
	struct timeval ts;
} raw_package;

typedef struct PROCESSED_PACKAGE {
	struct timeval start_time;
	struct timeval end_time;
	u_char protocol;
	struct in_addr source_address;
	struct in_addr destination_address;
	unsigned int source_port;
	unsigned int destination_port;
	unsigned long source_gb;
	unsigned long source_b;
	unsigned long destination_gb;
	unsigned long destination_b;
	unsigned long source_packages_g;
	unsigned long source_packages;
	unsigned long destination_packages_g;
	unsigned long destination_packages;
#ifdef _LISTENER
	unsigned int flow_state;
#endif
#ifdef _FLOW_ADDER
	long real_begin;
	long real_end;
	int aggregated;
#endif
} processed_package;

typedef struct MEMORY_ELEMENT {
	union {
		raw_package raw;
		processed_package processed;
	};
#ifdef _FLOW_ADDER
	struct MEMORY_ELEMENT *previous;
#endif
	struct MEMORY_ELEMENT *next;
} memory_element;

typedef struct {
	char interface[MAX_LENGTH_IFACE];
	unsigned int position;
	// Packets pending for process
	memory_element *free;
	memory_element *last_free;		// Donde concatenaremos la memoria alocatada...
	unsigned long cnt_free_mem;
	// Semaphore access
	pthread_mutex_t mutual_exclusion_access;
	// Time for .debug file
	time_t tmin;
	time_t tmax;
#ifdef _LISTENER
	pcap_t *pcap_descriptor;
	memory_element *first_process;
	memory_element *last_process;
	unsigned long dropped_packages_g;
	unsigned long dropped_packages;
	// Package container
	memory_element **array[MAX_PROTOCOL];
#endif
#ifdef _FLOW_ADDER
	memory_element **array_captured[MAX_PROTOCOL];		// Converted to hash table
	memory_element **array_processed[MAX_PROTOCOL];		// Converted to hash table
	memory_element **aux_array;
	memory_element **aux_array_zero;
	// Statistics fields
	unsigned long memory_allocated;
	unsigned long memory_used;
	unsigned long aggregated[MAX_PROTOCOL];
	unsigned long free_memory_released;
	unsigned long memory_recycled;
	unsigned long memory_recycled_protocol[MAX_PROTOCOL];
	unsigned long memory_released_captured[MAX_PROTOCOL];
	unsigned long memory_released_processed[MAX_PROTOCOL];
	// More statistics fields
	unsigned long long total_connections_ll;
	double total_connections_d;
	unsigned long long protocol_connections_ll[MAX_PROTOCOL];
	double protocol_connections_d[MAX_PROTOCOL];
	unsigned long long total_bytes_ll;
	double total_bytes_d;
	unsigned long long protocol_bytes_ll[MAX_PROTOCOL];
	double protocol_bytes_d[MAX_PROTOCOL];
	statistics_element_bytes **total_top_bytes_received;
	statistics_element_bytes **total_top_bytes_received_protocol[MAX_PROTOCOL];
	statistics_element_bytes **local_top_bytes_received;
	statistics_element_bytes **local_top_bytes_received_protocol[MAX_PROTOCOL];
	statistics_element_bytes **total_top_bytes_sent;
	statistics_element_bytes **total_top_bytes_sent_protocol[MAX_PROTOCOL];
	statistics_element_bytes **local_top_bytes_sent;
	statistics_element_bytes **local_top_bytes_sent_protocol[MAX_PROTOCOL];
	statistics_element_accessed **total_top_connections_received;
	statistics_element_accessed **total_top_connections_received_protocol[MAX_PROTOCOL];
	statistics_element_accessed **local_top_connections_received;
	statistics_element_accessed **local_top_connections_received_protocol[MAX_PROTOCOL];
	statistics_element_accessed **total_top_connections_sent;
	statistics_element_accessed **total_top_connections_sent_protocol[MAX_PROTOCOL];
	statistics_element_accessed **local_top_connections_sent;
	statistics_element_accessed **local_top_connections_sent_protocol[MAX_PROTOCOL];
	statistics_element_ports **total_top_ports_accessed[MAX_PROTOCOL];
	statistics_element_ports **local_top_ports_accessed[MAX_PROTOCOL];
#endif
} bytes_and_packets;

#ifdef _LISTENER
typedef struct {
	unsigned long captured_gb;
	unsigned long captured_b;
	unsigned long overflow_captured;
	unsigned long packets_captured_g;
	unsigned long packets_captured;
	unsigned long overflow_packets_captured;
	unsigned long processed_gb;
	unsigned long processed_b;
	unsigned long overflow_processed;
	unsigned long packets_processed_g;
	unsigned long packets_processed;
	unsigned long overflow_packets_processed;
	unsigned long packets_recycled_g;
	unsigned long packets_recycled;
	unsigned long overflow_packets_recycled;
	unsigned long memory_packets_allocated;	// How many cells are allocated (total)
	unsigned long memory_packets_released;	// How many cells are freeded (total)
	unsigned long memory_packets_released_free;	// How many cells are freeded (from free)
	unsigned long memory_packets_released_allocated;	// How many cells are freeded (allocate)
	unsigned long memory_packets_used;		// How many cells are used (total)
	pthread_mutex_t mutex_debug;			// Mutex
	unsigned long queued_packets_g;			// How many packets are pending to process at any time
	unsigned long queued_packets;			// How many packets are pending to process at any time
	unsigned long overflow_queued_packets;	
	unsigned long queued_packets_highwatermark_g;		// How many packets are pending to process at any time (MAX)
	unsigned long queued_packets_highwatermark;			// How many packets are pending to process at any time (MAX)
	unsigned long overflow_queued_packets_highwatermark;
	unsigned long overflow_source;
	unsigned long overflow_destination;
	unsigned long overflow_source_packages;
	unsigned long overflow_destination_packages;
	unsigned long overflow_dropped_packages;
} debug_str;
#endif

#define STATISTICS_CAPTURE

// #define DEBUG_P

void save_debug (char *interface, char *message);

#endif /* __DEFINES__ */
