#ifndef __STATISTICS__
#define __STATISTICS__

#define MODE_DOUBLE 0x12
#define MODE_LONGLONG 0x21

typedef struct STATISTICS_ELEMENT_BYTES {
	struct in_addr ip_address;
	union {
		unsigned long long ll_bytes;
		double d_bytes;
	};
	struct STATISTICS_ELEMENT_BYTES *next;
} statistics_element_bytes;

typedef struct STATISTICS_ELEMENT_ACCESSED {
	struct in_addr ip_address;
	union {
		unsigned long long ll_accessed;
		double d_accessed;
	};
	struct STATISTICS_ELEMENT_ACCESSED *next;
} statistics_element_accessed;

typedef struct STATISTICS_ELEMENT_PORTS {
	unsigned int port_number;
	union {
		unsigned long long ll_times;
		double d_times;
	};
	struct STATISTICS_ELEMENT_PORTS *next;
} statistics_element_ports;

#define GET_HASH_BYTES(bytes, max) (((unsigned long) bytes) %  max)
#define GET_HASH_ACCESSED(accessed, max) (((unsigned long) accessed) %  max)
#define GET_HASH_PORTS(times, max) (((unsigned long) times) %  max)

void deallocate_element_bytes (statistics_element_bytes *package);
void deallocate_element_accessed (statistics_element_accessed *package);
void deallocate_element_ports (statistics_element_ports *package);

int remove_list_bytes (unsigned long max);
int remove_list_accessed (unsigned long max);
int remove_list_ports (unsigned long max);

statistics_element_bytes *get_top_bytes (int mode, unsigned long max);
statistics_element_accessed *get_top_accessed (int mode, unsigned long max);
statistics_element_ports *get_top_ports (int mode, unsigned long max);

void reorder_hash_bytes (char *interface, statistics_element_bytes **list, unsigned long max, int mode);
void reorder_hash_accessed (char *interface, statistics_element_accessed **list, unsigned long max, int mode);
void reorder_hash_ports (char *interface, statistics_element_ports **list, unsigned long max, int mode);

statistics_element_bytes *add_address_bytes_statistics (char *interface, statistics_element_bytes **list, struct in_addr ip_address, unsigned long long bytes, int mode);
statistics_element_accessed *add_address_accessed_statistics (char *interface, statistics_element_accessed **list, struct in_addr ip_address, unsigned long times, int mode);
statistics_element_ports *add_address_ports_statistics (char *interface, statistics_element_ports **list, unsigned int port_number, unsigned long times, int mode);

#endif /* __STATISTICS__ */
