#include <defines.h>
#include <common.h>
#include <errno.h>

extern bytes_and_packets *iface_information;
extern char buffer_debug[MAX_BUFFER];

char *remove_blanks (char *word)
{
	char aux_word[MAX_LINE];
	int i, j;
	int word_length = strlen(word);

	j = 0;
	for (i=0; i<word_length; i++)
	{
		if (word[i] != ' ')
		{
			aux_word[j] = word[i];
			j++;
		}
	}
	aux_word[j] = '\0';
	strcpy(word, aux_word);

	return word;
}

char *remove_initial_final_blanks (char *word)
{
	char aux_word[MAX_LINE];
	int first_non_blank, last_non_blank;
	int word_length = strlen(word);

	memset(aux_word, '\0', MAX_LINE*sizeof(char));

	first_non_blank = 0;
	while (word[first_non_blank] == ' ') {
		first_non_blank++;
	}
	last_non_blank = word_length-1;
	while (word[last_non_blank] == ' ') {
		last_non_blank--;
	}
	strncpy(aux_word, &word[first_non_blank], (last_non_blank - first_non_blank + 1));
	strcpy(word, aux_word);

	return word;
}

char *give_filename_from_last_slash (char *file_path, char *result)
{
	char aux_path[MAX_LINE], aux_path2[MAX_LINE];
	int actual, bg;

	memset(aux_path, '\0', MAX_LINE*sizeof(char));
	memset(aux_path2, '\0', MAX_LINE*sizeof(char));

	actual = strlen(file_path) - 1;
	bg = 0;
	while (file_path[actual] != '/') {
		aux_path[bg] = file_path[actual];
		actual--;
		bg++;
	}
	// Salimos cuando hemos encontrado una barra... ahora le damos la vuelta al string
	actual = strlen(aux_path) - 1;
	bg = 0;
	while (actual >= 0) {
		aux_path2[bg] = aux_path[actual];
		actual--;
		bg++;
	}
	strcpy(result, aux_path2);
	return result;
}

void replace_string_with(char *line, char *to_substitute_token, char *final_token)
{
	int max_string = strlen(line);
	int max_first_token = strlen(to_substitute_token);
	char aux_line[MAX_LINE];

	memset(aux_line, '\0', sizeof(char)*MAX_LINE);
	max_string = max_string - max_first_token;
	strncpy(aux_line, line, max_string);
	sprintf(aux_line, "%s%s", aux_line, final_token);
	strcpy(line, aux_line);
}

int read_line(int fd, char *line)
{
	int res, i = 0;
	int salir = 0;
	int comentario = 0;
	char c;

	while (!salir) {
		res = read(fd, &c, 1);
		if (res != 1) salir = 2;
		else {
			if (c == '\n') salir = 1;
			else {
			    if ((c == ';') || (c == '#')) comentario = 1;
			    else {
			    	if (!comentario) {
						line[i] = c;
						i++;
					}
				}
			}
		}
	}
	line[i] = '\0';

	if (salir == 2) return -1;
	return 0;
}

unsigned long allocate_memory (bytes_and_packets *iface, unsigned long packets_to_allocate)
{
	int return_value = 0;
	unsigned int i = 0;
	unsigned long really_packets_allocated = 0;
	memory_element *aux_package = NULL;
	memory_element *new_package = NULL;
	memory_element *tmp_package = NULL;

	while ( (i < packets_to_allocate) && (return_value == 0) ) {
		if ( (new_package = (memory_element *) malloc (sizeof(memory_element))) != NULL ) {
			really_packets_allocated++;
			if (aux_package != NULL) {
				new_package->next = NULL;
				aux_package->next = new_package;
				aux_package = new_package;
			}
			else {
				tmp_package = new_package;
				tmp_package->next = NULL;
				aux_package = tmp_package;
			}
		}
		else {
			sprintf(buffer_debug, "ERROR allocating memory");
			save_debug (iface->interface, buffer_debug);
			return_value = -1;
		}
		if (return_value == 0) i++;
	}
	CONCATENATE_FREE_BLOCK(tmp_package, aux_package, iface->free, iface->last_free, iface->cnt_free_mem, really_packets_allocated, iface->mutual_exclusion_access);

	return really_packets_allocated;
}

long purge_sparse_memory(bytes_and_packets *iface, long packets_to_free)
{
	long remaining;
	memory_element *aux_element;

	// Cogemos el listado de paquetes libres y reducimos el numero que nos indican
	remaining = packets_to_free;
	while ((remaining > 0) && (iface->free != NULL)) {
		aux_element = iface->free;
		iface->free = iface->free->next;
		free(aux_element);
		iface->cnt_free_mem--;
#ifdef _FLOW_ADDER
		iface->memory_allocated--;
#endif
		remaining--;
	}

	if (iface->free == NULL) iface->last_free = NULL;

	return (packets_to_free - remaining);
}


int create_file_system (char *initial_directory, char *output_file, int type, int create_dirs, int use_time_passed, time_t time_passed, int fuid, int fgid)
{
	time_t actual_time;
	struct tm time_struct, *time_ptr_str;
	char command_create_fsalida[MAX_BUFFER];
	int semana, dia_semana, dia_mes, semana_mes;
	struct stat sb;

	// Primero de todo, obtenemos el tiempo...
	if (use_time_passed)
		actual_time = time_passed;
	else {
		if (time(&actual_time) < 0) {
			sprintf(buffer_debug, "Error obtaining actual time...");
			save_debug ("listener", buffer_debug);
			return -1;
		}
	}

	// Y lo transformamos de manera que sea mas sencillo acceder a el
	time_ptr_str = localtime(&actual_time);
	memcpy(&time_struct, time_ptr_str, sizeof(struct tm));

	if (create_dirs) {
		// Creamos la estructura de los directorios de salida y modificamos la variable "output_file"
		sprintf(command_create_fsalida, "mkdir -p %s", initial_directory);
		system(command_create_fsalida);
		chown(initial_directory, fuid, fgid);
		chmod(initial_directory, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);

		if(stat(initial_directory, &sb) < 0)
		{
			printf("Cann't create %s directory\n", initial_directory);
			return -1;
		}
		if(!S_ISDIR(sb.st_mode))
		{
			printf("%s is not a directory\n", initial_directory);
			return -1;
		}
	}

	sprintf(output_file, "%s", initial_directory);

	if (type == TYPE_CAPTURER) {
		// Anyadimos ANYO
		sprintf(output_file, "%s/%04d", output_file, (1900+time_struct.tm_year));

		if (create_dirs) {
			if (mkdir(output_file, 0770) < 0) {
				if (errno != EEXIST) {
					sprintf(buffer_debug, "Error in creating %s: %s", output_file, strerror(errno));
					save_debug ("createfs", buffer_debug);
					return -1;
				}
			}
			chown(output_file, fuid, fgid);
			chmod(output_file, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
		}

		// Anyadimos MES
		sprintf(output_file, "%s/%02d", output_file, (1+time_struct.tm_mon));
		
		// Y, finalmente, anyadimos DIA
		sprintf(output_file, "%s-%02d", output_file, time_struct.tm_mday);
		
		// Y creamos la carpeta contenedora para ese dia...
		//   /$data_capture/fecha/fichero
		//   fecha  : yyyy/mm-dd
		//   fichero: ssssssssss.interface.data

		if (create_dirs) {
			if (mkdir(output_file, 0770) < 0) {
				if (errno != EEXIST) {
					sprintf(buffer_debug,"Error in creating %s: %s", output_file, strerror(errno));
					save_debug ("createfs", buffer_debug);
					return -1;
				}
			}
			chown(output_file, fuid, fgid);
			chmod(output_file, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
		}

		// Y acabamos de afinar el nombre del fichero con la ssssssssss.interface.data
		sprintf(output_file, "%s/%10lu", output_file, (unsigned long) mktime(&time_struct));
	}
	else if (type == TYPE_PROCESSER) {
		// Y seguimos creando los subdirectorios... ANYO
		sprintf(output_file, "%s/%d", output_file, (1900+time_struct.tm_year));

		if (create_dirs) {
			if (mkdir(output_file, 0770) < 0) {
				if (errno != EEXIST) {
					sprintf(buffer_debug,"Error in creating %s: %s", output_file, strerror(errno));
					save_debug ("createfs", buffer_debug);
					return -1;
				}
			}
			chown(output_file, fuid, fgid);
			chmod(output_file, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
		}
		
		// Y seguimos creando los subdirectorios... MES
		sprintf(output_file, "%s/%02d", output_file, (1+time_struct.tm_mon));
		if (create_dirs) {
			if (mkdir(output_file, 0770) < 0) {
				if (errno != EEXIST) {
					sprintf(buffer_debug,"Error in creating %s: %s", output_file, strerror(errno));
					save_debug ("createfs", buffer_debug);
					return -1;
				}
			}
			chown(output_file, fuid, fgid);
			chmod(output_file, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
		}

		// semana, dia_semana, dia_mes
		semana = 1;
		dia_mes = time_struct.tm_mday;
		dia_semana = time_struct.tm_wday;
		if (dia_semana == 0) dia_semana = 7;
		semana_mes = dia_mes - (dia_semana - 1);
		while ( semana_mes > 1 ) {
			semana++;
			semana_mes = semana_mes - 7;
		}

		// Y seguimos creando los subdirectorios... SEMANA
		sprintf(output_file, "%s/%02d", output_file, semana);
		if (create_dirs) {
			if (mkdir(output_file, 0770) < 0) {
				if (errno != EEXIST) {
					sprintf(buffer_debug,"Error in creating %s: %s", output_file, strerror(errno));
					save_debug ("createfs", buffer_debug);
					return -1;
				}
			}
			chown(output_file, fuid, fgid);
			chmod(output_file, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
		}

		// Y seguimos creando los subdirectorios... DIA (1->lunes, 2->martes, 3->miercoles, 4->jueves, 5->viernes, 6->sabado, 7->domingo)
		dia_semana = time_struct.tm_wday;
		if (dia_semana == 0) dia_semana = 7;
		sprintf(output_file, "%s/%02d", output_file, dia_semana);
		if (create_dirs) {
			if (mkdir(output_file, 0770) < 0) {
				if (errno != EEXIST) {
					sprintf(buffer_debug,"Error in creating %s: %s", output_file, strerror(errno));
					save_debug ("createfs", buffer_debug);
					return -1;
				}
			}
			chown(output_file, fuid, fgid);
			chmod(output_file, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
		}

		// Y acabamos de afinar el nombre del fichero con la ssssssssss.interface.data
		sprintf(output_file, "%s/%10lu", output_file, (unsigned long) mktime(&time_struct));
	}
	return 0;
}
