#include <defines.h>
#include <flow_adder.h>
#include <common.h>

#include <sys/types.h>
#include <dirent.h>

extern int capture_time;
extern unsigned long max_packets;
extern unsigned long min_packets;
extern unsigned long max_hash;
extern int file_gid;
extern char *compress_bin_path;
extern char *uncompress_bin_path;
extern int keep_days_captured;
extern int top_n;
extern char *local_network;
extern long unfinished_connections_seconds;

extern char buffer_debug[MAX_BUFFER];

bytes_and_packets *iface_information;
int use_gzip_or_bzip;	// 0 -> gzip  |  1 -> bzip2

void initialize_bytes_and_packets (unsigned int max_intrfacs, interface_str *interface)
{
	unsigned int i, k;

	// First, we must allocate iface_information space
	iface_information = (bytes_and_packets *) malloc (max_intrfacs*sizeof(bytes_and_packets));

	for ( i = 0; i < max_intrfacs; i++) {
		pthread_mutex_init(&iface_information[i].mutual_exclusion_access, NULL);
		strcpy(iface_information[i].interface, interface[i].name);
		iface_information[i].position = i;
		iface_information[i].free = NULL;
		iface_information[i].last_free = NULL;
		iface_information[i].cnt_free_mem = 0;
		iface_information[i].tmin = -1;
		iface_information[i].tmax = -1;
		for (k = 0 ; k < MAX_PROTOCOL ; k++ ) {
			iface_information[i].array_captured[k] = (memory_element **) calloc(max_hash, sizeof(memory_element *));
			iface_information[i].array_processed[k] = (memory_element **) calloc(max_hash, sizeof(memory_element *));
			iface_information[i].protocol_connections_ll[k] = 0;
			iface_information[i].protocol_connections_d[k] = 0;
			iface_information[i].protocol_bytes_ll[k] = 0;
			iface_information[i].protocol_bytes_d[k] = 0;
			iface_information[i].aggregated[k] = 0;
			iface_information[i].memory_recycled_protocol[k] = 0;
			iface_information[i].memory_released_captured[k] = 0;
			iface_information[i].memory_released_processed[k] = 0;
			iface_information[i].total_top_bytes_received_protocol[k] = (statistics_element_bytes **) calloc(max_hash, sizeof(statistics_element_bytes *));
			iface_information[i].local_top_bytes_received_protocol[k] = (statistics_element_bytes **) calloc(max_hash, sizeof(statistics_element_bytes *));
			iface_information[i].total_top_bytes_sent_protocol[k] = (statistics_element_bytes **) calloc(max_hash, sizeof(statistics_element_bytes *));
			iface_information[i].local_top_bytes_sent_protocol[k] = (statistics_element_bytes **) calloc(max_hash, sizeof(statistics_element_bytes *));
			iface_information[i].total_top_connections_received_protocol[k] = (statistics_element_accessed **) calloc(max_hash, sizeof(statistics_element_accessed *));
			iface_information[i].local_top_connections_received_protocol[k] = (statistics_element_accessed **) calloc(max_hash, sizeof(statistics_element_accessed *));
			iface_information[i].total_top_connections_sent_protocol[k] = (statistics_element_accessed **) calloc(max_hash, sizeof(statistics_element_accessed *));
			iface_information[i].local_top_connections_sent_protocol[k] = (statistics_element_accessed **) calloc(max_hash, sizeof(statistics_element_accessed *));
			iface_information[i].total_top_ports_accessed[k] = (statistics_element_ports **) calloc(max_hash, sizeof(statistics_element_ports *));
			iface_information[i].local_top_ports_accessed[k] = (statistics_element_ports **) calloc(max_hash, sizeof(statistics_element_ports *));
		}
		// Para clasificar el volcado a disco, crearemos un par mas de hashes...
		iface_information[i].aux_array = (memory_element **) calloc(max_hash, sizeof(memory_element *));
		iface_information[i].aux_array_zero = (memory_element **) calloc(max_hash, sizeof(memory_element *));
		iface_information[i].memory_allocated = allocate_memory(&iface_information[i], max_packets);
		iface_information[i].free_memory_released = 0;
		iface_information[i].memory_recycled = 0;
		iface_information[i].memory_used = 0;
		// statistics
		iface_information[i].total_connections_ll = 0;
		iface_information[i].total_connections_d = 0;
		iface_information[i].total_bytes_ll = 0;
		iface_information[i].total_bytes_d = 0;
		iface_information[i].total_top_bytes_received = (statistics_element_bytes **) calloc(max_hash, sizeof(statistics_element_bytes *));
		iface_information[i].local_top_bytes_received = (statistics_element_bytes **) calloc(max_hash, sizeof(statistics_element_bytes *));
		iface_information[i].total_top_bytes_sent = (statistics_element_bytes **) calloc(max_hash, sizeof(statistics_element_bytes *));
		iface_information[i].local_top_bytes_sent = (statistics_element_bytes **) calloc(max_hash, sizeof(statistics_element_bytes *));
		iface_information[i].total_top_connections_received = (statistics_element_accessed **) calloc(max_hash, sizeof(statistics_element_accessed *));
		iface_information[i].local_top_connections_received = (statistics_element_accessed **) calloc(max_hash, sizeof(statistics_element_accessed *));
		iface_information[i].total_top_connections_sent = (statistics_element_accessed **) calloc(max_hash, sizeof(statistics_element_accessed *));
		iface_information[i].local_top_connections_sent = (statistics_element_accessed **) calloc(max_hash, sizeof(statistics_element_accessed *));
	}
}

int is_a_directory (char *filename)
{
	// Function that returns 1 if the filename is a directory
	// otherwise, return 0
	DIR *aux_dir;
	int return_val;

	aux_dir = opendir (filename);
	if (aux_dir == NULL) return_val = 0;
	else {
		return_val = 1;
		closedir (aux_dir);
	}

	return return_val;
}

int is_local (char *internet_addr)
{
	char *string_pointer;
	char aux_line[MAX_LINE], aux_line2[MAX_LINE];
	char separator_chars[2]=",";
	int exit_while;

	strcpy(aux_line, local_network);
	string_pointer = strtok (aux_line, separator_chars);
	exit_while = 0;
	while ((string_pointer != NULL) && !(exit_while)) {
		strcpy(aux_line2, string_pointer);
		remove_initial_final_blanks (aux_line2);
		if (strstr(internet_addr, aux_line2) != NULL) exit_while = 1;
		string_pointer = strtok (NULL, separator_chars);
	}

	return exit_while;
}

void print_and_reset_some_counters (bytes_and_packets *bytes_and_packs, int printf_summary)
{
	unsigned int k;

	if (printf_summary) {
		sprintf(buffer_debug, "[AGGREGATIONS] TCP: %lu, UDP: %lu, ICMP: %lu, OTHERS: %lu", bytes_and_packs->aggregated[HASH_TCP], bytes_and_packs->aggregated[HASH_UDP], bytes_and_packs->aggregated[HASH_ICMP], bytes_and_packs->aggregated[HASH_OTHERS]);
		save_debug (bytes_and_packs->interface, buffer_debug);
		sprintf(buffer_debug, "[MEMORY] Allocated: %lu, Used: %lu, Recycled: %lu [TCP: %lu, UDP: %lu, ICMP: %lu, OTHERS: %lu]", bytes_and_packs->memory_allocated, bytes_and_packs->memory_used, bytes_and_packs->memory_recycled, bytes_and_packs->memory_recycled_protocol[HASH_TCP], bytes_and_packs->memory_recycled_protocol[HASH_UDP], bytes_and_packs->memory_recycled_protocol[HASH_ICMP], bytes_and_packs->memory_recycled_protocol[HASH_OTHERS]);
		save_debug (bytes_and_packs->interface, buffer_debug);
	}

	// Ponemos a 0 las estadisticas ya volcadas
	bytes_and_packs->memory_recycled = 0;
	bytes_and_packs->total_connections_ll = 0;
	bytes_and_packs->total_connections_d = 0;
	bytes_and_packs->total_bytes_ll = 0;
	bytes_and_packs->total_bytes_d = 0;
	for (k = 0; k < MAX_PROTOCOL; k++) {
		bytes_and_packs->aggregated[k] = 0;
		bytes_and_packs->memory_recycled_protocol[k] = 0;
		bytes_and_packs->protocol_connections_ll[k] = 0;
		bytes_and_packs->protocol_connections_d[k] = 0;
		bytes_and_packs->protocol_bytes_ll[k] = 0;
		bytes_and_packs->protocol_bytes_d[k] = 0;
	}
}

void free_package(memory_element *package, bytes_and_packets *bytes_and_packs_ptr)
{
	int protocol;

	if (package->processed.protocol == IPPROTO_TCP) protocol = HASH_TCP;
	else if (package->processed.protocol == IPPROTO_UDP) protocol = HASH_UDP;
	else if (package->processed.protocol == IPPROTO_ICMP) protocol = HASH_ICMP;
	else protocol = HASH_OTHERS;

	// Attach the package to free_packages block list
	package->next = NULL;
	package->previous = NULL;
	CONCATENATE_FREE_BLOCK(package, package, bytes_and_packs_ptr->free, bytes_and_packs_ptr->last_free, bytes_and_packs_ptr->cnt_free_mem, 1, bytes_and_packs_ptr->mutual_exclusion_access);
	bytes_and_packs_ptr->memory_recycled++;
	bytes_and_packs_ptr->memory_recycled_protocol[protocol]++;
	bytes_and_packs_ptr->memory_used--;
}

void free_all_memory(int position)
{
	unsigned int k;
	unsigned long j;
	bytes_and_packets *bytes_and_packs_ptr;
	memory_element *memory_to_free = NULL;
	memory_element *memory_aux = NULL;

	bytes_and_packs_ptr = &iface_information[position];

	// Primero liberamos la lista de bloques libres
	memory_to_free = bytes_and_packs_ptr->free;
	while (memory_to_free != NULL) {
		memory_aux = memory_to_free;
		memory_to_free = memory_to_free->next;
		free(memory_aux);
		bytes_and_packs_ptr->free_memory_released++;
	}

	for ( k = 0 ; k < MAX_PROTOCOL ; k++) {
		// Y seguidamente, la que hay ocupada en memoria (captured)
		for (j = 0 ; j < max_hash; j++) {
			memory_to_free = bytes_and_packs_ptr->array_captured[k][j];
			while ( memory_to_free != NULL ) {
				memory_aux = memory_to_free;
				memory_to_free = memory_to_free->next;;
				free(memory_aux);
				bytes_and_packs_ptr->memory_released_captured[k]++;
			}
		}
		free (bytes_and_packs_ptr->array_captured[k]);
	}
	for ( k = 0 ; k < MAX_PROTOCOL ; k++) {
		// Y seguidamente, la que hay ocupada en memoria (processed)
		for (j = 0 ; j < max_hash; j++) {
			memory_to_free = bytes_and_packs_ptr->array_processed[k][j];
			while ( memory_to_free != NULL ) {
				memory_aux = memory_to_free;
				memory_to_free = memory_to_free->next;;
				free(memory_aux);
				bytes_and_packs_ptr->memory_released_processed[k]++;
			}
		}
		free (bytes_and_packs_ptr->array_processed[k]);
	}
}

memory_element *search_flow (memory_element *package, int *direction, memory_element *hash_package)
{
	int exit_function;
	int found = 0;
	int direction_aux = 0;
	memory_element *actual;

	actual = hash_package;

	if (actual != NULL) exit_function = 0;
	else exit_function = 1;

	while (!exit_function) {
		if ((package->processed.source_address.s_addr == actual->processed.source_address.s_addr) && (package->processed.source_port == actual->processed.source_port) && \
			(package->processed.destination_address.s_addr == actual->processed.destination_address.s_addr) && (package->processed.destination_port == actual->processed.destination_port) && \
			(package->processed.protocol == actual->processed.protocol)) {
				// Hemos encontrado una entrada con las mismas ips, puertos y protocolos.. comprobamos que no haya acabado (tiempos distintos de 0)...
				if (((actual->processed.end_time.tv_sec == 0) && (package->processed.start_time.tv_sec == 0)) || \
					((package->processed.end_time.tv_sec == 0) && (actual->processed.start_time.tv_sec == 0))) {
						found = 1;
						direction_aux = 1;
						exit_function = 1;
				}
				// En caso contrario, seguimos buscando...
		}

		// Tambien tenemos que vigilar que el hash_package no venga de vuelta...
		if ((package->processed.source_address.s_addr == actual->processed.destination_address.s_addr) && (package->processed.source_port == actual->processed.destination_port) && \
			(package->processed.destination_address.s_addr == actual->processed.source_address.s_addr) && (package->processed.destination_port == actual->processed.source_port) && \
			(package->processed.protocol == actual->processed.protocol)) {
				// Hemos encontrado una entrada con las mismas ips, puertos y protocolos.. comprobamos que no haya acabado (tiempos distintos de 0)...
				if (((actual->processed.end_time.tv_sec == 0) && (package->processed.start_time.tv_sec == 0)) || \
					((package->processed.end_time.tv_sec == 0) && (actual->processed.start_time.tv_sec == 0))) {
						found = 1;
						direction_aux = -1;
						exit_function = 1;
				}
				// En caso contrario, seguimos buscando...
		}

		if (!found) {
			if (actual->previous != NULL) {
				actual = actual->previous;
			}
			else exit_function = 1;
		}
	}
	memcpy(direction,&direction_aux,sizeof(direction_aux));	

	if (found) return actual;
	else return NULL;
}

void insert_without_order (memory_element *package, bytes_and_packets *bytes_and_packs_ptr, int mode)
{
	// Insertamos en la tabla de hash... sin orden... (ya lo ordenaremos antes de volcar)
	memory_element *actual = NULL, *aux_element = NULL;
	int protocol, direction;
	unsigned long hash_result;

	if (package->processed.protocol == IPPROTO_TCP) protocol = HASH_TCP;
	else if (package->processed.protocol == IPPROTO_UDP) protocol = HASH_UDP;
	else if (package->processed.protocol == IPPROTO_ICMP) protocol = HASH_ICMP;
	else protocol = HASH_OTHERS;

	hash_result = GET_HASH(package->processed.source_address, package->processed.source_port, package->processed.destination_address, package->processed.destination_port, max_hash);

	if (mode == DOT_CAPTURED) {
		actual = bytes_and_packs_ptr->array_captured[protocol][hash_result];
	}
	else if (mode == DOT_PROCESSED) {
		actual = bytes_and_packs_ptr->array_processed[protocol][hash_result];
	}

	if (actual == NULL) {
		// Estamos en el caso que no hay ningun elemento insertado
		package->next = package->previous = NULL;
		if (mode == DOT_CAPTURED) {
			bytes_and_packs_ptr->array_captured[protocol][hash_result] = package;
		}
		else if (mode == DOT_PROCESSED) {
			bytes_and_packs_ptr->array_processed[protocol][hash_result] = package;
		}
	}
	else {
		// En caso contrario, lo buscamos y si no lo encontramos lo agregamos al principio (es indiferente)
		aux_element = search_flow (package, &direction, actual);

		if ( aux_element != NULL ) {
			// Lo hemos encontrado... lo actualizamos
			aux_element->processed.aggregated = 1;
			bytes_and_packs_ptr->aggregated[protocol]++;
			// Simplemente, actualizaremos campos...
			// Primero, el tiempo...
			if ((aux_element->processed.end_time.tv_sec == 0) && (package->processed.start_time.tv_sec == 0)) {
				aux_element->processed.end_time.tv_sec = package->processed.end_time.tv_sec;
				aux_element->processed.end_time.tv_usec = package->processed.end_time.tv_usec;
				aux_element->processed.real_end = package->processed.real_end;
			}
			else if ((package->processed.end_time.tv_sec == 0) && (aux_element->processed.start_time.tv_sec == 0)) {
				aux_element->processed.start_time.tv_sec = package->processed.start_time.tv_sec;
				aux_element->processed.start_time.tv_usec = package->processed.start_time.tv_usec;
				aux_element->processed.real_begin = package->processed.real_begin;
			}
			// Y despues, contadores de informacion y paquetes intercambiados...
			if (direction == 1) {
				// de origen a destino
				RAISE_INFORMATION(aux_element->processed.source_packages, aux_element->processed.source_packages_g, package->processed.source_packages, package->processed.source_packages_g);
				RAISE_INFORMATION(aux_element->processed.destination_packages, aux_element->processed.destination_packages_g, package->processed.destination_packages, package->processed.destination_packages_g);
				RAISE_INFORMATION(aux_element->processed.source_b, aux_element->processed.source_gb, package->processed.source_b, package->processed.source_gb);
				RAISE_INFORMATION(aux_element->processed.destination_b, aux_element->processed.destination_gb, package->processed.destination_b, package->processed.destination_gb);
			}
			else if (direction == -1) {
				// de destino a origen
				RAISE_INFORMATION(aux_element->processed.source_packages, aux_element->processed.source_packages_g, package->processed.destination_packages, package->processed.destination_packages_g);
				RAISE_INFORMATION(aux_element->processed.destination_packages, aux_element->processed.destination_packages_g, package->processed.source_packages, package->processed.source_packages_g);
				RAISE_INFORMATION(aux_element->processed.source_b, aux_element->processed.source_gb, package->processed.destination_b, package->processed.destination_gb);
				RAISE_INFORMATION(aux_element->processed.destination_b, aux_element->processed.destination_gb, package->processed.source_b, package->processed.source_gb);
			}

			// Y liberamos el paquete...
			free_package(package, bytes_and_packs_ptr);
		}
		else {
			// Es el primer paquete que nos llega del flujo... lo insertamos al principio...
			package->processed.aggregated = 0;
			if (mode == DOT_CAPTURED) {
				package->previous = NULL;
				package->next = bytes_and_packs_ptr->array_captured[protocol][hash_result];
				if (bytes_and_packs_ptr->array_captured[protocol][hash_result] != NULL)
					bytes_and_packs_ptr->array_captured[protocol][hash_result]->previous = package;
				else
					bytes_and_packs_ptr->array_captured[protocol][hash_result]->previous = NULL;
				bytes_and_packs_ptr->array_captured[protocol][hash_result] = package;
			}
			else if (mode == DOT_PROCESSED) {
				package->previous = NULL;
				package->next = bytes_and_packs_ptr->array_processed[protocol][hash_result];
				if (bytes_and_packs_ptr->array_processed[protocol][hash_result] != NULL)
					bytes_and_packs_ptr->array_processed[protocol][hash_result]->previous = package;
				else
					bytes_and_packs_ptr->array_processed[protocol][hash_result]->previous = NULL;
				bytes_and_packs_ptr->array_processed[protocol][hash_result] = package;
			}
		}
	}
}

int compare_elements (memory_element *elem1, memory_element *elem2, int initial_or_final)
{
	if (initial_or_final == 0) {
		if (elem1->processed.start_time.tv_sec > elem2->processed.start_time.tv_sec) return 1;
		else if ((elem1->processed.start_time.tv_sec == elem2->processed.start_time.tv_sec) && \
				 (elem1->processed.start_time.tv_usec > elem2->processed.start_time.tv_usec)) return 1;
		else if (elem1->processed.start_time.tv_sec < elem2->processed.start_time.tv_sec) return 2;
		else if ((elem1->processed.start_time.tv_sec == elem2->processed.start_time.tv_sec) && \
				 (elem1->processed.start_time.tv_usec < elem2->processed.start_time.tv_usec)) return 2;
		else return 0;
	}
	else {
		if (elem1->processed.end_time.tv_sec > elem2->processed.end_time.tv_sec) return 1;
		else if ((elem1->processed.end_time.tv_sec == elem2->processed.end_time.tv_sec) && \
				 (elem1->processed.end_time.tv_usec > elem2->processed.end_time.tv_usec)) return 1;
		else if (elem2->processed.end_time.tv_sec > elem1->processed.end_time.tv_sec) return 2;
		else if ((elem2->processed.end_time.tv_sec == elem1->processed.end_time.tv_sec) && \
				 (elem2->processed.end_time.tv_usec > elem1->processed.end_time.tv_usec)) return 2;
		else return 0;
	}
}

void insert_to_ordered_hash (bytes_and_packets *bytes_and_packs, memory_element *package)
{
	// La idea es:
	// - Si el tiempo inicial != 0
	//   - Calcular el hash segun el tiempo inicial
	//   - Insertar ordenadamente en bytes_and_packs->aux_array
	// - Si es tiempo inicial == 0
	//   - Calcular el hash segun el tiempo final
	//   - Insertar ordenadamente en bytes_and_packs->aux_array_zero

	unsigned long local_hash;
	memory_element *actual, *previous;

	if (package->processed.start_time.tv_sec == 0) {
		// We must insert in aux_array_zero, ordered
		local_hash = GET_HASH_TIME(package->processed.end_time.tv_sec, max_hash);
		if (bytes_and_packs->aux_array_zero[local_hash] == NULL) {
			// First element...
			bytes_and_packs->aux_array_zero[local_hash] = package;
			package->next = NULL;
			package->previous = NULL;
		}
		else {
			// Not the first element...
			previous = NULL;
			actual = bytes_and_packs->aux_array_zero[local_hash];
			while ((actual != NULL) && ( compare_elements(actual, package, 1) == 2 )) {
				// Mientras el tiempo del paquete sea mas grande que el de la lista
				previous = actual;
				actual = actual->next;
			}
			if (actual == NULL) {
				// Hemos llegado al final de la lista... insertamos despues de previous
				package->next = NULL;
				package->previous = previous;
				previous->next = package;
			}
			else {
				// actual no es NULL. Verificamos si es el principio o no...
				if (previous == NULL) {
					// Principio de todo... debemos insertar antes de actual
					package->next = bytes_and_packs->aux_array_zero[local_hash];
					package->previous = NULL;
					bytes_and_packs->aux_array_zero[local_hash] = package;
				}
				else {
					// Por el medio de la lista... (entre previous y actual)
					package->next = actual;
					package->previous = previous;
					actual->previous = package;
					previous->next = package;
				}
			}
		}
	}
	else {
		// We must insert in aux_array
		local_hash = GET_HASH_TIME(package->processed.start_time.tv_sec, max_hash);
		if (bytes_and_packs->aux_array[local_hash] == NULL) {
			// First element...
			bytes_and_packs->aux_array[local_hash] = package;
			package->next = NULL;
			package->previous = NULL;
		}
		else {
			// Not the first element...
			previous = NULL;
			actual = bytes_and_packs->aux_array[local_hash];
			while ((actual != NULL) && ( compare_elements(actual, package, 1) == 2 )) {
				// Mientras el tiempo del paquete sea mas grande que el de la lista
				previous = actual;
				actual = actual->next;
			}
			if (actual == NULL) {
				// Hemos llegado al final de la lista... insertamos despues de previous
				package->next = NULL;
				package->previous = previous;
				previous->next = package;
			}
			else {
				// actual no es NULL. Verificamos si es el principio o no...
				if (previous == NULL) {
					// Principio de todo... debemos insertar antes de actual
					package->next = bytes_and_packs->aux_array[local_hash];
					package->previous = NULL;
					bytes_and_packs->aux_array[local_hash] = package;
				}
				else {
					// Por el medio de la lista... (entre previous y actual)
					package->next = actual;
					package->previous = previous;
					actual->previous = package;
					previous->next = package;
				}
			}
		}
	}
}

#ifdef _VIEWER
int save_statistics (char *file_name, bytes_and_packets *bytes_and_packs, int mode, char *output_file)
#else
int save_statistics (char *file_name, bytes_and_packets *bytes_and_packs, int mode)
#endif
{
	int fd, actual_top;
#ifdef _VIEWER
	int fd2;
	char aux_line2[MAX_LINE];
#endif
	char aux_line[MAX_LINE], statistics_file[MAX_LINE];
	statistics_element_bytes *aux_element_bytes;
	statistics_element_accessed *aux_element_accessed;
	statistics_element_ports *aux_element_ports;

	sprintf(statistics_file, "%s.%s.statistics.tmp", file_name, bytes_and_packs->interface);

	fd = open (statistics_file, O_WRONLY|O_CREAT|O_APPEND);
	if (fd < 0) {
		sprintf("Error creating %s... not saved", statistics_file);
		save_debug (bytes_and_packs->interface,buffer_debug);
		return -1;
	}

	// Escribimos el numero de conexiones

	// TOTAL
	if (mode == MODE_DOUBLE)
		sprintf(aux_line, "Total connections: %.0f\n", bytes_and_packs->total_connections_d);
	else if (mode == MODE_LONGLONG)
		sprintf(aux_line, "Total connections: %llu\n", bytes_and_packs->total_connections_ll);
	write(fd, aux_line, strlen(aux_line));

	// TCP
	if (mode == MODE_DOUBLE)
		sprintf(aux_line, "TCP connections: %.0f\n", bytes_and_packs->protocol_connections_d[HASH_TCP]);
	else if (mode == MODE_LONGLONG)
		sprintf(aux_line, "TCP connections: %llu\n", bytes_and_packs->protocol_connections_ll[HASH_TCP]);
	write(fd, aux_line, strlen(aux_line));

	// UDP
	if (mode == MODE_DOUBLE)
		sprintf(aux_line, "UDP connections: %.0f\n", bytes_and_packs->protocol_connections_d[HASH_UDP]);
	else if (mode == MODE_LONGLONG)
		sprintf(aux_line, "UDP connections: %llu\n", bytes_and_packs->protocol_connections_ll[HASH_UDP]);
	write(fd, aux_line, strlen(aux_line));

	// ICMP
	if (mode == MODE_DOUBLE)
		sprintf(aux_line, "ICMP connections: %.0f\n", bytes_and_packs->protocol_connections_d[HASH_ICMP]);
	else if (mode == MODE_LONGLONG)
		sprintf(aux_line, "ICMP connections: %llu\n", bytes_and_packs->protocol_connections_ll[HASH_ICMP]);
	write(fd, aux_line, strlen(aux_line));

	// OTHERS
	if (mode == MODE_DOUBLE)
		sprintf(aux_line, "OTHERS connections: %.0f\n", bytes_and_packs->protocol_connections_d[HASH_OTHERS]);
	else if (mode == MODE_LONGLONG)
		sprintf(aux_line, "OTHERS connections: %llu\n", bytes_and_packs->protocol_connections_ll[HASH_OTHERS]);
	write(fd, aux_line, strlen(aux_line));

	write(fd, "\n", strlen("\n"));

	// Y ahora, el ancho de banda

	// TOTAL
	if (mode == MODE_DOUBLE)
		sprintf(aux_line, "Total bandwidth (Bytes): %.0f\n", bytes_and_packs->total_bytes_d);
	else if (mode == MODE_LONGLONG)
		sprintf(aux_line, "Total bandwidth (Bytes): %llu\n", bytes_and_packs->total_bytes_ll);
	write(fd, aux_line, strlen(aux_line));

	// TCP
	if (mode == MODE_DOUBLE)
		sprintf(aux_line, "TCP bandwidth (Bytes): %.0f\n", bytes_and_packs->protocol_bytes_d[HASH_TCP]);
	else if (mode == MODE_LONGLONG)
		sprintf(aux_line, "TCP bandwidth (Bytes): %llu\n", bytes_and_packs->protocol_bytes_ll[HASH_TCP]);
	write(fd, aux_line, strlen(aux_line));

	// UDP
	if (mode == MODE_DOUBLE)
		sprintf(aux_line, "UDP bandwidth (Bytes): %.0f\n", bytes_and_packs->protocol_bytes_d[HASH_UDP]);
	else if (mode == MODE_LONGLONG)
		sprintf(aux_line, "UDP bandwidth (Bytes): %llu\n", bytes_and_packs->protocol_bytes_ll[HASH_UDP]);
	write(fd, aux_line, strlen(aux_line));

	// ICMP
	if (mode == MODE_DOUBLE)
		sprintf(aux_line, "ICMP bandwidth (Bytes): %.0f\n", bytes_and_packs->protocol_bytes_d[HASH_ICMP]);
	else if (mode == MODE_LONGLONG)
		sprintf(aux_line, "ICMP bandwidth (Bytes): %llu\n", bytes_and_packs->protocol_bytes_ll[HASH_ICMP]);
	write(fd, aux_line, strlen(aux_line));


	// OTHERS
	if (mode == MODE_DOUBLE)
		sprintf(aux_line, "OTHERS bandwidth (Bytes): %.0f\n", bytes_and_packs->protocol_bytes_d[HASH_OTHERS]);
	else if (mode == MODE_LONGLONG)
		sprintf(aux_line, "OTHERS bandwidth (Bytes): %llu\n", bytes_and_packs->protocol_bytes_ll[HASH_OTHERS]);
	write(fd, aux_line, strlen(aux_line));

	write(fd, "\n", strlen("\n"));

	// Y ahora escribimos los precalculados...


	// ************************** TOTAL BYTES RECEIVED ************************

	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->total_top_bytes_received, max_hash, mode);
	sprintf(aux_line, "Total top %d received (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);

	// Received bytes TCP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->total_top_bytes_received_protocol[HASH_TCP], max_hash, mode);
	sprintf(aux_line, "TCP total top %d received (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_tcp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_tcp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);

	// Received bytes UDP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->total_top_bytes_received_protocol[HASH_UDP], max_hash, mode);
	sprintf(aux_line, "UDP total top %d received (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_udp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_udp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);

	// Received bytes ICMP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->total_top_bytes_received_protocol[HASH_ICMP], max_hash, mode);
	sprintf(aux_line, "ICMP total top %d received (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_icmp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_icmp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);

	// Received bytes OTHERS
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->total_top_bytes_received_protocol[HASH_OTHERS], max_hash, mode);
	sprintf(aux_line, "OTHERS total top %d received (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_others_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_others_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// ************************** LOCAL BYTES RECEIVED ************************

	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->local_top_bytes_received, max_hash, mode);
	sprintf(aux_line, "Local top %d received (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// Received local bytes TCP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->local_top_bytes_received_protocol[HASH_TCP], max_hash, mode);
	sprintf(aux_line, "TCP local top %d received (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_tcp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_tcp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// Received local bytes UDP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->local_top_bytes_received_protocol[HASH_UDP], max_hash, mode);
	sprintf(aux_line, "UDP local top %d received (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_udp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_udp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// Received local bytes ICMP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->local_top_bytes_received_protocol[HASH_ICMP], max_hash, mode);
	sprintf(aux_line, "ICMP local top %d received (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_icmp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_icmp_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// Received local bytes OTHERS
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->local_top_bytes_received_protocol[HASH_OTHERS], max_hash, mode);
	sprintf(aux_line, "OTHERS local top %d received (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_others_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_others_bytes_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// ************************** TOTAL BYTES SENT ************************
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->total_top_bytes_sent, max_hash, mode);
	sprintf(aux_line, "Total top %d sent (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);

	// Sent bytes TCP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->total_top_bytes_sent_protocol[HASH_TCP], max_hash, mode);
	sprintf(aux_line, "TCP total top %d sent (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_tcp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_tcp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);

	// Sent bytes UDP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->total_top_bytes_sent_protocol[HASH_UDP], max_hash, mode);
	sprintf(aux_line, "UDP total top %d sent (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_udp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_udp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);

	// Sent bytes ICMP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->total_top_bytes_sent_protocol[HASH_ICMP], max_hash, mode);
	sprintf(aux_line, "ICMP total top %d sent (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_icmp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_icmp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);

	// Sent bytes OTHERS
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->total_top_bytes_sent_protocol[HASH_OTHERS], max_hash, mode);
	sprintf(aux_line, "OTHERS total top %d sent (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_others_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_others_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// ************************** LOCAL BYTES SENT ************************
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->local_top_bytes_sent, max_hash, mode);
	sprintf(aux_line, "Local top %d sent (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// Sent local bytes TCP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->local_top_bytes_sent_protocol[HASH_TCP], max_hash, mode);
	sprintf(aux_line, "TCP local top %d sent (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_tcp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_tcp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// Sent local bytes UDP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->local_top_bytes_sent_protocol[HASH_UDP], max_hash, mode);
	sprintf(aux_line, "UDP local top %d sent (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_udp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_udp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// Sent local bytes ICMP
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->local_top_bytes_sent_protocol[HASH_ICMP], max_hash, mode);
	sprintf(aux_line, "ICMP local top %d sent (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_icmp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_icmp_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// Sent local bytes OTHERS
	actual_top = 0;
	reorder_hash_bytes (bytes_and_packs->interface, bytes_and_packs->local_top_bytes_sent_protocol[HASH_OTHERS], max_hash, mode);
	sprintf(aux_line, "OTHERS local top %d sent (bytes)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_others_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_BYTES_VIS(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_others_bytes_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_BYTES(aux_element_bytes, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_bytes (max_hash);


	// ************************** TOTAL CONNECTIONS RECEIVED ************************
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->total_top_connections_received, max_hash, mode);
	sprintf(aux_line, "Total top %d received (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Received connections TCP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->total_top_connections_received_protocol[HASH_TCP], max_hash, mode);
	sprintf(aux_line, "TCP total top %d received (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_tcp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_tcp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Received connections UDP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->total_top_connections_received_protocol[HASH_UDP], max_hash, mode);
	sprintf(aux_line, "UDP total top %d received (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_udp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_udp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Received connections ICMP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->total_top_connections_received_protocol[HASH_ICMP], max_hash, mode);
	sprintf(aux_line, "ICMP total top %d received (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_icmp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_icmp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Received connections OTHERS
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->total_top_connections_received_protocol[HASH_OTHERS], max_hash, mode);
	sprintf(aux_line, "OTHERS total top %d received (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_others_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_others_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	// ************************** LOCAL CONNECTIONS RECEIVED ************************
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->local_top_connections_received, max_hash, mode);
	sprintf(aux_line, "Local top %d received (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Received connections TCP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->local_top_connections_received_protocol[HASH_TCP], max_hash, mode);
	sprintf(aux_line, "TCP local top %d received (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_tcp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_tcp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Received connections UDP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->local_top_connections_received_protocol[HASH_UDP], max_hash, mode);
	sprintf(aux_line, "UDP local top %d received (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_udp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_udp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Received connections ICMP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->local_top_connections_received_protocol[HASH_ICMP], max_hash, mode);
	sprintf(aux_line, "ICMP local top %d received (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_icmp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_icmp_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Received connections OTHERS
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->local_top_connections_received_protocol[HASH_OTHERS], max_hash, mode);
	sprintf(aux_line, "OTHERS local top %d received (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_others_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_others_connections_received.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	// ************************** TOTAL CONNECTIONS SENT ************************
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->total_top_connections_sent, max_hash, mode);
	sprintf(aux_line, "Total top %d sent (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Sent connections TCP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->total_top_connections_sent_protocol[HASH_TCP], max_hash, mode);
	sprintf(aux_line, "TCP total top %d sent (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_tcp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_tcp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Sent connections UDP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->total_top_connections_sent_protocol[HASH_UDP], max_hash, mode);
	sprintf(aux_line, "UDP total top %d sent (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_udp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_udp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Sent connections ICMP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->total_top_connections_sent_protocol[HASH_ICMP], max_hash, mode);
	sprintf(aux_line, "ICMP total top %d sent (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_icmp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_icmp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Sent connections OTHERS
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->total_top_connections_sent_protocol[HASH_OTHERS], max_hash, mode);
	sprintf(aux_line, "OTHERS total top %d sent (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_others_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_others_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	// ************************** LOCAL CONNECTIONS SENT ************************
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->local_top_connections_sent, max_hash, mode);
	sprintf(aux_line, "Local top %d sent (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Sent connections TCP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->local_top_connections_sent_protocol[HASH_TCP], max_hash, mode);
	sprintf(aux_line, "TCP local top %d sent (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_tcp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_tcp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Sent connections UDP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->local_top_connections_sent_protocol[HASH_UDP], max_hash, mode);
	sprintf(aux_line, "UDP local top %d sent (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_udp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_udp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Sent connections ICMP
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->local_top_connections_sent_protocol[HASH_ICMP], max_hash, mode);
	sprintf(aux_line, "ICMP local top %d sent (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_icmp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_icmp_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	//  Sent connections OTHERS
	actual_top = 0;
	reorder_hash_accessed (bytes_and_packs->interface, bytes_and_packs->local_top_connections_sent_protocol[HASH_OTHERS], max_hash, mode);
	sprintf(aux_line, "OTHERS local top %d sent (connections)\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_others_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_ACCESSED_VIS(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_others_connections_sent.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_ACCESSED(aux_element_accessed, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_accessed (max_hash);


	// ************************** TOTAL PROTOCOLS ACCESSED ************************

	// TCP ports
	actual_top = 0;
	reorder_hash_ports (bytes_and_packs->interface, bytes_and_packs->total_top_ports_accessed[HASH_TCP], max_hash, mode);
	sprintf(aux_line, "Total top %d TCP ports more accessed\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_tcp_ports.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_PORTS_VIS(aux_element_ports, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_tcp_ports.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_PORTS(aux_element_ports, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_ports (max_hash);

	// UDP ports
	actual_top = 0;
	reorder_hash_ports (bytes_and_packs->interface, bytes_and_packs->total_top_ports_accessed[HASH_UDP], max_hash, mode);
	sprintf(aux_line, "Total top %d UDP ports more accessed\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.total_top_udp_ports.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_PORTS_VIS(aux_element_ports, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.total_top_udp_ports.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_PORTS(aux_element_ports, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_ports (max_hash);


	// ************************** LOCAL PROTOCOLS ACCESSED ************************
	
	// TCP local ports
	actual_top = 0;
	reorder_hash_ports (bytes_and_packs->interface, bytes_and_packs->local_top_ports_accessed[HASH_TCP], max_hash, mode);
	sprintf(aux_line, "Local top %d TCP ports more accessed\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_tcp_ports.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_PORTS_VIS(aux_element_ports, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_tcp_ports.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_PORTS(aux_element_ports, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_ports (max_hash);


	// UDP local ports
	actual_top = 0;
	reorder_hash_ports (bytes_and_packs->interface, bytes_and_packs->local_top_ports_accessed[HASH_UDP], max_hash, mode);
	sprintf(aux_line, "Local top %d UDP ports more accessed\n", top_n);
	write(fd, aux_line, strlen(aux_line));
#ifdef _VIEWER
	sprintf(aux_line2, "%s.%s.local_top_udp_ports.gnuplot", output_file, bytes_and_packs->interface);
 	PRECALCULATED_PORTS_VIS(aux_element_ports, max_hash, mode, top_n, actual_top, aux_line, fd, aux_line2, fd2);
	sprintf(aux_line2, "%s.%s.local_top_udp_ports.gnuplot", output_file, bytes_and_packs->interface);
 	if (actual_top == 0) unlink (aux_line2); 
#else
	PRECALCULATED_PORTS(aux_element_ports, max_hash, mode, top_n, actual_top, aux_line, fd);
#endif
	remove_list_ports (max_hash);


	// And close statistics file
	close(fd);


	char aux_file[MAX_LINE];
	// Rename .statistics.tmp to .processed
	sprintf(aux_file, "%s.%s.statistics", file_name, bytes_and_packs->interface);
	rename(statistics_file, aux_file);
	chown(aux_file, (uid_t) -1, file_gid);
	chmod(aux_file, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);

	return 0;
}

void delete_position_from_hash (bytes_and_packets *bytes_and_packs, unsigned int protocol, unsigned long hash_position, memory_element *package, int mode)
{
	if (package->next != NULL) package->next->previous = package->previous;

	if (package->previous != NULL) package->previous->next = package->next;
	else {
		if (mode == DOT_CAPTURED)
			bytes_and_packs->array_captured[protocol][hash_position] = package->next;
		else if (mode == DOT_PROCESSED)
			bytes_and_packs->array_processed[protocol][hash_position] = package->next;
	}
	// Hemos enlazado... ahora podriamos retirar el elemento		
	free_package(package, bytes_and_packs);
}

void save_element_to_disk (bytes_and_packets *bytes_and_packs, int fd, memory_element *actual, int mode)
{
	char source_address_str[40];
	char destination_address_str[40];
	char line[MAX_LINE];
	struct in_addr aux_direction;

	int protocol;
	unsigned long long bytes_received;
	unsigned long long bytes_received2;
	unsigned long long packages_received;
	unsigned long long packages_received2;
	long secs_begin, msecs_begin, secs_end, msecs_end;
	unsigned long hash_position_src_ip, hash_position_dst_ip, hash_position_src_port, hash_position_dst_port;

	aux_direction.s_addr = 1;

	strcpy(source_address_str, inet_ntoa(actual->processed.source_address));
	strcpy(destination_address_str, inet_ntoa(actual->processed.destination_address));

	secs_begin = actual->processed.start_time.tv_sec;
	msecs_begin = actual->processed.start_time.tv_usec;
	secs_end = actual->processed.end_time.tv_sec;
	msecs_end = actual->processed.end_time.tv_usec;

	if (actual->processed.protocol == IPPROTO_TCP) protocol = HASH_TCP;
	else if (actual->processed.protocol == IPPROTO_UDP) protocol = HASH_UDP;
	else if (actual->processed.protocol == IPPROTO_ICMP) protocol = HASH_ICMP;
	else protocol = HASH_OTHERS;

	bytes_received = GIGA*actual->processed.source_gb + actual->processed.source_b;
	bytes_received2 = GIGA*actual->processed.destination_gb + actual->processed.destination_b;
	packages_received = GIGA*actual->processed.source_packages_g + actual->processed.source_packages;
	packages_received2 = GIGA*actual->processed.destination_packages_g + actual->processed.destination_packages;

	// En el caso que no sea un paquete tcp, printeamos lo que nos interese...
	// De host1 a host2, por lo que se cuentan los bytes recibidos en el host2
	sprintf(line, "%lu.%lu %lu.%lu %d %s %s %u %u %llu %llu %llu %llu\n", \
				 // (1)     (2)    (3)(4)(5)(6)(7) (8)  (9)  (10) (11)
			(unsigned long) secs_begin, (unsigned long) msecs_begin, \
			(unsigned long) secs_end, (unsigned long) msecs_end, \
			(int)actual->processed.protocol, source_address_str, destination_address_str, \
			actual->processed.source_port, actual->processed.destination_port, \
			bytes_received, bytes_received2, packages_received, packages_received2);
	write(fd, line, strlen(line));

	// Actualizamos estadisticas...

	// Conexiones totales
	if (mode == MODE_DOUBLE) {
		bytes_and_packs->total_connections_d += 1;
	}
	else if (mode == MODE_LONGLONG) {
		bytes_and_packs->total_connections_ll += 1;
	}

	// Conexiones por protocolo
	if (mode == MODE_DOUBLE) {
		bytes_and_packs->protocol_connections_d[protocol] += 1;
	}
	else if (mode == MODE_LONGLONG) {
		bytes_and_packs->protocol_connections_ll[protocol] += 1;
	}

	// Bytes totales (tanto los de ida como los de vuelta)
	if (mode == MODE_DOUBLE) {
		bytes_and_packs->total_bytes_d += bytes_received;
		bytes_and_packs->total_bytes_d += bytes_received2;
	}
	else if (mode == MODE_LONGLONG) {
		bytes_and_packs->total_bytes_ll += bytes_received;
		bytes_and_packs->total_bytes_ll += bytes_received2;
	}

	// Bytes por protocolo (tanto los de ida como los de vuelta)
	if (mode == MODE_DOUBLE) {
		bytes_and_packs->protocol_bytes_d[protocol] += bytes_received;
		bytes_and_packs->protocol_bytes_d[protocol] += bytes_received2;
	}
	else if (mode == MODE_LONGLONG) {
		bytes_and_packs->protocol_bytes_ll[protocol] += bytes_received;
		bytes_and_packs->protocol_bytes_ll[protocol] += bytes_received2;
	}

	hash_position_src_ip = GET_HASH(actual->processed.source_address, 1, aux_direction, 1, max_hash);
	hash_position_dst_ip = GET_HASH(aux_direction, 1, actual->processed.destination_address, 1, max_hash);
	hash_position_src_port = GET_HASH(aux_direction, actual->processed.source_port, aux_direction, 1, max_hash);
	hash_position_dst_port = GET_HASH(aux_direction, 1, aux_direction, actual->processed.destination_port, max_hash);
	
	// Total bytes received
	add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_received[hash_position_src_ip], actual->processed.source_address, bytes_received, mode);
	add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_received[hash_position_dst_ip], actual->processed.destination_address, bytes_received2, mode);
	// Total bytes received per protocol
	add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_received_protocol[protocol][hash_position_src_ip], actual->processed.source_address, bytes_received, mode);
	add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_received_protocol[protocol][hash_position_dst_ip], actual->processed.destination_address, bytes_received2, mode);
	// Total bytes sent
	add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_sent[hash_position_src_ip], actual->processed.source_address, bytes_received2, mode);
	add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_sent[hash_position_dst_ip], actual->processed.destination_address, bytes_received, mode);
	// Total bytes sent per protocol
	add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_sent_protocol[protocol][hash_position_src_ip], actual->processed.source_address, bytes_received2, mode);
	add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_sent_protocol[protocol][hash_position_dst_ip], actual->processed.destination_address, bytes_received, mode);

	// Total connections received
	add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_received[hash_position_dst_ip], actual->processed.destination_address, 1, mode);
	// Total connections received per protocol
	add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_received_protocol[protocol][hash_position_dst_ip], actual->processed.destination_address, 1, mode);
	// Total connections sent
	add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_sent[hash_position_src_ip], actual->processed.source_address, 1, mode);
	// Total connections sent per protocol
	add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_sent_protocol[protocol][hash_position_src_ip], actual->processed.source_address, 1, mode);

	// Total ports used
	add_address_ports_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_ports_accessed[protocol][hash_position_src_port], actual->processed.source_port, 1, mode);
	add_address_ports_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_ports_accessed[protocol][hash_position_dst_port], actual->processed.destination_port, 1, mode);

	// LOCAL network
	if ((is_local(source_address_str)) && (is_local(destination_address_str))) {
		// Ambas direcciones son locales... incrementamos contadores

		// Local bytes received
		add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_received[hash_position_src_ip], actual->processed.source_address, bytes_received, mode);
		add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_received[hash_position_dst_ip], actual->processed.destination_address, bytes_received2, mode);
		// Local bytes received per protocol
		add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_received_protocol[protocol][hash_position_src_ip], actual->processed.source_address, bytes_received, mode);
		add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_received_protocol[protocol][hash_position_dst_ip], actual->processed.destination_address, bytes_received2, mode);
		// Local bytes sent
		add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_sent[hash_position_src_ip], actual->processed.source_address, bytes_received2, mode);
		add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_sent[hash_position_dst_ip], actual->processed.destination_address, bytes_received, mode);
		// Local bytes sent per protocol
		add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_sent_protocol[protocol][hash_position_src_ip], actual->processed.source_address, bytes_received2, mode);
		add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_sent_protocol[protocol][hash_position_dst_ip], actual->processed.destination_address, bytes_received, mode);

		// Local connections received
		add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_received[hash_position_dst_ip], actual->processed.destination_address, 1, mode);
		// Local connections received per protocol
		add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_received_protocol[protocol][hash_position_dst_ip], actual->processed.destination_address, 1, mode);
		// Local connections sent
		add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_sent[hash_position_src_ip], actual->processed.source_address, 1, mode);
		// Local connections sent per protocol
		add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_sent_protocol[protocol][hash_position_src_ip], actual->processed.source_address, 1, mode);

		// Local ports used
		add_address_ports_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_ports_accessed[protocol][hash_position_src_port], actual->processed.source_port, 1, mode);
		add_address_ports_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_ports_accessed[protocol][hash_position_dst_port], actual->processed.destination_port, 1, mode);
	}
}

unsigned long get_first_array_element (memory_element **array, int start_or_end)
{
	unsigned long actual_pos, first;
	long seconds_first;

	first = 0;
	seconds_first = 2147483647L;
	
	if (start_or_end == 0) {
		for (actual_pos = 0 ; actual_pos < max_hash ; actual_pos++) {
			if ((array[actual_pos] != NULL) && (array[actual_pos]->processed.start_time.tv_sec < seconds_first)) {
				seconds_first = array[actual_pos]->processed.start_time.tv_sec;
				first = actual_pos;
			}
		}
	}
	else {
		for (actual_pos = 0 ; actual_pos < max_hash ; actual_pos++) {
			if ((array[actual_pos] != NULL) && (array[actual_pos]->processed.end_time.tv_sec < seconds_first)) {
				seconds_first = array[actual_pos]->processed.start_time.tv_sec;
				first = actual_pos;
			}
		}
	}
	return first;
}

int save_data_aggregations (char *file_name, bytes_and_packets *bytes_and_packs, long initial_time, long final_time, int mode, int mode2)
{
	int fd, fd2;
	unsigned int k;
	unsigned long actual_hash;
	char line[MAX_LINE];
	char data_file[MAX_LINE], debug_file[MAX_LINE];
	memory_element *aux_mem_elem, *actual, *next;

	unsigned long long bytes_received;
	unsigned long long bytes_received2;
	unsigned long long packages_received;
	unsigned long long packages_received2;

	unsigned long long auxiliar;

	unsigned long difference;
	unsigned long alloc_mem;
	unsigned int intervals;
	long actual_begin;
	long actual_end;

	/*
	 Estructura del fichero de salida

	 (1) hora inicio
	 (2) hora fin
	 (3) protocolo
	 (4) host 1 ip
	 (5) host 2 ip
	 (6) host 1 port
	 (7) host 2 port
	 (8) host 1 bytes received
	 (9) host 2 bytes received
	 (10) host 1 packets received (no tendra sentido en flujos no tcp/udp)
	 (11) host 2 packets received (no tendra sentido en flujos no tcp/udp)
	*/

	actual = NULL;

	sprintf(data_file, "%s.%s.processed.tmp", file_name, bytes_and_packs->interface);
	fd = open (data_file, O_WRONLY|O_CREAT|O_TRUNC);
	sprintf(debug_file, "%s.%s.debug.tmp", file_name, bytes_and_packs->interface);
	fd2 = open (debug_file, O_WRONLY|O_CREAT|O_TRUNC);

	// La idea es...
	// 1.- Recorremos los elementos del vector de hash mirando el tiempo...
	// 2.- Si el tiempo esta dentro del intervalo que nos ocupa, lo insertamos en el listado "final_save_list"
	//     ORDENADO
	// 3.- Si hay que actualizarlo, lo insertamos en el listado que toca actualizando el que queda en memoria
	// 4.- Finalmente, volcamos a disco el listado "final_save_list"

	for (actual_hash = 0 ; actual_hash < max_hash ; actual_hash++)
	{
		for ( k = 0 ; k < MAX_PROTOCOL ; k++ )
		{
			if (mode == DOT_CAPTURED) {
				actual = bytes_and_packs->array_captured[k][actual_hash];
			}
			else if (mode == DOT_PROCESSED) {
				actual = bytes_and_packs->array_processed[k][actual_hash];
			}
			// Tenemos que recorrer todas y cada una de las posiciones para ver si hay algun elemento que cumpla la
			// condicion...
			// OJO: En el caso de mode == DOT_PROCESSED, los retiramos todos!!
			while (actual != NULL) {

				next = actual->next;

				if (actual->processed.start_time.tv_sec != 0) actual_begin = actual->processed.start_time.tv_sec;
				else actual_begin = actual->processed.real_begin;
				if (actual->processed.end_time.tv_sec != 0) actual_end = actual->processed.end_time.tv_sec;
				else actual_end = actual->processed.real_end;

				if ((actual_begin < initial_time) || ((initial_time <= actual_begin) && (actual_begin < final_time)) || (mode == DOT_PROCESSED)) {
					// Solo procesaremos los paquetes que esten dentro del intervalo o anteriores a el...
					// O todos en el caso que estemos en mode == DOT_PROCESSED

					if ((mode == DOT_CAPTURED) && (actual->processed.end_time.tv_sec == 0) && ((actual_end + unfinished_connections_seconds) < initial_time))	{
						// El paquete ha expirado... lo eliminamos del hash y NO lo procesamos
						delete_position_from_hash (bytes_and_packs, k, actual_hash, actual, mode);
					}
					else {
						bytes_received = GIGA*actual->processed.source_gb + actual->processed.source_b;
						bytes_received2 = GIGA*actual->processed.destination_gb + actual->processed.destination_b;
						packages_received = GIGA*actual->processed.source_packages_g + actual->processed.source_packages;
						packages_received2 = GIGA*actual->processed.destination_packages_g + actual->processed.destination_packages;

						if ((mode == DOT_CAPTURED) && (actual_end > final_time)) {
							// We must get a new memory_element container
							if (bytes_and_packs->free != NULL) {
								aux_mem_elem = bytes_and_packs->free;
								if (bytes_and_packs->free->next == NULL)
									bytes_and_packs->free = bytes_and_packs->last_free = NULL;
								else
									bytes_and_packs->free = bytes_and_packs->free->next;
								bytes_and_packs->cnt_free_mem--;
							}
							else {
								// We must allocate more memory
								alloc_mem = allocate_memory(bytes_and_packs, min_packets);
								if (alloc_mem == 0) {
									sprintf(buffer_debug, "Error.. unable to allocate more memory");
									save_debug (bytes_and_packs->interface, buffer_debug);
									return -1;
								}
								else {
									bytes_and_packs->memory_allocated += alloc_mem;
									aux_mem_elem = bytes_and_packs->free;
									if (bytes_and_packs->free->next == NULL)
										bytes_and_packs->free = bytes_and_packs->last_free = NULL;
									else
										bytes_and_packs->free = bytes_and_packs->free->next;
									bytes_and_packs->cnt_free_mem--;
								}
							}
							bytes_and_packs->memory_used++;

							// And fill it...
							// Copiamos la posicion en la memoria alocatada...
							memcpy(aux_mem_elem, actual, sizeof(memory_element));
							
							// Tenemos que actualizar la informacion del hash, pero SIN eliminar esa posicion...
							difference = final_time - initial_time;
							intervals = 1;

							while (actual_begin < actual_end) {
								intervals++;
								actual_begin += difference;
							}

							// Y ahora actualizamos los valores...
							auxiliar = bytes_received - (bytes_received / intervals);
							bytes_received = bytes_received - auxiliar;
							if (auxiliar > GIGA) {
								actual->processed.source_gb = auxiliar / GIGA;
								actual->processed.source_b  = auxiliar % GIGA;
							}
	
							auxiliar = bytes_received2 - (bytes_received2 / intervals);
							bytes_received2 = bytes_received2 - auxiliar;
							if (auxiliar > GIGA) {
								actual->processed.destination_gb = auxiliar / GIGA;
								actual->processed.destination_b  = auxiliar % GIGA;
							}

							auxiliar = packages_received - (packages_received / intervals);
							packages_received = packages_received - auxiliar;
							if (auxiliar > GIGA) {
								actual->processed.source_packages_g = auxiliar / GIGA;
								actual->processed.source_packages   = auxiliar % GIGA;
							}

							auxiliar = packages_received2 - (packages_received2 / intervals);
							packages_received2 = packages_received2 - auxiliar;
							if (auxiliar > GIGA) {
								actual->processed.destination_packages_g = auxiliar / GIGA;
								actual->processed.destination_packages   = auxiliar % GIGA;
							}

							if ((actual->processed.end_time.tv_sec >= final_time) || \
								(actual->processed.real_end >= final_time))
							{
								// Actualizamos start_time y real_begin
								actual->processed.start_time.tv_sec = 0;
								actual->processed.start_time.tv_usec = 0;
								actual->processed.real_begin = final_time;
							}

							// Una vez actualizada la informacion del hash, tenemos que modificar ciertos
							// campos del paquete que meteremos en el vector...
							aux_mem_elem->processed.source_gb = bytes_received / GIGA;
							aux_mem_elem->processed.source_b  = bytes_received % GIGA;
							aux_mem_elem->processed.destination_gb = bytes_received2 / GIGA;
							aux_mem_elem->processed.destination_b  = bytes_received2 % GIGA;
							aux_mem_elem->processed.source_packages_g = packages_received / GIGA;
							aux_mem_elem->processed.source_packages   = packages_received % GIGA;
							aux_mem_elem->processed.destination_packages_g = packages_received2 / GIGA;
							aux_mem_elem->processed.destination_packages   = packages_received2 % GIGA;
							aux_mem_elem->processed.end_time.tv_sec = 0;
							aux_mem_elem->processed.end_time.tv_usec = 0;
						}
						else {
							// En cualquier otro caso, lo sacaremos del hash desordenado y lo reintroduciremos en el hash ordenado
							if (actual->next != NULL) actual->next->previous = actual->previous;
							if (actual->previous != NULL) actual->previous->next = actual->next;
							else {
								if (mode == DOT_CAPTURED)
									bytes_and_packs->array_captured[k][actual_hash] = actual->next;
								else if (mode == DOT_PROCESSED)
									bytes_and_packs->array_processed[k][actual_hash] = actual->next;
							}
							aux_mem_elem = actual;
						}
						// Una vez rellenado el paquete, lo insertamos ordenadamente en memoria
						insert_to_ordered_hash (bytes_and_packs, aux_mem_elem);
					}
				}
				actual = next;
			}
		}
	}

	unsigned long local_hash;

	// Buscamos el elemento mas pequenyo
	local_hash = get_first_array_element (bytes_and_packs->aux_array_zero, 1);

	while (bytes_and_packs->aux_array_zero[local_hash] != NULL) {
		aux_mem_elem = bytes_and_packs->aux_array_zero[local_hash];
		save_element_to_disk (bytes_and_packs, fd, aux_mem_elem, mode2);
		// And delete it...
		bytes_and_packs->aux_array_zero[local_hash] = bytes_and_packs->aux_array_zero[local_hash]->next;
		if (bytes_and_packs->aux_array_zero[local_hash] != NULL)
			bytes_and_packs->aux_array_zero[local_hash]->previous = NULL;
		free_package(aux_mem_elem, bytes_and_packs);
		// And get next element...
		local_hash = get_first_array_element (bytes_and_packs->aux_array_zero, 1);
	}

	// Repetimos lo mismo para el array de no initial_time == 0
	local_hash = get_first_array_element (bytes_and_packs->aux_array, 0);

	while (bytes_and_packs->aux_array[local_hash] != NULL) {
		aux_mem_elem = bytes_and_packs->aux_array[local_hash];
		save_element_to_disk (bytes_and_packs, fd, aux_mem_elem, mode2);
		// And delete it...
		bytes_and_packs->aux_array[local_hash] = bytes_and_packs->aux_array[local_hash]->next;
		if (bytes_and_packs->aux_array[local_hash] != NULL)
			bytes_and_packs->aux_array[local_hash]->previous = NULL;
		free_package(aux_mem_elem, bytes_and_packs);
		// And get next element...
		local_hash = get_first_array_element (bytes_and_packs->aux_array, 0);
	}

	// And save .debug information (time)
	sprintf(line, "%li\n%li\n", initial_time, final_time);
	write (fd2, line, strlen(line));

	close(fd);
	close(fd2);

	char aux_file[MAX_LINE];
	// Rename .processed.tmp to .processed
	sprintf(aux_file, "%s.%s.processed", file_name, bytes_and_packs->interface);
	rename(data_file, aux_file);
	chown(aux_file, (uid_t) -1, file_gid);
	chmod(aux_file, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);

	// If the file is in gzip...
	if (!use_gzip_or_bzip) {
		// File extension will be .gz
		sprintf(data_file, "%s.gz", aux_file);
	}
	else if (use_gzip_or_bzip) {
		// File extension will be .bz2
		sprintf(data_file, "%s.bz2", aux_file);
	}
	unlink (data_file);

	// We must now compress the data file... (not the .debug)
	sprintf(line, "%s %s", compress_bin_path, aux_file);
	system(line);

	chown(data_file, (uid_t) -1, file_gid);
	chmod(data_file, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);

	sprintf(buffer_debug,"%s saved to disk", data_file);
	save_debug(bytes_and_packs->interface, buffer_debug);

	// Rename .debug.tmp to .debug
	sprintf(aux_file, "%s.%s.debug", file_name, bytes_and_packs->interface);
	rename(debug_file, aux_file);
	chown(aux_file, (uid_t) -1, file_gid);
	chmod(aux_file, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);

	// Purgamos la memoria sobrante...
	purge_sparse_memory(bytes_and_packs, (bytes_and_packs->memory_allocated - max_packets));

	return 0;
}

int process_last_interval (flow_adder_struct *flow_elem, int position, int mode, int compressed)
{
	int fd, result;
	char rline[MAX_LINE];
	memory_element *mem_package = NULL;
	char space_char[2]=" ";
	char dot_char[2]=".";
	char *ptr_line;
	char str_ini_time[MAX_LINE], str_final_time[MAX_LINE], str_protocol[MAX_LINE], str_src_ip[MAX_LINE], str_dst_ip[MAX_LINE];
	char str_src_port[MAX_LINE], str_dst_port[MAX_LINE], str_src_bytes[MAX_LINE], str_dst_bytes[MAX_LINE];
	char str_src_packs[MAX_LINE], str_dst_packs[MAX_LINE];
	char aux_line[MAX_LINE];
	struct timeval initial_time, final_time;
	unsigned long long very_long;
	unsigned long alloc_mem;
	flow_adder_struct *aux_flo;
	char tmp_compressed_file[MAX_LINE];

	aux_flo = flow_elem;
	while (aux_flo != NULL) {
		sprintf(buffer_debug, "Processing file %s", aux_flo->filename);
		save_debug (iface_information[position].interface, buffer_debug);

		if (compressed) {
			// We must uncompress the data file first in a temporary file..
			sprintf(tmp_compressed_file, "/tmp/.flow_adder.%s.%d", iface_information[position].interface, (int) getpid());
			if ((fd = open (tmp_compressed_file, O_RDONLY)) >= 0) {
				// The file exists...
				sprintf(buffer_debug, "Temporary file exists... deleting it");
				save_debug(iface_information[position].interface, buffer_debug);
				close (fd);
				unlink (tmp_compressed_file);
			}
			sprintf(rline, "%s %s > %s", uncompress_bin_path, aux_flo->filename, tmp_compressed_file);
			system(rline);
		}

		// Primero de todo, tenemos que recorrer todos los ficheros a procesar...
		if (compressed)
			fd = open(tmp_compressed_file, O_RDONLY);
		else
			fd = open(aux_flo->filename, O_RDONLY);

		if (fd < 0) {
			if (compressed)
				sprintf(buffer_debug, "Error... filename %s not found", tmp_compressed_file);
			else
				sprintf(buffer_debug, "Error... filename %s not found", aux_flo->filename);
			save_debug (iface_information[position].interface, buffer_debug);
			return -1;
		}
		else {
			result = read_line(fd, rline);

			while ((result >= 0) || (strlen(rline) > 0)) {

				// Now, we must fill the package with the line read...
				ptr_line = strtok(rline, space_char);

				// First, corresponds to initial_time
				strcpy(str_ini_time, ptr_line);
				ptr_line = strtok(NULL, space_char);

				// Second, to final_time
				strcpy(str_final_time, ptr_line);
				ptr_line = strtok(NULL, space_char);

				// 3d, to protocol
				strcpy(str_protocol, ptr_line);
				ptr_line = strtok(NULL, space_char);

				// 4, source IP
				strcpy(str_src_ip, ptr_line);
				ptr_line = strtok(NULL, space_char);

				// 5, destination IP
				strcpy(str_dst_ip, ptr_line);
				ptr_line = strtok(NULL, space_char);

				// 6, source port
				strcpy(str_src_port, ptr_line);
				ptr_line = strtok(NULL, space_char);

				// 7, destination port
				strcpy(str_dst_port, ptr_line);
				ptr_line = strtok(NULL, space_char);

				// 8, source bytes
				strcpy(str_src_bytes, ptr_line);
				ptr_line = strtok(NULL, space_char);

				// 9, destination bytes
				strcpy(str_dst_bytes, ptr_line);
				ptr_line = strtok(NULL, space_char);

				// 10, source packages
				strcpy(str_src_packs, ptr_line);
				ptr_line = strtok(NULL, space_char);

				// 11, destination packages
				strcpy(str_dst_packs, ptr_line);

				ptr_line = strtok(str_ini_time, dot_char);
				strcpy (aux_line, ptr_line);
				remove_blanks(aux_line);
				initial_time.tv_sec = atol(aux_line);
				ptr_line = strtok(NULL, dot_char);
				strcpy (aux_line, ptr_line);
				remove_blanks(aux_line);
				initial_time.tv_usec = atol(aux_line);

				ptr_line = strtok(str_final_time, dot_char);
				strcpy (aux_line, ptr_line);
				remove_blanks(aux_line);
				final_time.tv_sec = atol(aux_line);
				ptr_line = strtok(NULL, dot_char);
				strcpy (aux_line, ptr_line);
				remove_blanks(aux_line);
				final_time.tv_usec = atol(aux_line);

				// We must get a new memory_element container
				if (iface_information[position].free != NULL) {
					mem_package = iface_information[position].free;
					if (iface_information[position].free->next == NULL)
						iface_information[position].free = iface_information[position].last_free = NULL;
					else
						iface_information[position].free = iface_information[position].free->next;
					iface_information[position].cnt_free_mem--;
				}
				else {
					// We must allocate more memory
					alloc_mem = allocate_memory(&iface_information[position], min_packets);
					if (alloc_mem == 0) {
						sprintf(buffer_debug, "Error.. unable to allocate more memory");
						save_debug (iface_information[position].interface, buffer_debug);
						return -1;
					}
					else {
						iface_information[position].memory_allocated += alloc_mem;
						mem_package = iface_information[position].free;
						if (iface_information[position].free->next == NULL)
							iface_information[position].free = iface_information[position].last_free = NULL;
						else
							iface_information[position].free = iface_information[position].free->next;
						iface_information[position].cnt_free_mem--;
					}
				}
				iface_information[position].memory_used++;

				// And fill it...
				memcpy(&mem_package->processed.start_time, &initial_time, sizeof(struct timeval));
				if (mem_package->processed.start_time.tv_sec == 0) mem_package->processed.real_begin = aux_flo->begin;
				else mem_package->processed.real_begin = mem_package->processed.start_time.tv_sec;

				memcpy(&mem_package->processed.end_time, &final_time, sizeof(struct timeval));
				if (mem_package->processed.end_time.tv_sec == 0) mem_package->processed.real_end = aux_flo->end;
				else mem_package->processed.real_end = mem_package->processed.end_time.tv_sec;

				mem_package->processed.protocol = (u_char) atoi(str_protocol);

				if (inet_aton(str_src_ip, &mem_package->processed.source_address) != 1) {
					sprintf(buffer_debug, "Error interpreting source address");
					save_debug (iface_information[position].interface, buffer_debug);
				}

				if (inet_aton(str_dst_ip, &mem_package->processed.destination_address) != 1) {
					sprintf(buffer_debug, "Error interpreting destination address");
					save_debug (iface_information[position].interface, buffer_debug);
				}

				mem_package->processed.source_port = (unsigned int) atoi(str_src_port);
				mem_package->processed.destination_port = (unsigned int) atoi(str_dst_port);

				very_long = (unsigned long long) atoll(str_src_bytes);
				if (very_long > GIGA) {
					mem_package->processed.source_gb = very_long / GIGA;
					mem_package->processed.source_b = very_long % GIGA;
				}
				else {
					mem_package->processed.source_gb = 0;
					mem_package->processed.source_b = (unsigned long) very_long;
				}

				very_long = (unsigned long long) atoll(str_dst_bytes);
				if (very_long > GIGA) {
					mem_package->processed.destination_gb = very_long / GIGA;
					mem_package->processed.destination_b = very_long % GIGA;
				}
				else {
					mem_package->processed.destination_gb = 0;
					mem_package->processed.destination_b = (unsigned long) very_long;
				}

				very_long = (unsigned long long) atoll(str_src_packs);
				if (very_long > GIGA) {
					mem_package->processed.source_packages_g = very_long / GIGA;
					mem_package->processed.source_packages = very_long % GIGA;
				}
				else {
					mem_package->processed.source_packages_g = 0;
					mem_package->processed.source_packages = (unsigned long) very_long;
				}

				very_long = (unsigned long long) atoll(str_dst_packs);
				if (very_long > GIGA) {
					mem_package->processed.destination_packages_g = very_long / GIGA;
					mem_package->processed.destination_packages = very_long % GIGA;
				}
				else {
					mem_package->processed.destination_packages_g = 0;
					mem_package->processed.destination_packages = (unsigned long) very_long;
				}

				mem_package->next = NULL;

				// process_packet(mem_package, &iface_information[position], mode);
				insert_without_order (mem_package, &iface_information[position], mode);

				result = read_line(fd, rline);
			}
			aux_flo = aux_flo->next;
			close (fd);
			if (compressed)	unlink(tmp_compressed_file);
		}
	}

	return 0;
}

void get_min_max (char *filename, long *first, long *last)
{
	int fd;
	char file_line[MAX_LINE];
	unsigned long aux_time;

	fd = open (filename, O_RDONLY);

	read_line(fd, file_line);
	aux_time = strtol (file_line, NULL, 10);
	memcpy(first, &aux_time, sizeof(unsigned long));

	read_line(fd, file_line);
	aux_time = strtol (file_line, NULL, 10);
	memcpy(last, &aux_time, sizeof(unsigned long));

	close(fd);
}

int write_dot_files (char *processed_dir, flow_adder_struct *file_struct, int position)
{
	int fd;
	char filename_debug[MAX_LINE];
	char aux_line[MAX_LINE];
	flow_adder_struct *aux_flo;

	sprintf(filename_debug,"%s.%s.files.tmp", processed_dir, iface_information[position].interface);

	fd = open (filename_debug, O_WRONLY|O_CREAT|O_TRUNC);
	if (fd < 0) {
		sprintf(buffer_debug, "Cannot create %s", filename_debug);
		save_debug (iface_information[position].interface, buffer_debug);
		return -1;
	}
	aux_flo = file_struct;
	while ( aux_flo != NULL ) {
		sprintf(aux_line, "%s\n", aux_flo->filename);
		write (fd, aux_line, strlen(aux_line));
		aux_flo = aux_flo->next;
	}
	close(fd);
	chown(filename_debug, (uid_t) -1, file_gid);
	chmod(filename_debug, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);

	// Rename .files.tmp to .files
	sprintf(aux_line, "%s.%s.files", processed_dir, iface_information[position].interface);
	rename(filename_debug, aux_line);
	chown(aux_line, (uid_t) -1, file_gid);
	chmod(aux_line, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);

	return 0;
}

#ifdef _VIEWER
int read_and_process_statistics_file (char *filename, bytes_and_packets *bytes_and_packs, int mode, char *output_file, char *time_to_write)
#else
int read_and_process_statistics_file (char *filename, bytes_and_packets *bytes_and_packs, int mode)
#endif
{
	char aux_line[MAX_LINE], aux_line2[MAX_LINE], rline[MAX_LINE], *tmp_line;
	int fd, result, state;
	char double_point_char[2] = ":";
	char space_char[2] = " ";
	unsigned long long very_long;
	unsigned int simple_unsigned_int;
	struct in_addr ip_address;

#ifdef _VIEWER
	double total_connections = 0, tcp_connections = 0, udp_connections = 0, icmp_connections = 0, others_connections = 0;
	double total_bytes = 0, tcp_bytes = 0, udp_bytes = 0, icmp_bytes = 0, others_bytes = 0;
	double aux_float = 0;
#endif

	if ((fd = open(filename, O_RDONLY)) < 0) {
		sprintf(buffer_debug, "Error in getting statistics from %s", filename);
		save_debug(bytes_and_packs->interface, buffer_debug);
		return -1;
	}
	result = read_line(fd, rline);
	state = STATISTICS_FILE_BEGIN;

	while ((result >= 0) || (strlen(rline) > 0)) {
		if ((state == STATISTICS_FILE_BEGIN) && (strstr(rline, "Total connections") != NULL)) {
			// Leemos las conexiones Totales
#ifdef _VIEWER
			aux_float = bytes_and_packs->total_connections_d;
#endif
			READ_STATISTICS_DATA(tmp_line, rline, double_point_char, aux_line, bytes_and_packs->total_connections_d, bytes_and_packs->total_connections_ll, mode);
#ifdef _VIEWER
			// Volvemos a comprobar despues de incrementar...
			total_connections = bytes_and_packs->total_connections_d - aux_float;
#endif
			state = STATISTICS_FILE_CONNECTIONS;
		}
		else if ((state == STATISTICS_FILE_CONNECTIONS) && (strstr(rline, "TCP connections") != NULL)) {
			// Leemos las conexiones TCP
#ifdef _VIEWER
			aux_float = bytes_and_packs->protocol_connections_d[HASH_TCP];
#endif
			READ_STATISTICS_DATA(tmp_line, rline, double_point_char, aux_line, bytes_and_packs->protocol_connections_d[HASH_TCP], bytes_and_packs->protocol_connections_ll[HASH_TCP], mode);
#ifdef _VIEWER
			tcp_connections = bytes_and_packs->protocol_connections_d[HASH_TCP] - aux_float;
#endif
		}
		else if ((state == STATISTICS_FILE_CONNECTIONS) && (strstr(rline, "UDP connections") != NULL)) {
			// Leemos las conexiones UDP
#ifdef _VIEWER
			aux_float = bytes_and_packs->protocol_connections_d[HASH_UDP];
#endif
			READ_STATISTICS_DATA(tmp_line, rline, double_point_char, aux_line, bytes_and_packs->protocol_connections_d[HASH_UDP], bytes_and_packs->protocol_connections_ll[HASH_UDP], mode);
#ifdef _VIEWER
			udp_connections = bytes_and_packs->protocol_connections_d[HASH_UDP] - aux_float;
#endif
		}
		else if ((state == STATISTICS_FILE_CONNECTIONS) && (strstr(rline, "ICMP connections") != NULL)) {
			// Leemos las conexiones ICMP
#ifdef _VIEWER
			aux_float = bytes_and_packs->protocol_connections_d[HASH_ICMP];
#endif
			READ_STATISTICS_DATA(tmp_line, rline, double_point_char, aux_line, bytes_and_packs->protocol_connections_d[HASH_ICMP], bytes_and_packs->protocol_connections_ll[HASH_ICMP], mode);
#ifdef _VIEWER
			icmp_connections = bytes_and_packs->protocol_connections_d[HASH_ICMP] - aux_float;
#endif
		}
		else if ((state == STATISTICS_FILE_CONNECTIONS) && (strstr(rline, "OTHERS connections") != NULL)) {
			// Leemos las conexiones OTHERS
#ifdef _VIEWER
			aux_float = bytes_and_packs->protocol_connections_d[HASH_OTHERS];
#endif
			READ_STATISTICS_DATA(tmp_line, rline, double_point_char, aux_line, bytes_and_packs->protocol_connections_d[HASH_OTHERS], bytes_and_packs->protocol_connections_ll[HASH_OTHERS], mode);
#ifdef _VIEWER
			others_connections = bytes_and_packs->protocol_connections_d[HASH_OTHERS] - aux_float;
#endif
		}
		else if ((state == STATISTICS_FILE_CONNECTIONS) && (strstr(rline, "Total bandwidth (Bytes)") != NULL)) {
			// Leemos los bytes totales
#ifdef _VIEWER
			aux_float = bytes_and_packs->total_bytes_d;
#endif
			READ_STATISTICS_DATA(tmp_line, rline, double_point_char, aux_line, bytes_and_packs->total_bytes_d, bytes_and_packs->total_bytes_ll, mode);
#ifdef _VIEWER
			total_bytes = bytes_and_packs->total_bytes_d - aux_float;
#endif
			state = STATISTICS_FILE_BANDWIDTH;
		}
		else if ((state == STATISTICS_FILE_BANDWIDTH) && (strstr(rline, "TCP bandwidth (Bytes)") != NULL)) {
			// Leemos los bytes TCP
#ifdef _VIEWER
			aux_float = bytes_and_packs->protocol_bytes_d[HASH_TCP];
#endif
			READ_STATISTICS_DATA(tmp_line, rline, double_point_char, aux_line, bytes_and_packs->protocol_bytes_d[HASH_TCP], bytes_and_packs->protocol_bytes_ll[HASH_TCP], mode);
#ifdef _VIEWER
			tcp_bytes = bytes_and_packs->protocol_bytes_d[HASH_TCP] - aux_float;
#endif
		}
		else if ((state == STATISTICS_FILE_BANDWIDTH) && (strstr(rline, "UDP bandwidth (Bytes)") != NULL)) {
			// Leemos los bytes UDP
#ifdef _VIEWER
			aux_float = bytes_and_packs->protocol_bytes_d[HASH_UDP];
#endif
			READ_STATISTICS_DATA(tmp_line, rline, double_point_char, aux_line, bytes_and_packs->protocol_bytes_d[HASH_UDP], bytes_and_packs->protocol_bytes_ll[HASH_UDP], mode);
#ifdef _VIEWER
			udp_bytes = bytes_and_packs->protocol_bytes_d[HASH_UDP] - aux_float;
#endif
		}
		else if ((state == STATISTICS_FILE_BANDWIDTH) && (strstr(rline, "ICMP bandwidth (Bytes)") != NULL)) {
			// Leemos los bytes ICMP
#ifdef _VIEWER
			aux_float = bytes_and_packs->protocol_bytes_d[HASH_ICMP];
#endif
			READ_STATISTICS_DATA(tmp_line, rline, double_point_char, aux_line, bytes_and_packs->protocol_bytes_d[HASH_ICMP], bytes_and_packs->protocol_bytes_ll[HASH_ICMP], mode);
#ifdef _VIEWER
			icmp_bytes = bytes_and_packs->protocol_bytes_d[HASH_ICMP] - aux_float;
#endif
		}
		else if ((state == STATISTICS_FILE_BANDWIDTH) && (strstr(rline, "OTHERS bandwidth (Bytes)") != NULL)) {
			// Leemos los bytes OTHERS
#ifdef _VIEWER
			aux_float = bytes_and_packs->protocol_bytes_d[HASH_OTHERS];
#endif
			READ_STATISTICS_DATA(tmp_line, rline, double_point_char, aux_line, bytes_and_packs->protocol_bytes_d[HASH_OTHERS], bytes_and_packs->protocol_bytes_ll[HASH_OTHERS], mode);
#ifdef _VIEWER
			others_bytes = bytes_and_packs->protocol_bytes_d[HASH_OTHERS] - aux_float;
#endif
		}
		else if ((state == STATISTICS_FILE_BANDWIDTH) && (strstr(rline, "Total top") != NULL) && (strstr(rline, "received (bytes)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_BYTES_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_received[0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_RECEIVED) && (strstr(rline, "TCP total top") != NULL) && (strstr(rline, "received (bytes)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_BYTES_TCP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_TCP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_received_protocol[HASH_TCP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_TCP_RECEIVED) && (strstr(rline, "UDP total top") != NULL) && (strstr(rline, "received (bytes)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_BYTES_UDP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_UDP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_received_protocol[HASH_UDP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_UDP_RECEIVED) && (strstr(rline, "ICMP total top") != NULL) && (strstr(rline, "received (bytes)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_BYTES_ICMP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_ICMP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_received_protocol[HASH_ICMP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_ICMP_RECEIVED) && (strstr(rline, "OTHERS total top") != NULL) && (strstr(rline, "received (bytes)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_BYTES_OTHERS_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_OTHERS_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_received_protocol[HASH_OTHERS][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_OTHERS_RECEIVED) && (strstr(rline, "Local top") != NULL) && (strstr(rline, "received (bytes)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_BYTES_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_received[0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_RECEIVED) && (strstr(rline, "TCP local top") != NULL) && (strstr(rline, "received (bytes)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_BYTES_TCP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_TCP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_received_protocol[HASH_TCP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_TCP_RECEIVED) && (strstr(rline, "UDP local top") != NULL) && (strstr(rline, "received (bytes)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_BYTES_UDP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_UDP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_received_protocol[HASH_UDP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_UDP_RECEIVED) && (strstr(rline, "ICMP local top") != NULL) && (strstr(rline, "received (bytes)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_BYTES_ICMP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_ICMP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_received_protocol[HASH_ICMP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_ICMP_RECEIVED) && (strstr(rline, "OTHERS local top") != NULL) && (strstr(rline, "received (bytes)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_BYTES_OTHERS_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_OTHERS_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_received_protocol[HASH_OTHERS][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_OTHERS_RECEIVED) && (strstr(rline, "Total top") != NULL) && (strstr(rline, "sent (bytes)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_BYTES_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_sent[0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_SENT) && (strstr(rline, "TCP total top") != NULL) && (strstr(rline, "sent (bytes)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_BYTES_TCP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_TCP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_sent_protocol[HASH_TCP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_TCP_SENT) && (strstr(rline, "UDP total top") != NULL) && (strstr(rline, "sent (bytes)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_BYTES_UDP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_UDP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_sent_protocol[HASH_UDP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_UDP_SENT) && (strstr(rline, "ICMP total top") != NULL) && (strstr(rline, "sent (bytes)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_BYTES_ICMP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_ICMP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_sent_protocol[HASH_ICMP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_ICMP_SENT) && (strstr(rline, "OTHERS total top") != NULL) && (strstr(rline, "sent (bytes)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_BYTES_OTHERS_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_OTHERS_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_bytes_sent_protocol[HASH_OTHERS][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_BYTES_OTHERS_SENT) && (strstr(rline, "Local top") != NULL) && (strstr(rline, "sent (bytes)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_BYTES_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_sent[0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_SENT) && (strstr(rline, "TCP local top") != NULL) && (strstr(rline, "sent (bytes)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_BYTES_TCP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_TCP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_sent_protocol[HASH_TCP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_TCP_SENT) && (strstr(rline, "UDP local top") != NULL) && (strstr(rline, "sent (bytes)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_BYTES_UDP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_UDP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_sent_protocol[HASH_UDP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_UDP_SENT) && (strstr(rline, "ICMP local top") != NULL) && (strstr(rline, "sent (bytes)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_BYTES_ICMP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_ICMP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_sent_protocol[HASH_ICMP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_ICMP_SENT) && (strstr(rline, "OTHERS local top") != NULL) && (strstr(rline, "sent (bytes)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_BYTES_OTHERS_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_OTHERS_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Bytes") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos los bytes... podemos proceder
			add_address_bytes_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_bytes_sent_protocol[HASH_OTHERS][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_BYTES_OTHERS_SENT) && (strstr(rline, "Total top") != NULL) && (strstr(rline, "received (connections)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_received[0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_RECEIVED) && (strstr(rline, "TCP total top") != NULL) && (strstr(rline, "received (connections)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_TCP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_TCP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_received_protocol[HASH_TCP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_TCP_RECEIVED) && (strstr(rline, "UDP total top") != NULL) && (strstr(rline, "received (connections)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_UDP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_UDP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_received_protocol[HASH_UDP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_UDP_RECEIVED) && (strstr(rline, "ICMP total top") != NULL) && (strstr(rline, "received (connections)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_ICMP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_ICMP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_received_protocol[HASH_ICMP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_ICMP_RECEIVED) && (strstr(rline, "OTHERS total top") != NULL) && (strstr(rline, "received (connections)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_OTHERS_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_OTHERS_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_received_protocol[HASH_OTHERS][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_OTHERS_RECEIVED) && (strstr(rline, "Local top") != NULL) && (strstr(rline, "received (connections)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_received[0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_RECEIVED) && (strstr(rline, "TCP local top") != NULL) && (strstr(rline, "received (connections)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_TCP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_TCP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_received_protocol[HASH_TCP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_TCP_RECEIVED) && (strstr(rline, "UDP local top") != NULL) && (strstr(rline, "received (connections)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_UDP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_UDP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_received_protocol[HASH_UDP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_UDP_RECEIVED) && (strstr(rline, "ICMP local top") != NULL) && (strstr(rline, "received (connections)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_ICMP_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_ICMP_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_received_protocol[HASH_ICMP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_ICMP_RECEIVED) && (strstr(rline, "OTHERS local top") != NULL) && (strstr(rline, "received (connections)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_OTHERS_RECEIVED;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_OTHERS_RECEIVED) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_received_protocol[HASH_OTHERS][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_OTHERS_RECEIVED) && (strstr(rline, "Total top") != NULL) && (strstr(rline, "sent (connections)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_sent[0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_SENT) && (strstr(rline, "TCP total top") != NULL) && (strstr(rline, "sent (connections)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_TCP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_TCP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_sent_protocol[HASH_TCP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_TCP_SENT) && (strstr(rline, "UDP total top") != NULL) && (strstr(rline, "sent (connections)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_UDP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_UDP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_sent_protocol[HASH_UDP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_UDP_SENT) && (strstr(rline, "ICMP total top") != NULL) && (strstr(rline, "sent (connections)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_ICMP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_ICMP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_sent_protocol[HASH_ICMP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_ICMP_SENT) && (strstr(rline, "OTHERS total top") != NULL) && (strstr(rline, "sent (connections)") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_OTHERS_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_OTHERS_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_connections_sent_protocol[HASH_OTHERS][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_CONNECTIONS_OTHERS_SENT) && (strstr(rline, "Local top") != NULL) && (strstr(rline, "sent (connections)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_sent[0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_SENT) && (strstr(rline, "TCP local top") != NULL) && (strstr(rline, "sent (connections)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_TCP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_TCP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_sent_protocol[HASH_TCP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_TCP_SENT) && (strstr(rline, "UDP local top") != NULL) && (strstr(rline, "sent (connections)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_UDP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_UDP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_sent_protocol[HASH_UDP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_UDP_SENT) && (strstr(rline, "ICMP local top") != NULL) && (strstr(rline, "sent (connections)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_ICMP_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_ICMP_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_sent_protocol[HASH_ICMP][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_ICMP_SENT) && (strstr(rline, "OTHERS local top") != NULL) && (strstr(rline, "sent (connections)") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_OTHERS_SENT;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_OTHERS_SENT) && (strstr(rline, "IP") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_IP_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, ip_address, very_long);
			// Tenemos la IP y tenemos las veces... podemos proceder
			add_address_accessed_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_connections_sent_protocol[HASH_OTHERS][0], ip_address, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_CONNECTIONS_OTHERS_SENT) && (strstr(rline, "Total top") != NULL) && (strstr(rline, "TCP ports more accessed") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_TCP_PORTS;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_TCP_PORTS) && (strstr(rline, "Port") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_PORTS_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, simple_unsigned_int, very_long);
			// Tenemos el puerto y tenemos los bytes... podemos proceder
			add_address_ports_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_ports_accessed[HASH_TCP][0], simple_unsigned_int, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_TCP_PORTS) && (strstr(rline, "Total top") != NULL) && (strstr(rline, "UDP ports more accessed") != NULL)) {
			state = STATISTICS_FILE_TOTAL_TOP_UDP_PORTS;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_TOTAL_TOP_UDP_PORTS) && (strstr(rline, "Port") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_PORTS_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, simple_unsigned_int, very_long);
			// Tenemos el puerto y tenemos los bytes... podemos proceder
			add_address_ports_statistics (bytes_and_packs->interface, &bytes_and_packs->total_top_ports_accessed[HASH_UDP][0], simple_unsigned_int, very_long, mode);
		}

		else if ((state == STATISTICS_FILE_TOTAL_TOP_UDP_PORTS) && (strstr(rline, "Local top") != NULL) && (strstr(rline, "TCP ports more accessed") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_TCP_PORTS;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_TCP_PORTS) && (strstr(rline, "Port") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_PORTS_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, simple_unsigned_int, very_long);
			// Tenemos el puerto y tenemos los bytes... podemos proceder
			add_address_ports_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_ports_accessed[HASH_TCP][0], simple_unsigned_int, very_long, mode);
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_TCP_PORTS) && (strstr(rline, "Local top") != NULL) && (strstr(rline, "UDP ports more accessed") != NULL)) {
			state = STATISTICS_FILE_LOCAL_TOP_UDP_PORTS;
			// No hacemos nada mas porque la linea no contiene informacion... esperamos a la siguiente
		}
		else if ((state == STATISTICS_FILE_LOCAL_TOP_UDP_PORTS) && (strstr(rline, "Port") != NULL) && (strstr(rline, "Times") != NULL)) {
			READ_STATISTICS_PORTS_DATA(tmp_line, rline, double_point_char, space_char, aux_line, aux_line2, simple_unsigned_int, very_long);
			// Tenemos el puerto y tenemos los bytes... podemos proceder
			add_address_ports_statistics (bytes_and_packs->interface, &bytes_and_packs->local_top_ports_accessed[HASH_UDP][0], simple_unsigned_int, very_long, mode);
		}

		result = read_line(fd, rline);
	}

#ifdef _VIEWER
	// Despues de todo, volcamos los ficheros gnuplot a disco...
	int fd2;

	sprintf(aux_line2, "%s.%s.total_connections.gnuplot", output_file, bytes_and_packs->interface);
	fd2 = open(aux_line2, O_WRONLY|O_APPEND|O_CREAT);
	if (fd2 >= 0) {
		sprintf(aux_line2, "%s\t%.0f\t%.0f\t%.0f\t%.0f\t%.0f\n", time_to_write, total_connections, tcp_connections, \
				udp_connections, icmp_connections, others_connections);
		write(fd2, aux_line2, strlen(aux_line2));
	}
	else {
		printf("Cannot open %s\n", aux_line2);
	}
	close(fd2);

	sprintf(aux_line2, "%s.%s.total_bytes.gnuplot", output_file, bytes_and_packs->interface);
	fd2 = open(aux_line2, O_WRONLY|O_APPEND|O_CREAT);
	if (fd2 >= 0) {
		sprintf(aux_line2, "%s\t%.0f\t%.0f\t%.0f\t%.0f\t%.0f\n", time_to_write, total_bytes, tcp_bytes, \
				udp_bytes, icmp_bytes, others_bytes);
		write(fd2, aux_line2, strlen(aux_line2));
	}
	else {
		printf("Cannot open %s\n", aux_line2);
	}
	close(fd2);

#endif

	close (fd);
	return 0;
}

int sum_statistics (char *process_file, flow_adder_struct *files_struct, int position, int mode, int mode2)
{
	// Por cada uno de los ficheros en "files_struct", tenemos que sumar su contenido... (seran .statistics)
	flow_adder_struct *aux_flo;
	char aux_line[MAX_LINE];
	int ret_value;

	// We must reset previous statistics before...
	aux_flo = files_struct;
	while (aux_flo != NULL) {
		// We must replace .processed.xx by .statistics
		strcpy(aux_line, aux_flo->filename);
		if (mode == TYPE_DAY) {
			if (!use_gzip_or_bzip) {
				// File extension will be .gz
				replace_string_with(aux_line, ".processed.gz", ".statistics");
			}
			else if (use_gzip_or_bzip) {
				// File extension will be .bz2
				replace_string_with(aux_line, ".processed.bz2", ".statistics");
			}
		}
		sprintf(aux_flo->filename, "%s/%s", process_file, aux_line);
		aux_flo = aux_flo->next;
	}

	aux_flo = files_struct;
	while (aux_flo != NULL) {
#ifdef _VIEWER
		read_and_process_statistics_file (aux_flo->filename, &iface_information[position], mode2, "", "");
#else
		read_and_process_statistics_file (aux_flo->filename, &iface_information[position], mode2);
#endif
		aux_flo = aux_flo->next;
	}
	// And, after saving to disk, reset statistics again
#ifdef _VIEWER
	ret_value = save_statistics (process_file, &iface_information[position], mode2, "");
#else
	ret_value = save_statistics (process_file, &iface_information[position], mode2);
#endif

	print_and_reset_some_counters (&iface_information[position], 0);

	return ret_value;
}

int save_process_substruct (char *process_file, flow_adder_struct *files_struct, int position, int time_type, long last_time, disk_path_struct *directory_time)
{
	char aux_line[MAX_LINE], rline[MAX_LINE];
	int fd, result;
	int line_counter, list_counter;
	flow_adder_struct *aux_flo, *aux_flo2;
	struct tm last_time_str;
	disk_path_struct local_time_str;

	time_t aux_time;

	if (files_struct == NULL) return -1;

	aux_time = (time_t) last_time;

	if (localtime_r (&aux_time, &last_time_str) == NULL)
	{
		sprintf(buffer_debug, "Error getting aproximative last time");
		save_debug (iface_information[position].interface, buffer_debug);
		return -1;
	}
	// And we must convert this struct to directory_time struct
	local_time_str.dir_year = 1900 + last_time_str.tm_year;
	local_time_str.dir_month = last_time_str.tm_mon + 1;

	int week, day_month, day_week, week_month;
	week = 1;
	day_month = last_time_str.tm_mday;
	day_week = last_time_str.tm_wday;
	if (day_week == 0) day_week = 7;
	week_month = day_month - (day_week - 1);
	while ( week_month > 1 ) {
		week++;
		week_month = week_month - 7;
	}
	local_time_str.dir_week = week;
	if (last_time_str.tm_wday == 0) local_time_str.dir_day = 7;
	else local_time_str.dir_day = last_time_str.tm_wday;

	if (time_type == TYPE_DAY) {
		// Vamos a procesar los ficheros del dia...
		// Casos:
		// 1.- El dia ha acabado. Tenemos que generar el dia.processed (con todos los datos del dia) y el dia.statistics...
		// 2.- El dia NO ha acabado. Simplemente generaremos el dia.statistics con los ficheros .statistics de cada 15 mins

		// Estamos en el mismo dia o lo hemos pasado...
		if ((local_time_str.dir_year > directory_time->dir_year) || \
			((local_time_str.dir_year == directory_time->dir_year) && (local_time_str.dir_month > directory_time->dir_month)) || \
			((local_time_str.dir_year == directory_time->dir_year) && (local_time_str.dir_month == directory_time->dir_month) && \
			 (local_time_str.dir_week > directory_time->dir_week)) || \
			((local_time_str.dir_year == directory_time->dir_year) && (local_time_str.dir_month == directory_time->dir_month) && \
			 (local_time_str.dir_week == directory_time->dir_week) && (local_time_str.dir_day > directory_time->dir_day))) {
			// Hemos pasado el dia
			// Tendriamos que:
			// 1.- Procesar de nuevo todos los ficheros diarios (solo en el caso que hubiera acabado el dia recientemente)
			//     Para ello, comparamos con el fichero de ficheros procesados...
			line_counter = list_counter = 0;
			aux_flo = files_struct;

			sprintf(aux_line, "%s.%s.files", process_file, iface_information[position].interface);
			fd = open(aux_line, O_RDONLY);
			if (fd >= 0) {
				result = read_line(fd, rline);
		
				while (((result >= 0) || (strlen(rline) > 0))) {
					if ((aux_flo != NULL) && (strcmp(rline, aux_flo->filename) == 0)) {
						list_counter++;
						aux_flo = aux_flo->next; 
					}
					line_counter++;
					result = read_line(fd, rline);
				}
				close(fd);
				fd = 1;
			}

			if ((fd < 0) || (line_counter != list_counter) || (aux_flo != NULL)) {
				// Escribimos la estructura en un fichero (para poder comparar despues)
				write_dot_files (process_file, files_struct, position);
				aux_flo = files_struct;
				while (aux_flo != NULL) {
					strcpy(aux_line, aux_flo->filename);
					sprintf(aux_flo->filename, "%s/%s", process_file, aux_line);
					aux_flo = aux_flo->next;
				}
				process_last_interval (files_struct, position, DOT_PROCESSED, 1);
				// Borramos el .files...
				sprintf(aux_line, "rm -f %s.%s.files", process_file, iface_information[position].interface);
				system (aux_line);
				// 2.- Salvar ese dia y generar el .statistics de ese dia
				save_data_aggregations (process_file, &iface_information[position], 0, 0, DOT_PROCESSED, MODE_LONGLONG);
#ifdef _VIEWER
				save_statistics (process_file, &iface_information[position], MODE_LONGLONG, "");
#else
				save_statistics (process_file, &iface_information[position], MODE_LONGLONG);
#endif
				print_and_reset_some_counters (&iface_information[position], 0);
				// 3.- Borrar la carpeta de ese dia y ciertos ficheros como dia.debug y dia.files
				sprintf(aux_line, "rm -rf %s/*.debug %s/*.processed* %s.%s.debug", process_file, process_file, process_file, iface_information[position].interface);
				system(aux_line);
			}
		}
		else if ((local_time_str.dir_year == directory_time->dir_year) && \
				 (local_time_str.dir_month == directory_time->dir_month) && \
				 (local_time_str.dir_week == directory_time->dir_week) && \
				 (local_time_str.dir_day == directory_time->dir_day)) {
			// Estamos en el mismo dia... solo sumamos los .statistics
			line_counter = list_counter = 0;
			aux_flo = files_struct;

			sprintf(aux_line, "%s.%s.files", process_file, iface_information[position].interface);
			fd = open(aux_line, O_RDONLY);
			if (fd >= 0) {
				result = read_line(fd, rline);
		
				while (((result >= 0) || (strlen(rline) > 0))) {
					if ((aux_flo != NULL) && (strcmp(rline, aux_flo->filename) == 0)) {
						list_counter++;
						aux_flo = aux_flo->next; 
					}
					line_counter++;
					result = read_line(fd, rline);
				}
				close(fd);
				fd = 1;
			}

			if ((fd < 0) || (line_counter != list_counter) || (aux_flo != NULL)) {
				// Escribimos la estructura en un fichero (para poder comparar despues)
				write_dot_files (process_file, files_struct, position);
				// Solo generamos la suma si han variado los ficheros...
				sum_statistics (process_file, files_struct, position, TYPE_DAY, MODE_LONGLONG);
			}
		}
		else {
			// En cualquier otro caso (o sea, fecha de last_time < que la de procesado), quiere decir
			// que todavia no hemos procesado ningun fichero nuevo... no hacemos nada...
		}
	}
	else if (time_type == TYPE_WEEK) {
		// Solo procesamos las semanas una vez, en el caso que han acabado y que no existe el fichero de estadisticas
		if ((local_time_str.dir_year > directory_time->dir_year) || \
			((local_time_str.dir_year == directory_time->dir_year) && (local_time_str.dir_month > directory_time->dir_month)) || \
			((local_time_str.dir_year == directory_time->dir_year) && (local_time_str.dir_month == directory_time->dir_month) && \
			 (local_time_str.dir_week > directory_time->dir_week))) {
			// Hemos acabado la semana... si no existe el fichero semana.statistics, procesamos.
			// En caso contrario, no hacemos nada
			sprintf(aux_line, "%s.%s.statistics", process_file, iface_information[position].interface);
			fd = open(aux_line, O_RDONLY);
			if (fd < 0) {
				// Debemos procesar, pero solo sumando estadisticas
				sum_statistics (process_file, files_struct, position, TYPE_WEEK, MODE_DOUBLE);
			}
			else {
				// El fichero de estadisticas ya existia... no hacemos nada.
				close(fd);
			}
		}
	}
	else if (time_type == TYPE_MONTH) {
		// Solo procesamos los meses una vez, en el caso que han acabado y que no existe el fichero de estadisticas
		if ((local_time_str.dir_year > directory_time->dir_year) || \
			((local_time_str.dir_year == directory_time->dir_year) && (local_time_str.dir_month > directory_time->dir_month))) {
			// Hemos acabado el mes... si no existe el fichero mes.statistics, procesamos.
			// En caso contrario, no hacemos nada
			sprintf(aux_line, "%s.%s.statistics", process_file, iface_information[position].interface);
			fd = open(aux_line, O_RDONLY);
			if (fd < 0) {
				// Debemos procesar, pero solo sumando estadisticas
				sum_statistics (process_file, files_struct, position, TYPE_MONTH, MODE_DOUBLE);
			}
			else {
				// El fichero de estadisticas ya existia... no hacemos nada.
				close(fd);
			}
		}
	}
	else if (time_type == TYPE_YEAR) {
		// Solo procesamos los anyos una vez, en el caso que han acabado y que no existe el fichero de estadisticas
		if ((local_time_str.dir_year > directory_time->dir_year)) {
			// Hemos acabado el anyo... si no existe el fichero anyo.statistics, procesamos.
			// En caso contrario, no hacemos nada
			sprintf(aux_line, "%s.%s.statistics", process_file, iface_information[position].interface);
			fd = open(aux_line, O_RDONLY);
			if (fd < 0) {
				// Debemos procesar, pero solo sumando estadisticas
				sum_statistics (process_file, files_struct, position, TYPE_YEAR, MODE_DOUBLE);
			}
			else {
				// El fichero de estadisticas ya existia... no hacemos nada.
				close(fd);
			}
		}
	}

	// Y borramos el listado de memoria...
	aux_flo = files_struct;
	while (aux_flo != NULL) {
		aux_flo2 = aux_flo;
		aux_flo = aux_flo->next;
		free(aux_flo2);
	}

	return 0;
}

flow_adder_struct *add_flow_adder_str_element (flow_adder_struct *list, flow_adder_struct *element)
{
	flow_adder_struct *tmp_elem, *first;

	first = tmp_elem = list;
	if (tmp_elem == NULL) {
		tmp_elem = element;
		element->next = NULL;
		return tmp_elem;
	}

	while (tmp_elem->next != NULL)
		tmp_elem = tmp_elem->next;

	tmp_elem->next = element;
	element->next = NULL;
	return first;
}

int flow_adder_process (char *captured_directory, char *processed_directory, char *interface, int position, long *first_time, long *last_time)
{
	/*
	 La idea es:
	 - Procese los de la hora anterior (o todos los anteriores si no encuentra los .processed)
	 - Escribir el resultado en %DIRECTORY_PROCESSED
	 - gzip --best de los .data procesados!!! (excepto del ultimo si pasa de hora, se tendra en cuenta en (^))
	 - Procesa los del dia anterior a partir de los de la horas. 
	 - gzip --best de los .data procesados!!! (excepto del ultimo si pasa de hora, se tendra en cuenta en (^))
	 - Procesa los de la semana
	 - .....
	*/
	char aux_line[MAX_LINE], aux_line2[MAX_LINE], output_filename[MAX_LINE];
	char captured_file[MAX_LINE];
	long file_first_time, file_last_time;
	flow_adder_struct *file_files, *day_files, *week_files, *month_files;
	flow_adder_struct *aux_element, *aux_element2;
	flow_adder_struct *files_flow_to_process;

	char file_name_data[MAX_LINE];
	char network_interface[MAX_LENGTH_IFACE];
	char dot_char[2]=".";
	char *string_ptr;
	int saved_disk;
	int total_days;

	time_t aux_time;
	struct tm tm_str_time;

	int num_entries_years, num_entries_days, num_entries_files;
	int count_years, count_days, count_files;
    struct dirent **years_list, **days_list, **files_list;
    char year_string[MAX_LINE], day_string[MAX_LINE], files_string[MAX_LINE];
    char aux_line_year[MAX_LINE], aux_line_day[MAX_LINE], aux_line_file[MAX_LINE];

	// Vamos a procesar los ficheros capturados que no lo esten...
	int num_entries_year_p, num_entries_month_p, num_entries_week_p, num_entries_day_p, num_entries_file_p;
	int count_year_p, count_month_p, count_week_p, count_day_p, count_file_p;
    struct dirent **year_list_p, **month_list_p, **week_list_p, **day_list_p, **file_list_p;
    char year_string_p[MAX_LINE], month_string_p[MAX_LINE], week_string_p[MAX_LINE], day_string_p[MAX_LINE], file_string_p[MAX_LINE];
    char aux_line_year_p[MAX_LINE], aux_line_month_p[MAX_LINE], aux_line_week_p[MAX_LINE], aux_line_day_p[MAX_LINE], aux_line_file_p[MAX_LINE];
	disk_path_struct actual_path_dir;

	file_files = day_files = week_files = month_files = NULL;
	files_flow_to_process = NULL;

	if (strstr(compress_bin_path, "gzip") != NULL) {
		// File extension will be .gz
		use_gzip_or_bzip = 0;
	}
	else if (strstr(compress_bin_path, "bzip2") != NULL) {
		// File extension will be .bz2
		use_gzip_or_bzip = 1;
	}

	total_days = 0;
	if ((num_entries_years = scandir(captured_directory, &years_list, NULL, alphasort)) < 0) {
		return 1;
	}
	if (num_entries_years) {
		count_years = 0;
		while (count_years < num_entries_years) {
			strcpy(aux_line_year, years_list[count_years]->d_name);
			sprintf(year_string, "%s/%s", captured_directory, aux_line_year);
			if ((strcmp(years_list[count_years]->d_name, ".") != 0) && \
				(strcmp(years_list[count_years]->d_name, "..") != 0) && \
				(is_a_directory(year_string))) {				
				if ((num_entries_days = scandir(year_string, &days_list, NULL, alphasort)) < 0) {
					return 1;
				}
				if (num_entries_days) {
					count_days = 0;
					while (count_days < num_entries_days) {
						strcpy(aux_line_day, days_list[count_days]->d_name);
						sprintf(day_string, "%s/%s", year_string, aux_line_day);
						if ((strcmp(days_list[count_days]->d_name, ".") != 0) && \
							(strcmp(days_list[count_days]->d_name, "..") != 0) && \
							(is_a_directory(day_string))) {
							total_days++;
							if ((num_entries_files = scandir(day_string, &files_list, NULL, alphasort)) < 0) {
								return 1;
							}
							if (num_entries_files) {
								count_files = 0;
								while (count_files < num_entries_files) {
									strcpy(aux_line_file, files_list[count_files]->d_name);
									sprintf(files_string, "%s/%s", day_string, aux_line_file);
									if ((strcmp(files_list[count_files]->d_name, ".") != 0) && \
										(strcmp(files_list[count_files]->d_name, "..") != 0) && \
										(!is_a_directory(files_string))) {
										if (((!use_gzip_or_bzip) && (strstr(aux_line_file, ".debug.gz") == NULL) && (strstr(aux_line_file, ".debug") != NULL)) || \
											((use_gzip_or_bzip) && (strstr(aux_line_file, ".debug.bz2") == NULL) && (strstr(aux_line_file, ".debug") != NULL))) {
											// First of all, we must be sure that file interface is correct
											strcpy(aux_line, aux_line_file);
											string_ptr = strtok(aux_line, dot_char);
											strcpy(file_name_data, string_ptr);
											string_ptr = strtok(NULL, dot_char);
											strcpy(network_interface, string_ptr);
											if (strcmp(network_interface, interface) == 0) {
												// Si es un fichero de la interficie de red que nos ocupa, lo procesamos...
												saved_disk = 0;
												strcpy(captured_file, files_string);
												get_min_max (captured_file, &file_first_time, &file_last_time);
												if (*first_time > file_first_time) {
													// Tenemos que poner este intervalo en un multiplo de 15 minutos
													aux_time = (time_t) file_first_time;
													if (localtime_r(&aux_time, &tm_str_time) == NULL) {
														sprintf(buffer_debug, "Error getting aproximative time");
														save_debug (interface, buffer_debug);
														return -1;
													}
													// PROCESS_PARSE_TIME (in minutes)
													int i, how_many_intervals;
													i = 0;
													how_many_intervals = 1;
													i = (tm_str_time.tm_min*60) + tm_str_time.tm_sec;
													while ( i >= (PROCESS_PARSE_TIME*60) ) {
														i = i - (PROCESS_PARSE_TIME*60);
														how_many_intervals++;
													}
													if ((how_many_intervals * PROCESS_PARSE_TIME) >= 60) {
														tm_str_time.tm_min = 0;
														tm_str_time.tm_sec = 0;
														tm_str_time.tm_hour++;
													}
													else {
														tm_str_time.tm_min = how_many_intervals * PROCESS_PARSE_TIME;
														tm_str_time.tm_sec = 0;
													}
													aux_time = mktime(&tm_str_time);
													*first_time = (long) aux_time;
												}
												if (*last_time < file_last_time){
													*last_time = file_last_time;
												}
												replace_string_with(captured_file, ".debug", ".data");
												aux_element = (flow_adder_struct *) malloc(sizeof(flow_adder_struct));
												if (aux_element != NULL) {
													strcpy (aux_element->filename, captured_file);
													aux_element->begin = file_first_time;
													aux_element->end = file_last_time;
													aux_element->next = NULL;
													files_flow_to_process = add_flow_adder_str_element (files_flow_to_process, aux_element);
												}
												else {
													sprintf (buffer_debug, "Error allocating files memory");
													save_debug (interface, buffer_debug);
												}
									
												// Y miramos si podemos volcar a disco los ficheros en espera
												while ((long) (*last_time) > ((long) (*first_time + PROCESS_PARSE_TIME_SECONDS))) {
													// Process element (only 1 time)
													if (!saved_disk) process_last_interval (files_flow_to_process, position, DOT_CAPTURED, 0);
													// And save to disk
													create_file_system (processed_directory, output_filename, TYPE_PROCESSER, 1, 1, (time_t) (*first_time + (PROCESS_PARSE_TIME_SECONDS-1)), (uid_t) -1, file_gid);
													save_data_aggregations (output_filename, &iface_information[position], *first_time, *first_time + (PROCESS_PARSE_TIME_SECONDS-1), DOT_CAPTURED, MODE_LONGLONG);
#ifdef _VIEWER
													save_statistics (output_filename, &iface_information[position], MODE_LONGLONG, "");
#else
													save_statistics (output_filename, &iface_information[position], MODE_LONGLONG);
#endif
													print_and_reset_some_counters (&iface_information[position], 1);
													*first_time += PROCESS_PARSE_TIME_SECONDS;
													saved_disk = 1;
												}
												if (saved_disk) {
													// Now, we can compress .data and .debug files
													aux_element = files_flow_to_process;
													while (aux_element != NULL) {
														// Compress .data file
														sprintf(aux_line, "%s %s", compress_bin_path, aux_element->filename);
														system(aux_line);
														// And compress .debug file
														strcpy(aux_line, aux_element->filename);
														replace_string_with(aux_line, ".data", ".debug");
														sprintf(aux_line2, "%s %s", compress_bin_path, aux_line);
														system(aux_line2);
														aux_element2 = aux_element;
														aux_element = aux_element->next;
														free(aux_element2);
													}
													files_flow_to_process = NULL;
												}
											}
										}
									}
									free(files_list[count_files]);
									count_files++;
								}
								free(files_list);
							}
						}
						free(days_list[count_days]);
						count_days++;
					}
					free(days_list);
				}
			}
			free(years_list[count_years]);
			count_years++;
		}
		free(years_list);
	}

	// Con esto, ya tenemos los dias parseados por el intervalo especificado en el fichero de configuracion
	// Ahora procesaremos los siguientes intervalos... dias, semanas, meses y, finalmente, anyos
	// IPSS/data/processed/				-> processed_directory
	//						2007/		-> year
	//						04/			-> month
	//						04/			-> week
	//						07/			-> day
	//						1177273800.fxp0.processed	-> data_file


	if ((num_entries_year_p = scandir(processed_directory, &year_list_p, NULL, alphasort)) < 0) {
		sprintf(buffer_debug, "No files yet to process...");
		save_debug(interface, buffer_debug);
		return 1;
	}
	if (num_entries_year_p) {
		count_year_p = 0;
		while (count_year_p < num_entries_year_p) {
			strcpy(aux_line_year_p, year_list_p[count_year_p]->d_name);
			sprintf(year_string_p, "%s/%s", processed_directory, aux_line_year_p);
			if ((strcmp(year_list_p[count_year_p]->d_name, ".") != 0) && \
				(strcmp(year_list_p[count_year_p]->d_name, "..") != 0) && \
				(is_a_directory(year_string_p))) {

				actual_path_dir.dir_year = atoi(year_list_p[count_year_p]->d_name);

				if ((num_entries_month_p = scandir(year_string_p, &month_list_p, NULL, alphasort)) < 0) {
					return 1;
				}
				if (num_entries_month_p) {
					month_files = NULL;
					count_month_p = 0;
					while (count_month_p < num_entries_month_p) {
						strcpy(aux_line_month_p, month_list_p[count_month_p]->d_name);
						sprintf(month_string_p, "%s/%s", year_string_p, aux_line_month_p);
						if ((strcmp(month_list_p[count_month_p]->d_name, ".") != 0) && \
							(strcmp(month_list_p[count_month_p]->d_name, "..") != 0) && \
							(is_a_directory(month_string_p))) {

							actual_path_dir.dir_month = atoi(month_list_p[count_month_p]->d_name);

							if ((num_entries_week_p = scandir(month_string_p, &week_list_p, NULL, alphasort)) < 0) {
								return 1;
							}
							if (num_entries_week_p) {
								week_files = NULL;
								count_week_p = 0;
								while (count_week_p < num_entries_week_p) {
									strcpy(aux_line_week_p, week_list_p[count_week_p]->d_name);
									sprintf(week_string_p, "%s/%s", month_string_p, aux_line_week_p);
									if ((strcmp(week_list_p[count_week_p]->d_name, ".") != 0) && \
										(strcmp(week_list_p[count_week_p]->d_name, "..") != 0) && \
										(is_a_directory(week_string_p))) {

										actual_path_dir.dir_week = atoi(week_list_p[count_week_p]->d_name);

										if ((num_entries_day_p = scandir(week_string_p, &day_list_p, NULL, alphasort)) < 0) {
											return 1;
										}
										if (num_entries_day_p) {
											day_files = NULL;
											count_day_p = 0;
											while (count_day_p < num_entries_day_p) {
												strcpy(aux_line_day_p, day_list_p[count_day_p]->d_name);
												sprintf(day_string_p, "%s/%s", week_string_p, aux_line_day_p);
												if ((strcmp(day_list_p[count_day_p]->d_name, ".") != 0) && \
													(strcmp(day_list_p[count_day_p]->d_name, "..") != 0) && \
													(is_a_directory(day_string_p))) {

													actual_path_dir.dir_day = atoi(day_list_p[count_day_p]->d_name);

													if ((num_entries_file_p = scandir(day_string_p, &file_list_p, NULL, alphasort)) < 0) {
														return 1;
													}
													if (num_entries_file_p) {
														file_files = NULL;
														count_file_p = 0;
														while (count_file_p < num_entries_file_p) {
															strcpy(aux_line_file_p, file_list_p[count_file_p]->d_name);
															sprintf(file_string_p, "%s/%s", day_string_p, aux_line_file_p);
															if ((strcmp(file_list_p[count_file_p]->d_name, ".") != 0) && \
																(strcmp(file_list_p[count_file_p]->d_name, "..") != 0) && \
																(!is_a_directory(file_string_p)) && \
																(strstr(file_list_p[count_file_p]->d_name, "processed") != NULL)) {

																strcpy(aux_line, aux_line_file_p);
																string_ptr = strtok(aux_line, dot_char);
																strcpy(file_name_data, string_ptr);
																string_ptr = strtok(NULL, dot_char);
																strcpy(network_interface, string_ptr);
																if (strcmp(network_interface, interface) == 0) {
																	// En la estructura, solo guardamos el nombre del fichero... no la ruta absoluta
																	aux_element = (flow_adder_struct *) malloc(sizeof(flow_adder_struct));
																	if (aux_element != NULL) {
																		strcpy (aux_element->filename, aux_line_file_p);
																		file_files = add_flow_adder_str_element (file_files, aux_element);
																	}
																	else {
																		sprintf (buffer_debug, "Error allocating files memory");
																		save_debug (interface, buffer_debug);
																	}
																}
															}
															free(file_list_p[count_file_p]);
															count_file_p++;
														}
														save_process_substruct (day_string_p, file_files, position, TYPE_DAY, *first_time, &actual_path_dir);

														// En la estructura, solo guardamos el nombre del fichero... no la ruta absoluta
														aux_element = (flow_adder_struct *) malloc(sizeof(flow_adder_struct));
														if (aux_element != NULL) {
															sprintf(aux_line, "%s.%s.statistics", aux_line_day_p, interface);
															strcpy (aux_element->filename, aux_line);
															day_files = add_flow_adder_str_element (day_files, aux_element);
														}
														else {
															sprintf (buffer_debug, "Error allocating day memory");
															save_debug (interface, buffer_debug);
														}
														free(file_list_p);
													}
												}
												free(day_list_p[count_day_p]);
												count_day_p++;
											}
											save_process_substruct (week_string_p, day_files, position, TYPE_WEEK, *first_time, &actual_path_dir);

											// En la estructura, solo guardamos el nombre del fichero... no la ruta absoluta
											aux_element = (flow_adder_struct *) malloc(sizeof(flow_adder_struct));
											if (aux_element != NULL) {
												sprintf(aux_line, "%s.%s.statistics", aux_line_week_p, interface);
												strcpy (aux_element->filename, aux_line);
												week_files = add_flow_adder_str_element (week_files, aux_element);
											}
											else {
												sprintf (buffer_debug, "Error allocating week memory");
												save_debug (interface, buffer_debug);
											}
											free(day_list_p);
										}
									}
									free(week_list_p[count_week_p]);
									count_week_p++;
								}
								save_process_substruct (month_string_p, week_files, position, TYPE_MONTH, *first_time, &actual_path_dir);

								// En la estructura, solo guardamos el nombre del fichero... no la ruta absoluta
								aux_element = (flow_adder_struct *) malloc(sizeof(flow_adder_struct));
								if (aux_element != NULL) {
									sprintf(aux_line, "%s.%s.statistics", aux_line_month_p, interface);
									strcpy (aux_element->filename, aux_line);
									month_files = add_flow_adder_str_element (month_files, aux_element);
								}
								else {
									sprintf (buffer_debug, "Error allocating month memory");
									save_debug (interface, buffer_debug);
								}
								free(week_list_p);
							}
						}
						free(month_list_p[count_month_p]);
						count_month_p++;
					}
					save_process_substruct (year_string_p, month_files, position, TYPE_YEAR, *first_time, &actual_path_dir);
					free(month_list_p);
				}
			}
			free(year_list_p[count_year_p]);
			count_year_p++;
		}
		free(year_list_p);
	}

	// If keep_days_captured <= 0, feature disabled
	if (keep_days_captured > 0) {
		// If the number of days is greater than "keep_days_captured", we must delete some days
		while (total_days > keep_days_captured) {
			if ((num_entries_years = scandir(captured_directory, &years_list, NULL, alphasort)) < 0) {
				return 1;
			}
			if (num_entries_years) {
				count_years = 0;
				while (count_years < num_entries_years) {
					strcpy(aux_line_year, years_list[count_years]->d_name);
					sprintf(year_string, "%s/%s", captured_directory, aux_line_year);
					if ((strcmp(years_list[count_years]->d_name, ".") != 0) && \
						(strcmp(years_list[count_years]->d_name, "..") != 0) && \
						(is_a_directory(year_string))) {

						if ((num_entries_days = scandir(year_string, &days_list, NULL, alphasort)) < 0) {
							return 1;
						}
						if (num_entries_days) {
							count_days = 0;
							while (count_days < num_entries_days) {
								strcpy(aux_line_day, days_list[count_days]->d_name);
								sprintf(day_string, "%s/%s", year_string, aux_line_day);
								if ((strcmp(days_list[count_days]->d_name, ".") != 0) && \
									(strcmp(days_list[count_days]->d_name, "..") != 0) && \
									(is_a_directory(day_string))) {

									if (total_days > keep_days_captured) {
										sprintf(aux_line, "\\rm -rf %s", day_string);
										system(aux_line);
										total_days--;
									}
								}
								free(days_list[count_days]);
								count_days++;
							}
							free(days_list);
						}
						if (total_days > keep_days_captured) {
							// If this is true, we must delete, also, the year directory
							sprintf(aux_line, "\\rm -rf %s", year_string);
							system(aux_line);
							// But we mustn't rest anything on total_days
						}
					}
					free(years_list[count_years]);
					count_years++;
				}
				free(years_list);
			}
		}
	}
	return 0;
}
