#include <defines.h>
#include <config_file.h>
#include <flow_adder.h>
#include <common.h>

unsigned int max_interfaces;

interface_str *interface;

char buffer_debug[MAX_BUFFER];
int exit_main;

int capture_time;
int process_time;
long unfinished_connections_seconds;
unsigned long max_packets;
unsigned long min_packets;
unsigned long max_hash;
char *flow_adder_pid;
char *captured_directory;
char *processed_directory;
char *flow_adder_log_file;
int file_gid;
char *compress_bin_path;
char *uncompress_bin_path;
int keep_days_captured;
int top_n;
char *local_network;

void exit_signal (int signum)
{
	signal(signum, exit_signal);
	exit_main = 1;
	sprintf(buffer_debug, "STOPPED by user");
	save_debug ("main", buffer_debug);
}

int parse_interfaces (char *interfaces, unsigned long wait_time)
{
	unsigned int i;
	int is_valid;
	char characters[2]=",";
	char *result = NULL;
	char interfaces_copy[MAX_STRING_IFACES];

	int file_descriptor;
	char aux_line[MAX_LINE], rline[MAX_LINE];
	int fd_result;

	sprintf(aux_line, "%s/.interfaces", captured_directory);
	file_descriptor = open (aux_line, O_RDONLY);
	while (file_descriptor < 0) {
		sprintf(buffer_debug, "Information about the capture not available... sleeping");
		save_debug("main", buffer_debug);
		sleep ((unsigned int) wait_time);
		file_descriptor = open (aux_line, O_RDONLY);
	}

	fd_result = read_line(file_descriptor, rline);

	i = 0;
	while ((fd_result >= 0) || (strlen(rline) > 0)) {
		strcpy(interfaces_copy, interfaces);
		result = strtok(interfaces_copy, characters);
		is_valid = 0;
		while ((result != NULL) && (is_valid == 0)) {
			if (strcmp(rline,result) == 0) {
				i++;
				is_valid = 1;
			}
			result = strtok(NULL, characters);
		}
		fd_result = read_line(file_descriptor, rline);
	}
	close (fd_result);

	max_interfaces = i;

	interface = (interface_str *) malloc(max_interfaces*sizeof(interface_str));

	file_descriptor = open (aux_line, O_RDONLY);
	while (file_descriptor < 0) {
		sprintf(buffer_debug, "Information about the capture not available... sleeping");
		save_debug("main", buffer_debug);
		sleep ((unsigned int) wait_time);
		file_descriptor = open (aux_line, O_RDONLY);
	}

	fd_result = read_line(file_descriptor, rline);

	i = 0;
	while ((fd_result >= 0) || (strlen(rline) > 0)) {
		strcpy(interfaces_copy, interfaces);
		result = strtok(interfaces_copy, characters);
		is_valid = 0;
		while ((result != NULL) && (is_valid == 0)) {
			if (strcmp(rline, result) == 0) {
				strcpy(interface[i].name, result);
				i++;
				is_valid = 1;
			}
			result = strtok(NULL, characters);
		}
		fd_result = read_line(file_descriptor, rline);
	}
	close (fd_result);

	if ( max_interfaces == 0 ) {
		sprintf(buffer_debug, "Error... No valid network interface found... exiting");
		save_debug ("main", buffer_debug);
		return -2;
	}

	return 0;
}

void save_debug (char *interface, char *message)
{
	int fd;
	time_t time_debug;
	struct tm *time_str_debug;
	char buffer_string[1024];
	char output_file[MAX_LINE];

	strcpy(output_file, flow_adder_log_file);
	fd = open (output_file, O_WRONLY|O_CREAT|O_APPEND);
	if (fd < 0) {
		printf("Error creating log file... aborting\n");
		exit (0);
	}
	else {
		time(&time_debug);
		time_str_debug = localtime(&time_debug);
		sprintf(buffer_string, "[ flow_adder - %-6s ] | %02d/%02d/%02d %02d:%02d:%02d | %s\n", interface, time_str_debug->tm_mday, time_str_debug->tm_mon, \
				time_str_debug->tm_year-100, time_str_debug->tm_hour, time_str_debug->tm_min, time_str_debug->tm_sec, message);
		write(fd, buffer_string, strlen(buffer_string));
		close(fd);
		chown(output_file, (uid_t) -1, file_gid);
		chmod(output_file, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
	}
}

int main(int argc, char *argv[])
{
	int start_main, stop_main, args_counter, specified_filename, file_descriptor;
	unsigned int i;
	unsigned long process_time_seconds;
	int unfinished_connections;
	int nice_level;
	int fd;
	struct stat sb;
	char config_file[MAX_LINE];
	time_t actual_time, final_time;
	unsigned long memory_size;

	start_main = stop_main = exit_main = specified_filename = 0;
	args_counter = 1;
	strcpy(config_file, "./IPSS.conf");

	if ((argc < 2) || (argc > 4)) {
		printf("Error in number of arguments\n  Usage: %s [start|stop] [-c IPSS.conf]\n", argv[0]);
		return -2;
	}
	while (args_counter < argc) {
		if (strcmp(argv[args_counter], "start") == 0) {
			start_main = 1;
		}
		else if (strcmp(argv[args_counter], "stop") == 0) {
			stop_main = 1;
		}
		else if (strcmp(argv[args_counter], "-c") == 0) {
			args_counter++;
			if ((args_counter < argc) && (strlen(argv[args_counter]) < MAX_LINE)) {
					strcpy(config_file, argv[args_counter]);
					specified_filename = 1;
			}
		}
		else {
			printf("Error, unknown option \"%s\"\n", argv[args_counter]);
		}
		args_counter++;
	}
	if ((file_descriptor = open(config_file, O_RDONLY)) < 0) {
		if (specified_filename) {
			printf("Error, filename %s not found\n", config_file);
			return -1;
		}
		else {
			printf("Error, you must specify a configuration file with \"-c\"\n");
			return -1;
		}
	}
	close(file_descriptor);
	if ((start_main) && (stop_main)) {
		printf("Error, you must specify start or stop, not both\n");
		return -1;
	}
	else if ((!start_main) && (!stop_main)) {
		printf("Error, you must specify one action to attend...\n");
		return -1;
	}

	flow_adder_pid = get_flow_adder_pid(config_file);
	captured_directory = get_captured_directory(config_file);
	processed_directory = get_processed_directory(config_file);
	flow_adder_log_file = get_flow_adder_log_file(config_file);
	nice_level = get_nice_level(config_file);
	compress_bin_path = get_compress_path(config_file);
	uncompress_bin_path = get_uncompress_path(config_file);
	keep_days_captured = get_keep_days (config_file);
	top_n = get_top_n (config_file);
	local_network = get_local_network (config_file);

	file_gid = get_gid_group(config_file);
	if (file_gid < 0) {
		printf("Error, group %s inexisting\n", get_group(config_file));
		return -1;
	}
	// Creamos el fichero de log
	fd = open(flow_adder_log_file, O_RDWR|O_CREAT|O_APPEND);
	if (fd < 0) {
		printf("Cann't create log file %s.. exiting\n", flow_adder_log_file);
		return -1;
	}
	close(fd);
	chown(flow_adder_log_file, (uid_t) -1, file_gid);
	chmod(flow_adder_log_file, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);

	capture_time = get_capture_time(config_file);
	if (capture_time < 1) {
		sprintf(buffer_debug, "Error... capture_time parameter has forbidden value (it must be >= 1)");
		save_debug("main", buffer_debug);
	}
	process_time = get_process_time(config_file);
	if (process_time < 5) {
		sprintf(buffer_debug, "Error... process_time parameter has forbidden value (it must be >= 5)");
		save_debug("main", buffer_debug);
	}

	// Convertimos process_time en segundos
	process_time_seconds = process_time * SECONDS_PER_MINUTE;
	unfinished_connections = get_unfinished_connections_time (config_file);
	unfinished_connections_seconds = unfinished_connections * SECONDS_PER_MINUTE;
	max_packets = get_max_block_number(config_file);
	min_packets = get_min_block_number(config_file);
	max_hash = get_hash_size_flow_adder(config_file);

	// We must execute flow_adder process at low priority
	nice(nice_level);

	if(stat(get_home_directory(config_file), &sb) < 0)
	{
		// The path element didn't exist
		mkdir(get_home_directory(config_file), 0770);
		if (stat(get_home_directory(config_file), &sb) < 0) {
			printf("Cann't create home directory %s\n", get_home_directory(config_file));
			return -1;
		}
		chown(get_home_directory(config_file), (uid_t) -1, file_gid);
		chmod(get_home_directory(config_file), S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
	}
	if(!S_ISDIR(sb.st_mode))
	{
		printf("home (%s) is not a directory\n", get_home_directory(config_file));
		return -1;
	}

	if (stop_main) {
		file_descriptor = open(flow_adder_pid, O_RDONLY);
		if (file_descriptor < 0) {
			sprintf(buffer_debug, "Error, pid file doesn't exist");
			save_debug("main", buffer_debug);
		}
		else {
			char fpid[40];
			read(file_descriptor, fpid, 40);
			kill(atoi(fpid), SIGTERM);
			close(file_descriptor);
			unlink(flow_adder_pid);
		}
	}
	else if (start_main) {

		if (process_time < capture_time) {
			sprintf(buffer_debug, "Warning... process time value greater than capture time");
			save_debug("main", buffer_debug);
		}

		file_descriptor = open(flow_adder_pid, O_RDONLY);
		if (file_descriptor >= 0) {
			sprintf(buffer_debug, "Error, another process running ? %s not started", argv[0]);
			save_debug ("main", buffer_debug);
			close(file_descriptor);
			return -1;
		}

		int detach_pid;

		detach_pid = fork();
		if (detach_pid == 0) {
			if (parse_interfaces(get_interfaces(config_file), process_time_seconds) < 0) return -2;
			sprintf(buffer_debug, "Starting flow_adder (v%s) [", VERSION);
			for (i=0; i<max_interfaces; i++) {
				strcat(buffer_debug, interface[i].name);
				if ((i+1) < max_interfaces) strcat(buffer_debug, ", ");
			}
			memory_size = ((max_packets * sizeof(memory_element)) + sizeof(bytes_and_packets) + \
						   ( NUMBER_OF_HASHES * max_hash * sizeof (memory_element *)))/1024;
			sprintf(buffer_debug,"%s] x %luKB (process time: %d minutes)", buffer_debug, memory_size, process_time);
			save_debug ("main", buffer_debug);

			// Reprogram signals...
			signal(SIGTERM, exit_signal); signal(SIGKILL, exit_signal); signal(SIGINT, exit_signal);

			// We must save pid on file
			file_descriptor = open(flow_adder_pid, O_WRONLY|O_CREAT|O_TRUNC);
			if (file_descriptor < 0) {
				sprintf(buffer_debug,"Cannot save pid on %s", flow_adder_pid);
				save_debug("main", buffer_debug);
				return -1;
			}
			else {
				char pid_buffer[40];
				sprintf(pid_buffer, "%i", getpid());
				write(file_descriptor, pid_buffer, strlen(pid_buffer));
				close(file_descriptor);
				chown(flow_adder_pid, (uid_t) -1, file_gid);
				chmod(flow_adder_pid, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
			}

			initialize_bytes_and_packets (max_interfaces, interface);

			long first_time[max_interfaces], last_time[max_interfaces];
			for (i = 0; i < max_interfaces; i++) {
				first_time[i] = 2147483647L;
				last_time[i] = 0;
			}

			time(&actual_time);
			final_time = actual_time;

			// Main while
			while (!exit_main) {
				if (((long) (actual_time + process_time_seconds)) > (long) final_time) {
					sleep ((unsigned int) difftime(actual_time + process_time_seconds, final_time));
				}
				else {
					// Si nos hemos pasado del tiempo establecido en el fichero, procesamos sin esperar...
				}
				time(&actual_time);
				if (!exit_main) {
					for (i = 0; i < max_interfaces; i++) {
						// Generamos ficheros en intervalos de 30 minutos...
						flow_adder_process (captured_directory, processed_directory, interface[i].name, i, &first_time[i], &last_time[i]);
					}
				}
				time(&final_time);
			}
			for (i = 0; i < max_interfaces; i++) {
				free_all_memory(i);
			}
		}
	}
	return 0;
}
