#include <defines.h>

extern char buffer_debug[MAX_BUFFER];

statistics_element_bytes **aux_array_bytes;			// Vector auxiliar ordenado por bytes...
statistics_element_accessed **aux_array_accessed;	// Vector auxiliar ordenado por veces accedidas
statistics_element_ports **aux_array_ports;			// Vector auxiliar ordenado por veces accedidas los puertos

unsigned long long element_bytes_allocated = 0;
unsigned long long element_accessed_allocated = 0;
unsigned long long element_ports_allocated = 0;

statistics_element_bytes *allocate_new_element_bytes (char *interface, int mode)
{
	statistics_element_bytes *element;

	element = (statistics_element_bytes *) malloc (sizeof(statistics_element_bytes));
	if (element != NULL) {
		element_bytes_allocated++;
		if (mode == MODE_DOUBLE)
			element->d_bytes = 0;
		else if (mode == MODE_LONGLONG)
			element->ll_bytes = 0;
		element->next = NULL;
	}
	else {
		sprintf(buffer_debug, "Error in getting more memory for allocate_new_element_bytes");
		save_debug(interface, buffer_debug);
	}
	return element;
}

statistics_element_accessed *allocate_new_element_accessed (char *interface, int mode)
{
	statistics_element_accessed *element;
	
	element = (statistics_element_accessed *) malloc (sizeof(statistics_element_accessed));
	if (element != NULL) {
		element_accessed_allocated++;
		if (mode == MODE_DOUBLE)
			element->d_accessed = 0;
		else if (mode == MODE_LONGLONG)
			element->ll_accessed = 0;
		element->next = NULL;
	}
	else {
		sprintf(buffer_debug, "Error in getting more memory for allocate_new_element_accessed");
		save_debug(interface, buffer_debug);
	}
	return element;
}

statistics_element_ports *allocate_new_element_ports (char *interface, int mode)
{
	statistics_element_ports *element;
	
	element = (statistics_element_ports *) malloc (sizeof(statistics_element_ports));
	if (element != NULL) {
		element_ports_allocated++;
		if (mode == MODE_DOUBLE)
			element->d_times = 0;
		else if (mode == MODE_LONGLONG)
			element->ll_times = 0;
		element->next = NULL;
	}
	else {
		sprintf(buffer_debug, "Error in getting more memory for allocate_new_element_port");
		save_debug(interface, buffer_debug);
	}
	return element;
}

void deallocate_element_bytes (statistics_element_bytes *package)
{
	free (package);
	element_bytes_allocated--;
}

void deallocate_element_accessed (statistics_element_accessed *package)
{
	free (package);
	element_accessed_allocated--;
}

void deallocate_element_ports (statistics_element_ports *package)
{
	free (package);
	element_ports_allocated--;
}


statistics_element_bytes *search_element_bytes (statistics_element_bytes **list, struct in_addr ip_address)
{
	statistics_element_bytes *actual;

	actual = *list;
	while ((actual != NULL) && (actual->ip_address.s_addr != ip_address.s_addr)) {
		actual = actual->next;
	}

	return actual;
}

statistics_element_accessed *search_element_accessed (statistics_element_accessed **list, struct in_addr ip_address)
{
	statistics_element_accessed *actual;
	
	actual = *list;
	while ((actual != NULL) && (actual->ip_address.s_addr != ip_address.s_addr)) {
		actual = actual->next;
	}
	
	return actual;
}

statistics_element_ports *search_element_ports (statistics_element_ports **list, unsigned int port_number)
{
	statistics_element_ports *actual;
	
	actual = *list;
	while ((actual != NULL) && (actual->port_number != port_number)) {
		actual = actual->next;
	}
	
	return actual;
}


int compare_element_bytes (statistics_element_bytes *first, statistics_element_bytes *second, int mode)
{
	if (mode == MODE_DOUBLE) {
		if (first->d_bytes > second->d_bytes) return 1;
		else if (first->d_bytes < second->d_bytes) return 2;
		else return 0;
	}
	else if (mode == MODE_LONGLONG) {
		if (first->ll_bytes > second->ll_bytes) return 1;
		else if (first->ll_bytes < second->ll_bytes) return 2;
		else return 0;
	}

	return -1;
}

int compare_element_accessed (statistics_element_accessed *first, statistics_element_accessed *second, int mode)
{
	if (mode == MODE_DOUBLE) {
		if (first->d_accessed > second->d_accessed) return 1;
		else if (first->d_accessed < second->d_accessed) return 2;
		else return 0;
	}
	else if (mode == MODE_LONGLONG) {
		if (first->ll_accessed > second->ll_accessed) return 1;
		else if (first->ll_accessed < second->ll_accessed) return 2;
		else return 0;
	}

	return -1;
}

int compare_element_ports (statistics_element_ports *first, statistics_element_ports *second, int mode)
{
	if (mode == MODE_DOUBLE) {
		if (first->d_times > second->d_times) return 1;
		else if (first->d_times < second->d_times) return 2;
		else return 0;
	}
	else if (mode == MODE_LONGLONG) {
		if (first->ll_times > second->ll_times) return 1;
		else if (first->ll_times < second->ll_times) return 2;
		else return 0;
	}

	return -1;
}

int insert_element_bytes (statistics_element_bytes **list, statistics_element_bytes *element)
{
	statistics_element_bytes *actual;

	actual = *list;

	// Insertamos al principio
	*list = element;
	element->next = actual;

	return 0;
}

int insert_element_accessed (statistics_element_accessed **list, statistics_element_accessed *element)
{
	statistics_element_accessed *actual;

	actual = *list;

	// Insertamos al principio
	*list = element;
	element->next = actual;

	return 0;
}

int insert_element_ports (statistics_element_ports **list, statistics_element_ports *element)
{
	statistics_element_ports *actual;

	actual = *list;

	// Insertamos al principio
	*list = element;
	element->next = actual;

	return 0;
}

int remove_list_bytes (unsigned long max)
{
	statistics_element_bytes *aux;
	unsigned long actual_hash;

	for (actual_hash = 0 ; actual_hash < max ; actual_hash++) {
		while (aux_array_bytes[actual_hash] != NULL) {
			aux = aux_array_bytes[actual_hash];
			aux_array_bytes[actual_hash] = aux_array_bytes[actual_hash]->next;
			deallocate_element_bytes (aux);
		}
	}
	free (aux_array_bytes);
	return 0;
}

int remove_list_accessed (unsigned long max)
{
	statistics_element_accessed *aux;
	unsigned long actual_hash;

	for (actual_hash = 0 ; actual_hash < max ; actual_hash++) {
		while (aux_array_accessed[actual_hash] != NULL) {
			aux = aux_array_accessed[actual_hash];
			aux_array_accessed[actual_hash] = aux_array_accessed[actual_hash]->next;
			deallocate_element_accessed (aux);
		}
	}
	free (aux_array_accessed);
	return 0;
}

int remove_list_ports (unsigned long max)
{
	statistics_element_ports *aux;
	unsigned long actual_hash;

	for (actual_hash = 0 ; actual_hash < max ; actual_hash++) {
		while (aux_array_ports[actual_hash] != NULL) {
			aux = aux_array_ports[actual_hash];
			aux_array_ports[actual_hash] = aux_array_ports[actual_hash]->next;
			deallocate_element_ports (aux);
		}
	}
	free (aux_array_ports);
	return 0;
}

void insert_to_ordered_hash_bytes (statistics_element_bytes **list, statistics_element_bytes *package, unsigned long max, int mode)
{
	unsigned long local_hash = 0;
	statistics_element_bytes *actual, *previous;

	// We must insert in aux_array_bytes, ordered
	if (mode == MODE_DOUBLE) local_hash = GET_HASH_BYTES(package->d_bytes, max);
	else if (mode == MODE_LONGLONG) local_hash = GET_HASH_BYTES(package->ll_bytes, max);

	if (list[local_hash] == NULL) {
		// First element...
		list[local_hash] = package;
		package->next = NULL;
	}
	else {
		// Not the first element...
		previous = NULL;
		actual = list[local_hash];
		while ((actual != NULL) && ( compare_element_bytes(actual, package, mode) == 1 )) {
			// Mientras los bytes del paquete sean mas pequenyos que los de la lista
			previous = actual;
			actual = actual->next;
		}
		if (actual == NULL) {
			// Hemos llegado al final de la lista... insertamos despues de previous
			package->next = NULL;
			previous->next = package;
		}
		else {
			// actual no es NULL. Verificamos si es el principio o no...
			if (previous == NULL) {
				// Principio de todo... debemos insertar antes de actual
				package->next = actual;
				list[local_hash] = package;
			}
			else {
				// Por el medio de la lista... (entre previous y actual)
				package->next = actual;
				previous->next = package;
			}
		}
	}
}

void insert_to_ordered_hash_accessed (statistics_element_accessed **list, statistics_element_accessed *package, unsigned long max, int mode)
{
	unsigned long local_hash = 0;
	statistics_element_accessed *actual, *previous;

	// We must insert in aux_array_accessed, ordered
	if (mode == MODE_DOUBLE) local_hash = GET_HASH_ACCESSED(package->d_accessed, max);
	else if (mode == MODE_LONGLONG) local_hash = GET_HASH_ACCESSED(package->ll_accessed, max);

	if (list[local_hash] == NULL) {
		// First element...
		list[local_hash] = package;
		package->next = NULL;
	}
	else {
		// Not the first element...
		previous = NULL;
		actual = list[local_hash];
		while ((actual != NULL) && ( compare_element_accessed(actual, package, mode) == 1 )) {
			// Mientras los accessed del paquete sean mas pequenyos que los de la lista
			previous = actual;
			actual = actual->next;
		}
		if (actual == NULL) {
			// Hemos llegado al final de la lista... insertamos despues de previous
			package->next = NULL;
			previous->next = package;
		}
		else {
			// actual no es NULL. Verificamos si es el principio o no...
			if (previous == NULL) {
				// Principio de todo... debemos insertar antes de actual
				package->next = actual;
				list[local_hash] = package;
			}
			else {
				// Por el medio de la lista... (entre previous y actual)
				package->next = actual;
				previous->next = package;
			}
		}
	}
}

void insert_to_ordered_hash_ports (statistics_element_ports **list, statistics_element_ports *package, unsigned long max, int mode)
{
	unsigned long local_hash = 0;
	statistics_element_ports *actual, *previous;

	// We must insert in aux_array_ports, ordered
	if (mode == MODE_DOUBLE) local_hash = GET_HASH_PORTS(package->d_times, max);
	else if (mode == MODE_LONGLONG) local_hash = GET_HASH_PORTS(package->ll_times, max);

	if (list[local_hash] == NULL) {
		// First element...
		list[local_hash] = package;
		package->next = NULL;
	}
	else {
		// Not the first element...
		previous = NULL;
		actual = list[local_hash];
		while ((actual != NULL) && ( compare_element_ports(actual, package, mode) == 1 )) {
			// Mientras los ports del paquete sean mas pequenyos que los de la lista
			previous = actual;
			actual = actual->next;
		}
		if (actual == NULL) {
			// Hemos llegado al final de la lista... insertamos despues de previous
			package->next = NULL;
			previous->next = package;
		}
		else {
			// actual no es NULL. Verificamos si es el principio o no...
			if (previous == NULL) {
				// Principio de todo... debemos insertar antes de actual
				package->next = actual;
				list[local_hash] = package;
			}
			else {
				// Por el medio de la lista... (entre previous y actual)
				package->next = actual;
				previous->next = package;
			}
		}
	}
}

statistics_element_bytes *get_top_bytes (int mode, unsigned long max)
{
	unsigned long actual_pos, first;
	unsigned long long bytes_first_ll;
	double bytes_first_d;
	statistics_element_bytes *aux_mem;

	first = 0;

	if (mode == MODE_DOUBLE) {
		bytes_first_d = 0;
		for (actual_pos = 0 ; actual_pos < max ; actual_pos++) {
			if ((aux_array_bytes[actual_pos] != NULL) && (aux_array_bytes[actual_pos]->d_bytes >= bytes_first_d)) {
				bytes_first_d = aux_array_bytes[actual_pos]->d_bytes;
				first = actual_pos;
			}
		}
	}
	else if (mode == MODE_LONGLONG) {
		bytes_first_ll = 0;
		for (actual_pos = 0 ; actual_pos < max ; actual_pos++) {
			if ((aux_array_bytes[actual_pos] != NULL) && (aux_array_bytes[actual_pos]->ll_bytes >= bytes_first_ll)) {
				bytes_first_ll = aux_array_bytes[actual_pos]->ll_bytes;
				first = actual_pos;
			}
		}
	}

	// Finalmente, retiramos el elemento del vector y lo devolvemos...
	aux_mem = aux_array_bytes[first];
	if (aux_mem != NULL) {
		aux_array_bytes[first] = aux_array_bytes[first]->next;
		aux_mem->next = NULL;
	}
	return aux_mem;
}

statistics_element_accessed *get_top_accessed (int mode, unsigned long max)
{
	unsigned long actual_pos, first;
	unsigned long long accessed_first_ll;
	double accessed_first_d;
	statistics_element_accessed *aux_mem;

	first = 0;

	if (mode == MODE_DOUBLE) {
		accessed_first_d = 0;
		for (actual_pos = 0 ; actual_pos < max ; actual_pos++) {
			if ((aux_array_accessed[actual_pos] != NULL) && (aux_array_accessed[actual_pos]->d_accessed >= accessed_first_d)) {
				accessed_first_d = aux_array_accessed[actual_pos]->d_accessed;
				first = actual_pos;
			}
		}
	}
	else if (mode == MODE_LONGLONG) {
		accessed_first_ll = 0;
		for (actual_pos = 0 ; actual_pos < max ; actual_pos++) {
			if ((aux_array_accessed[actual_pos] != NULL) && (aux_array_accessed[actual_pos]->ll_accessed >= accessed_first_ll)) {
				accessed_first_ll = aux_array_accessed[actual_pos]->ll_accessed;
				first = actual_pos;
			}
		}
	}

	// Finalmente, retiramos el elemento del vector y lo devolvemos...
	aux_mem = aux_array_accessed[first];
	if (aux_mem != NULL) {
		aux_array_accessed[first] = aux_array_accessed[first]->next;
		aux_mem->next = NULL;
	}
	return aux_mem;
}

statistics_element_ports *get_top_ports (int mode, unsigned long max)
{
	unsigned long actual_pos, first;
	unsigned long long ports_first_ll;
	double ports_first_d;
	statistics_element_ports *aux_mem;

	first = 0;

	if (mode == MODE_DOUBLE) {
		ports_first_d = 0;
		for (actual_pos = 0 ; actual_pos < max ; actual_pos++) {
			if ((aux_array_ports[actual_pos] != NULL) && (aux_array_ports[actual_pos]->d_times >= ports_first_d)) {
				ports_first_d = aux_array_ports[actual_pos]->d_times;
				first = actual_pos;
			}
		}
	}
	else if (mode == MODE_LONGLONG) {
		ports_first_ll = 0;
		for (actual_pos = 0 ; actual_pos < max ; actual_pos++) {
			if ((aux_array_ports[actual_pos] != NULL) && (aux_array_ports[actual_pos]->ll_times >= ports_first_ll)) {
				ports_first_ll = aux_array_ports[actual_pos]->ll_times;
				first = actual_pos;
			}
		}
	}

	// Finalmente, retiramos el elemento del vector y lo devolvemos...
	aux_mem = aux_array_ports[first];
	if (aux_mem != NULL) {
		aux_array_ports[first] = aux_array_ports[first]->next;
		aux_mem->next = NULL;
	}
	return aux_mem;
}

void reorder_hash_bytes (char *interface, statistics_element_bytes **list, unsigned long max, int mode)
{
	statistics_element_bytes *actual_list_position;
	unsigned long actual_hash;

	// Alocatamos espacio para el array
	aux_array_bytes = (statistics_element_bytes **) calloc(max, sizeof(statistics_element_bytes *));

	for (actual_hash = 0; actual_hash < max; actual_hash++) {
		while (list[actual_hash] != NULL) {
			// Movemos el elemento del hash desordenado al que tendremos ordenado...
			actual_list_position = list[actual_hash];
			list[actual_hash] = list[actual_hash]->next;
			// e insertamos el elemento a su nueva posicion...
			actual_list_position->next = NULL;
			insert_to_ordered_hash_bytes (aux_array_bytes, actual_list_position, max, mode);
		}
	}
}

void reorder_hash_accessed (char *interface, statistics_element_accessed **list, unsigned long max, int mode)
{
	statistics_element_accessed *actual_list_position;
	unsigned long actual_hash = 0;

	// Alocatamos espacio para el array
	aux_array_accessed = (statistics_element_accessed **) calloc(max, sizeof(statistics_element_accessed *));

	while (actual_hash < max) {
		while (list[actual_hash] != NULL) {
			// Movemos el elemento del hash desordenado al que tendremos ordenado...
			actual_list_position = list[actual_hash];
			list[actual_hash] = list[actual_hash]->next;
			// e insertamos el elemento a su nueva posicion...
			actual_list_position->next = NULL;
			insert_to_ordered_hash_accessed (aux_array_accessed, actual_list_position, max, mode);
		}
		actual_hash++;
	}
}

void reorder_hash_ports (char *interface, statistics_element_ports **list, unsigned long max, int mode)
{
	statistics_element_ports *actual_list_position;
	unsigned long actual_hash = 0;

	// Alocatamos espacio para el array
	aux_array_ports = (statistics_element_ports **) calloc(max, sizeof(statistics_element_ports *));

	while (actual_hash < max) {
		while (list[actual_hash] != NULL) {
			// Movemos el elemento del hash desordenado al que tendremos ordenado...
			actual_list_position = list[actual_hash];
			list[actual_hash] = list[actual_hash]->next;
			// e insertamos el elemento a su nueva posicion...
			actual_list_position->next = NULL;
			insert_to_ordered_hash_ports (aux_array_ports, actual_list_position, max, mode);
		}
		actual_hash++;
	}
}

statistics_element_bytes *add_address_bytes_statistics (char *interface, statistics_element_bytes **list, struct in_addr ip_address, unsigned long long bytes, int mode)
{
	statistics_element_bytes *actual = NULL;
	statistics_element_bytes *new_element = NULL;

	actual = search_element_bytes(list, ip_address);
	if (actual == NULL) {
		// El elemento no estaba insertado, por lo que tendremos que alocatar memoria, actualizarlo e insertarlo.
		new_element = allocate_new_element_bytes(interface, mode);
		if (new_element != NULL) {
			memcpy(&new_element->ip_address, &ip_address, sizeof(struct in_addr));
			if (mode == MODE_DOUBLE) 
				new_element->d_bytes += bytes;
			else if (mode == MODE_LONGLONG)
				new_element->ll_bytes += bytes;
			insert_element_bytes (list, new_element);
			return new_element;
		}
	}
	else {
		// Lo hemos encontrado. Tenemos que actualizar el elemento.
		if (mode == MODE_DOUBLE) 
			actual->d_bytes += bytes;
		else if (mode == MODE_LONGLONG)
			actual->ll_bytes += bytes;
		return actual;
	}
	return NULL;
}

statistics_element_accessed *add_address_accessed_statistics (char *interface, statistics_element_accessed **list, struct in_addr ip_address, unsigned long times, int mode)
{
	statistics_element_accessed *actual = NULL;
	statistics_element_accessed *new_element = NULL;

	actual = search_element_accessed(list, ip_address);
	if (actual == NULL) {
		// El elemento no estaba insertado, por lo que tendremos que alocatar memoria, actualizarlo e insertarlo.
		new_element = allocate_new_element_accessed(interface, mode);
		if (new_element != NULL) {
			memcpy(&new_element->ip_address, &ip_address, sizeof(struct in_addr));
			if (mode == MODE_DOUBLE) 
				new_element->d_accessed += times;
			else if (mode == MODE_LONGLONG)
				new_element->ll_accessed += times;
			insert_element_accessed (list, new_element);
			return new_element;
		}
	}
	else {
		// Lo hemos encontrado. Tenemos que actualizar el elemento.
		if (mode == MODE_DOUBLE) 
			actual->d_accessed += times;
		else if (mode == MODE_LONGLONG)
			actual->ll_accessed += times;
		return actual;
	}
	return NULL;
}

statistics_element_ports *add_address_ports_statistics (char *interface, statistics_element_ports **list, unsigned int port_number, unsigned long times, int mode)
{
	statistics_element_ports *actual = NULL;
	statistics_element_ports *new_element = NULL;

	actual = search_element_ports(list, port_number);
	if (actual == NULL) {
		// El elemento no estaba insertado, por lo que tendremos que alocatar memoria, actualizarlo e insertarlo.
		new_element = allocate_new_element_ports(interface, mode);
		if (new_element != NULL) {
			new_element->port_number = port_number;
			if (mode == MODE_DOUBLE) 
				new_element->d_times += times;
			else if (mode == MODE_LONGLONG)
				new_element->ll_times += times;
			insert_element_ports (list, new_element);
			return new_element;
		}
	}
	else {
		// Lo hemos encontrado. Tenemos que actualizar el elemento.
		if (mode == MODE_DOUBLE) 
			actual->d_times += times;
		else if (mode == MODE_LONGLONG)
			actual->ll_times += times;
		return actual;
	}
	return NULL;
}
