#include <defines.h>
#include <listener.h>
#include <capturer_thread.h>
#include <common.h>

extern int end_threads;
extern int end_threads_processer;
extern unsigned int max_interfaces;
extern unsigned int capture_time;

char fsalida[MAX_BUFFER];

extern char buffer_debug[MAX_BUFFER];

extern unsigned int stopped_by_user;
extern unsigned long min_packets;
extern unsigned long max_hash;
extern char *captured_directory;
extern int file_gid;

extern bytes_and_packets *iface_information;

#ifdef STATISTICS_CAPTURE
extern debug_str *debug;
#endif

unsigned int search_tcp_state (char *interface, struct tcphdr *tcp_package, unsigned int flow_state)
{
	struct tcphdr *tcp_header;
	unsigned char flags;
	unsigned int return_value = FLOW_ERROR;

	tcp_header = tcp_package;
	flags = tcp_header->th_flags;

	switch (flow_state) {
		case EMPTY_TCP:
			if ( (flags & TH_SYN) && !(flags & TH_ACK) ) {
				return_value = TWHS_SYN;
			}
			else {
				return_value = TCP_DATA;
			}
			break;
		case TWHS_SYN:
			if ( (flags & TH_SYN) && (flags & TH_ACK) ) {
				return_value = TWHS_SYN_ACK;
			}
			else {
				return_value = TWHS_SYN;
			}
			break;
		case TWHS_SYN_ACK:
			if (flags & TH_ACK) {
				return_value = THWS_ACK;
			}
			else {
				return_value = TWHS_SYN_ACK;
			}
			break;
		case THWS_ACK:
			return_value = DATA;
			break;
		case DATA:
			if (flags & TH_FIN) {
				return_value = FWHS_FIN;
			}
			else {
				return_value = DATA;
			}
			break;
		case FWHS_FIN:
			if (flags & TH_ACK) {
				return_value = FWHS_ACK;
			}
			else {
				return_value = FWHS_FIN;
			}
			break;
		case FWHS_ACK:
			if (flags & TH_FIN) {
				return_value = FWHS_FIN2;
			}
			else {
				return_value = FWHS_ACK;
			}
			break;
		case FWHS_FIN2:
			if (flags & TH_ACK) {
				return_value = FWHS_ACK2_CLOSED;
			}
			else {
				return_value = FWHS_FIN2;
			}
			break;
		case FWHS_ACK2_CLOSED:
			sprintf(buffer_debug, "Error entering in FWHS_ACK2_CLOSED");
			save_debug (interface, buffer_debug);
			break;
		case TCP_DATA:
			if (flags & TH_FIN) {
				return_value = TCP_FIN;
			}
			else {
				return_value = TCP_DATA;
			}
			break;
		case TCP_FIN:
			if (flags & TH_ACK) {
				return_value = TCP_ACK;
			}
			else {
				return_value = TCP_FIN;
			}
			break;
		case TCP_ACK:
			if (flags & TH_FIN) {
				return_value = TCP_FIN2;
			}
			else {
				return_value = TCP_ACK;
			}
			break;
		case TCP_FIN2:
			if (flags & TH_ACK) {
				return_value = TCP_ACK2_CLOSED;
			}
			else {
				return_value = TCP_FIN2;
			}
			break;
		case TCP_ACK2_CLOSED:
			sprintf(buffer_debug, "Error entering in TCP_ACK2_CLOSED");
			save_debug (interface, buffer_debug);
			break;
		case FLOW_ERROR:
			sprintf(buffer_debug, "Flow error state reached");
			save_debug (interface, buffer_debug);
			break;
		default:
			sprintf(buffer_debug, "Tcp flow error");
			save_debug (interface, buffer_debug);
			break;
	}
	return return_value;
}

memory_element *search_flow (struct in_addr source_address, unsigned int source_port, struct in_addr destination_address, unsigned int destination_port, u_char protocol, int *direction, memory_element *hash_package)
{
	int exit_function;
	int found = 0;
	int direction_aux = 0;

	if (hash_package != NULL) exit_function = 0;
	else exit_function = 1;

	while (!exit_function) {
		if ( (source_address.s_addr == hash_package->processed.source_address.s_addr) && (source_port == hash_package->processed.source_port) &&
			 (destination_address.s_addr == hash_package->processed.destination_address.s_addr) && (destination_port == hash_package->processed.destination_port) &&
			 (protocol == hash_package->processed.protocol) )
		{
			// Hemos encontrado una entrada con las mismas ips y puertos.. comprobamos que no este cerrado...
			if ( (hash_package->processed.flow_state != FWHS_ACK2_CLOSED) && (hash_package->processed.flow_state != TCP_ACK2_CLOSED) ) {
				// Hemos encontrado la conexion activa... devolvemos el puntero al hash_package
				found = 1;
				direction_aux = 1;
				exit_function = 1;
			}
			// En caso contrario, seguimos buscando...
		}

		// Tambien tenemos que vigilar que el hash_package no venga de vuelta...
		if ( (source_address.s_addr == hash_package->processed.destination_address.s_addr) && (source_port == hash_package->processed.destination_port) &&
			 (destination_address.s_addr == hash_package->processed.source_address.s_addr) && (destination_port == hash_package->processed.source_port) &&
			 (protocol == hash_package->processed.protocol) )
		{
			// Hemos encontrado una entrada con las mismas ips y puertos.. comprobamos que no este cerrado...
			if ( (hash_package->processed.flow_state != FWHS_ACK2_CLOSED) && (hash_package->processed.flow_state != TCP_ACK2_CLOSED) ) {
				// Hemos encontrado la conexion activa... devolvemos el puntero al hash_package
				found = 1;
				direction_aux = -1;
				exit_function = 1;
			}
			// En caso contrario, seguimos buscando...
		}

		if (!found) {
			if (hash_package->next != NULL) hash_package = hash_package->next;
			else exit_function = 1;
		}
	}
	memcpy(direction,&direction_aux,sizeof(direction_aux));	

	if (found) return hash_package;
	else return NULL;
}

void free_package(memory_element *package, bytes_and_packets *bytes_and_packs_ptr)
{
	// Attach the package to free_packages block list
	CONCATENATE_FREE_BLOCK(package, package, bytes_and_packs_ptr->free, bytes_and_packs_ptr->last_free, bytes_and_packs_ptr->cnt_free_mem, 1, bytes_and_packs_ptr->mutual_exclusion_access);
#ifdef STATISTICS_CAPTURE
	pthread_mutex_lock(&debug[bytes_and_packs_ptr->position].mutex_debug);
	RAISE_INFORMATION(debug[bytes_and_packs_ptr->position].packets_recycled, debug[bytes_and_packs_ptr->position].packets_recycled_g, 1, debug[bytes_and_packs_ptr->position].overflow_packets_recycled);
	pthread_mutex_unlock(&debug[bytes_and_packs_ptr->position].mutex_debug);
#endif
}

void free_all_memory(bytes_and_packets *bytes_and_packs_ptr)
{
	unsigned long i;
	unsigned int k;
	memory_element *memory_to_free = NULL;
	memory_element *memory_aux = NULL;

	pthread_mutex_lock(&bytes_and_packs_ptr->mutual_exclusion_access);

	// Primero liberamos la lista de bloques libres
	memory_to_free = bytes_and_packs_ptr->free;
	while (memory_to_free != NULL) {
		memory_aux = memory_to_free;
		memory_to_free = memory_to_free->next;
		free(memory_aux);
#ifdef STATISTICS_CAPTURE
		pthread_mutex_lock(&debug[bytes_and_packs_ptr->position].mutex_debug);
		debug[bytes_and_packs_ptr->position].memory_packets_released++;
		debug[bytes_and_packs_ptr->position].memory_packets_released_free++;
		pthread_mutex_unlock(&debug[bytes_and_packs_ptr->position].mutex_debug);
#endif
	}

	for ( k = 0 ; k < MAX_PROTOCOL ; k++) {
		// Y seguidamente, la que hay ocupada en memoria
		i = 0;
		while ( i < max_hash ) {
			memory_to_free = bytes_and_packs_ptr->array[k][i];
			while (memory_to_free != NULL) {
				memory_aux = memory_to_free;
				memory_to_free = memory_to_free->next;
				free(memory_aux);
#ifdef STATISTICS_CAPTURE
				pthread_mutex_lock(&debug[bytes_and_packs_ptr->position].mutex_debug);
				debug[bytes_and_packs_ptr->position].memory_packets_released++;
				debug[bytes_and_packs_ptr->position].memory_packets_released_allocated++;
				pthread_mutex_unlock(&debug[bytes_and_packs_ptr->position].mutex_debug);
#endif
			}
			i++;
		}
		free(bytes_and_packs_ptr->array[k]);
	}

	pthread_mutex_unlock(&bytes_and_packs_ptr->mutual_exclusion_access);
}

void process_packet (memory_element *package, bytes_and_packets *bytes_and_packs_ptr)
{
	unsigned int length_ip_package;
	unsigned long hash_position = 0L;
	int direction;
	unsigned int tcp_state = 0;
	memory_element new_package;
	memory_element *aux_package;
	u_char icmp_type;
	struct ip *ip_header = NULL;
	struct icmp *icmp_header = NULL;
	struct tcphdr *tcp_header = NULL;
	struct udphdr *udp_header = NULL;
	struct timeval ts;

	memcpy (&ts, &package->raw.ts, sizeof(struct timeval));

	ip_header = &package->raw.ip_h;

	new_package.processed.start_time.tv_sec = 0;
	new_package.processed.start_time.tv_usec = 0;				
	new_package.processed.end_time.tv_sec = 0;
	new_package.processed.end_time.tv_usec = 0;				
	new_package.processed.protocol = ip_header->ip_p;
	memcpy(&new_package.processed.source_address, &ip_header->ip_src, sizeof(struct in_addr));
	memcpy(&new_package.processed.destination_address, &ip_header->ip_dst, sizeof(struct in_addr));
	new_package.processed.source_port = 0;
	new_package.processed.destination_port = 0;
	new_package.processed.source_gb = 0;
	new_package.processed.source_b = 0;
	new_package.processed.destination_gb = 0;
	new_package.processed.destination_b = 0;
	new_package.processed.source_packages_g = 0;
	new_package.processed.source_packages = 0;
	new_package.processed.destination_packages_g = 0;
	new_package.processed.destination_packages = 0;
	new_package.next = NULL;

	length_ip_package = ntohs(ip_header->ip_len);

#ifdef STATISTICS_CAPTURE
	RAISE_INFORMATION(debug[bytes_and_packs_ptr->position].packets_processed, debug[bytes_and_packs_ptr->position].packets_processed_g, 1, debug[bytes_and_packs_ptr->position].overflow_packets_processed);
	RAISE_INFORMATION(debug[bytes_and_packs_ptr->position].processed_b, debug[bytes_and_packs_ptr->position].processed_gb, ntohs(ip_header->ip_len), debug[bytes_and_packs_ptr->position].overflow_processed);
#endif
	
	switch ( new_package.processed.protocol ) {
		case IPPROTO_ICMP:
			icmp_header = &package->raw.icmp_h;
			icmp_type = ntohs(icmp_header->icmp_code);
			break;
		case IPPROTO_TCP:
			tcp_header = &package->raw.tcp_h;
			new_package.processed.source_port = ntohs(tcp_header->th_sport);
			new_package.processed.destination_port = ntohs(tcp_header->th_dport);
			break;
		case IPPROTO_UDP:
			udp_header = &package->raw.udp_h;
			new_package.processed.source_port = ntohs(udp_header->uh_sport);
			new_package.processed.destination_port = ntohs(udp_header->uh_dport);
			break;
		default:
			break;
	}

	hash_position = GET_HASH(new_package.processed.source_address, new_package.processed.source_port, new_package.processed.destination_address, new_package.processed.destination_port, max_hash);

	if (new_package.processed.protocol == IPPROTO_TCP) {
		aux_package = search_flow (new_package.processed.source_address, new_package.processed.source_port, new_package.processed.destination_address, new_package.processed.destination_port, new_package.processed.protocol, &direction, bytes_and_packs_ptr->array[HASH_TCP][hash_position]);
		if ( aux_package != NULL ) {
			// Simplemente, actualizaremos los campos que varien...
			// Llamamos a la funcion "search_tcp_state" pasandole el estado del flujo actual
			tcp_state = search_tcp_state (bytes_and_packs_ptr->interface, tcp_header, aux_package->processed.flow_state);

			if ( (tcp_state == FWHS_ACK2_CLOSED) || (tcp_state == TCP_ACK2_CLOSED) ) {
				// Como ha acabado el flujo, actualizamos la end_time
				memcpy(&aux_package->processed.end_time, &ts, sizeof(struct timeval));
				if ((ts.tv_sec != 0) && ((bytes_and_packs_ptr->tmax == -1) || (ts.tv_sec > bytes_and_packs_ptr->tmax)))
					bytes_and_packs_ptr->tmax = ts.tv_sec;
			}
			aux_package->processed.flow_state = tcp_state;
			if (direction == 1) {
				// de origen a destino
				RAISE_INFORMATION(aux_package->processed.source_packages, aux_package->processed.source_packages_g, 1, debug[bytes_and_packs_ptr->position].overflow_source_packages);
				RAISE_INFORMATION(aux_package->processed.source_b, aux_package->processed.source_gb, (length_ip_package - TCP_PAYLOAD(tcp_header)), debug[bytes_and_packs_ptr->position].overflow_source);
			}
			else if (direction == -1) {
				// de destino a origen
				RAISE_INFORMATION(aux_package->processed.destination_packages, aux_package->processed.destination_packages_g, 1, debug[bytes_and_packs_ptr->position].overflow_destination_packages);
				RAISE_INFORMATION(aux_package->processed.destination_b, aux_package->processed.destination_gb, (length_ip_package - TCP_PAYLOAD(tcp_header)), debug[bytes_and_packs_ptr->position].overflow_destination);
			}
			free_package(package, bytes_and_packs_ptr);
		}
		else {
			// Es el primer paquete que nos llega del flujo...
			new_package.processed.flow_state = EMPTY_TCP;
			tcp_state = search_tcp_state (bytes_and_packs_ptr->interface, tcp_header, new_package.processed.flow_state);
			if (tcp_state == TWHS_SYN) {
				// Estamos al principio del flujo... inicializamos "start_time"
				memcpy(&new_package.processed.start_time, &ts, sizeof(struct timeval));
				if ((ts.tv_sec != 0) && ((bytes_and_packs_ptr->tmin == -1) || (ts.tv_sec < bytes_and_packs_ptr->tmin)))
					bytes_and_packs_ptr->tmin = ts.tv_sec;
			}
			new_package.processed.flow_state = tcp_state;
			new_package.processed.source_b = length_ip_package - TCP_PAYLOAD(tcp_header);
			new_package.processed.source_packages = 1;
			memcpy(package, &new_package, sizeof(memory_element));

			if (bytes_and_packs_ptr->array[HASH_TCP][hash_position] != NULL) {
				package->next = bytes_and_packs_ptr->array[HASH_TCP][hash_position];
			}
			bytes_and_packs_ptr->array[HASH_TCP][hash_position] = package;
		}
	}
	else if (new_package.processed.protocol == IPPROTO_UDP) {
		aux_package = search_flow (new_package.processed.source_address, new_package.processed.source_port, new_package.processed.destination_address, new_package.processed.destination_port, new_package.processed.protocol, &direction, bytes_and_packs_ptr->array[HASH_UDP][hash_position]);
		if ( aux_package != NULL ) {
			memcpy(&aux_package->processed.end_time, &ts, sizeof(struct timeval));
			if ((ts.tv_sec != 0) && ((bytes_and_packs_ptr->tmax == -1) || (ts.tv_sec > bytes_and_packs_ptr->tmax)))
				bytes_and_packs_ptr->tmax = ts.tv_sec;
			if (direction == 1) {
				// de origen a destino
				RAISE_INFORMATION(aux_package->processed.source_packages, aux_package->processed.source_packages_g, 1, debug[bytes_and_packs_ptr->position].overflow_source_packages);
				RAISE_INFORMATION(aux_package->processed.source_b, aux_package->processed.source_gb, (length_ip_package - sizeof(struct udphdr)), debug[bytes_and_packs_ptr->position].overflow_source);
			}
			else if (direction == -1) {
				// de destino a origen
				RAISE_INFORMATION(aux_package->processed.destination_packages, aux_package->processed.destination_packages_g, 1, debug[bytes_and_packs_ptr->position].overflow_destination_packages);
				RAISE_INFORMATION(aux_package->processed.destination_b, aux_package->processed.destination_gb, (length_ip_package - sizeof(struct udphdr)), debug[bytes_and_packs_ptr->position].overflow_destination);
			}
			free_package(package, bytes_and_packs_ptr);
		}
		else {
			// Es el primer paquete que nos llega del flujo...
			new_package.processed.flow_state = UDP_DATA;
			memcpy(&new_package.processed.start_time, &ts, sizeof(struct timeval));
			if ((ts.tv_sec != 0) && ((bytes_and_packs_ptr->tmin == -1) || (ts.tv_sec < bytes_and_packs_ptr->tmin)))
				bytes_and_packs_ptr->tmin = ts.tv_sec;
			memcpy(&new_package.processed.end_time, &ts, sizeof(struct timeval));
			if ((ts.tv_sec != 0) && ((bytes_and_packs_ptr->tmax == -1) || (ts.tv_sec > bytes_and_packs_ptr->tmax)))
				bytes_and_packs_ptr->tmax = ts.tv_sec;
			new_package.processed.source_b = length_ip_package - sizeof(struct udphdr);
			new_package.processed.source_packages = 1;
			memcpy(package, &new_package, sizeof(memory_element));

			if (bytes_and_packs_ptr->array[HASH_UDP][hash_position] != NULL) {
				package->next = bytes_and_packs_ptr->array[HASH_UDP][hash_position];
			}
			bytes_and_packs_ptr->array[HASH_UDP][hash_position] = package;
		}
	}
	else if (new_package.processed.protocol == IPPROTO_ICMP) {
		aux_package = search_flow (new_package.processed.source_address, new_package.processed.source_port, new_package.processed.destination_address, new_package.processed.destination_port, new_package.processed.protocol, &direction, bytes_and_packs_ptr->array[HASH_ICMP][hash_position]);
		if ( aux_package != NULL ) {
			memcpy(&aux_package->processed.end_time, &ts, sizeof(struct timeval));
			if ((ts.tv_sec != 0) && ((bytes_and_packs_ptr->tmax == -1) || (ts.tv_sec > bytes_and_packs_ptr->tmax)))
				bytes_and_packs_ptr->tmax = ts.tv_sec;
			if (direction == 1) {
				// de origen a destino
				RAISE_INFORMATION(aux_package->processed.source_packages, aux_package->processed.source_packages_g, 1, debug[bytes_and_packs_ptr->position].overflow_source_packages);
				RAISE_INFORMATION(aux_package->processed.source_b, aux_package->processed.source_gb, (length_ip_package - sizeof(struct icmp)), debug[bytes_and_packs_ptr->position].overflow_source);
			}
			else if (direction == -1) {
				// de destino a origen
				RAISE_INFORMATION(aux_package->processed.destination_packages, aux_package->processed.destination_packages_g, 1, debug[bytes_and_packs_ptr->position].overflow_destination_packages);
				RAISE_INFORMATION(aux_package->processed.destination_b, aux_package->processed.destination_gb, (length_ip_package - sizeof(struct icmp)), debug[bytes_and_packs_ptr->position].overflow_destination);
			}
			free_package(package, bytes_and_packs_ptr);
		}
		else {
			// Es el primer paquete que nos llega del flujo...
			new_package.processed.flow_state = ICMP_DATA;
			memcpy(&new_package.processed.start_time, &ts, sizeof(struct timeval));
			if ((ts.tv_sec != 0) && ((bytes_and_packs_ptr->tmin == -1) || (ts.tv_sec < bytes_and_packs_ptr->tmin)))
				bytes_and_packs_ptr->tmin = ts.tv_sec;
			memcpy(&new_package.processed.end_time, &ts, sizeof(struct timeval));
			if ((ts.tv_sec != 0) && ((bytes_and_packs_ptr->tmax == -1) || (ts.tv_sec > bytes_and_packs_ptr->tmax)))
				bytes_and_packs_ptr->tmax = ts.tv_sec;
			new_package.processed.source_b = length_ip_package - sizeof(struct udphdr);
			new_package.processed.source_packages = 1;
			memcpy(package, &new_package, sizeof(memory_element));
			if (bytes_and_packs_ptr->array[HASH_ICMP][hash_position] != NULL) {
				package->next = bytes_and_packs_ptr->array[HASH_ICMP][hash_position];
			}
			bytes_and_packs_ptr->array[HASH_ICMP][hash_position] = package;
		}
	}
	else {
		// Conexion no TCP, no UDP o no ICMP
		new_package.processed.flow_state = UNDEFINED;
		memcpy(&new_package.processed.start_time, &ts, sizeof(struct timeval));
		if ((ts.tv_sec != 0) && ((bytes_and_packs_ptr->tmin == -1) || (ts.tv_sec < bytes_and_packs_ptr->tmin)))
			bytes_and_packs_ptr->tmin = ts.tv_sec;
		memcpy(&new_package.processed.end_time, &ts, sizeof(struct timeval));
		if ((ts.tv_sec != 0) && ((bytes_and_packs_ptr->tmax == -1) || (ts.tv_sec > bytes_and_packs_ptr->tmax)))
			bytes_and_packs_ptr->tmax = ts.tv_sec;
		new_package.processed.source_b = length_ip_package;
		new_package.processed.source_packages = 1;
		memcpy(package, &new_package, sizeof(memory_element));
		if (bytes_and_packs_ptr->array[HASH_OTHERS][hash_position] != NULL) {
			package->next = bytes_and_packs_ptr->array[HASH_OTHERS][hash_position];
		}
		bytes_and_packs_ptr->array[HASH_OTHERS][hash_position] = package;
	}
}

void *processer_thread (void *args)
{
	bytes_and_packets *bytes_and_packs = NULL;
	memory_element *aux_package;
	unsigned long allocated = 0;
	
	bytes_and_packs = (bytes_and_packets *) args;

	while ((bytes_and_packs->first_process != NULL) || (!GET_END_THREADS_PROCESSER)) {

		if (bytes_and_packs->first_process != NULL) {
			pthread_mutex_lock(&bytes_and_packs->mutual_exclusion_access);
			aux_package = bytes_and_packs->first_process;
			if (bytes_and_packs->first_process->next == NULL)
				bytes_and_packs->first_process = bytes_and_packs->last_process = NULL;
			else
				bytes_and_packs->first_process = bytes_and_packs->first_process->next;
			pthread_mutex_unlock(&bytes_and_packs->mutual_exclusion_access);
			process_packet (aux_package, bytes_and_packs);
#ifdef STATISTICS_CAPTURE
			pthread_mutex_lock(&debug[bytes_and_packs->position].mutex_debug);
			if ((debug[bytes_and_packs->position].queued_packets == 0) && (debug[bytes_and_packs->position].queued_packets_g == 0)){
				sprintf(buffer_debug, "Vamos a decrementar y dar valor negativo");
				save_debug (bytes_and_packs->interface, buffer_debug);
			}
			UPDATE_MAX(debug[bytes_and_packs->position].queued_packets, debug[bytes_and_packs->position].queued_packets_g, debug[bytes_and_packs->position].queued_packets_highwatermark, debug[bytes_and_packs->position].queued_packets_highwatermark_g);
			DECREASE_INFORMATION(debug[bytes_and_packs->position].queued_packets, debug[bytes_and_packs->position].queued_packets_g, 1);
			pthread_mutex_unlock(&debug[bytes_and_packs->position].mutex_debug);
#endif
		}
		else {
			sleep(1);
		}
		// Y nos encargamos de alocatar memoria si falta
		if (bytes_and_packs->cnt_free_mem <= min_packets)
			allocated = allocate_memory(bytes_and_packs, min_packets);
#ifdef STATISTICS_CAPTURE
			debug[bytes_and_packs->position].memory_packets_allocated += allocated;
#endif
	}

	// Migramos estas llamadas aqu...
	save_to_disk (fsalida, bytes_and_packs);
	free_all_memory(bytes_and_packs);

#ifdef STATISTICS_CAPTURE
	unsigned long long current_hwm, max_hwm, total_recycled;
	
	current_hwm = GIGA*debug[bytes_and_packs->position].queued_packets_g + debug[bytes_and_packs->position].queued_packets;
	max_hwm = GIGA*debug[bytes_and_packs->position].queued_packets_highwatermark_g + debug[bytes_and_packs->position].queued_packets_highwatermark;
	total_recycled = GIGA*debug[bytes_and_packs->position].packets_recycled_g + debug[bytes_and_packs->position].packets_recycled;
	sprintf(buffer_debug, "Allocated: %lu(%luKB)  Used: %lu  Recycled: %llu  Released: %lu (%lu free | %lu allocated)  Current HWM: %llu  Max HWM: %llu", \
			debug[bytes_and_packs->position].memory_packets_allocated, \
			(debug[bytes_and_packs->position].memory_packets_allocated*sizeof(memory_element)/1024), \
			debug[bytes_and_packs->position].memory_packets_used, \
			total_recycled, debug[bytes_and_packs->position].memory_packets_released, \
			debug[bytes_and_packs->position].memory_packets_released_free, debug[bytes_and_packs->position].memory_packets_released_allocated, \
			current_hwm, max_hwm);
	save_debug (bytes_and_packs->interface, buffer_debug);
#endif

	return NULL;
}

void capturer_thread (u_char *args, const struct pcap_pkthdr *pkthdr, const u_char *network_package)
{
	bytes_and_packets *bytes_and_packs = NULL;
	struct ip *ip_header = NULL;
	struct icmp *icmp_header = NULL;
	struct tcphdr *tcp_header = NULL;
	struct udphdr *udp_header = NULL;
	memory_element *mem_package = NULL;

	bytes_and_packs = (bytes_and_packets *) args;

	if (bytes_and_packs->free != NULL) {
		pthread_mutex_lock(&bytes_and_packs->mutual_exclusion_access);
		mem_package = bytes_and_packs->free;
		if (bytes_and_packs->free->next == NULL)
			bytes_and_packs->free = bytes_and_packs->last_free = NULL;
		else
			bytes_and_packs->free = bytes_and_packs->free->next;
		bytes_and_packs->cnt_free_mem--;
		pthread_mutex_unlock(&bytes_and_packs->mutual_exclusion_access);

		ip_header = (struct ip *) (network_package + ETHER_HDR_LEN);
		memcpy(&mem_package->raw.ip_h, ip_header, sizeof(struct ip));

		switch (ip_header->ip_p) {
			case IPPROTO_ICMP:
				icmp_header = (struct icmp *) (network_package + ETHER_HDR_LEN + IP_PAYLOAD(ip_header));
				memcpy(&mem_package->raw.icmp_h, icmp_header, sizeof(struct icmp));
				break;
			case IPPROTO_TCP:
				tcp_header = (struct tcphdr *) (network_package + ETHER_HDR_LEN + IP_PAYLOAD(ip_header));
				memcpy(&mem_package->raw.tcp_h, tcp_header, sizeof(struct tcphdr));
				break;
			case IPPROTO_UDP:
				udp_header = (struct udphdr *) (network_package + ETHER_HDR_LEN + IP_PAYLOAD(ip_header));
				memcpy(&mem_package->raw.udp_h, udp_header, sizeof(struct udphdr));
				break;
			default:
				// protocol_header_size sera 0 en su valor por defecto (inicializado a 0 anteriormente)
				break;
		}
		memcpy(&mem_package->raw.ts, &pkthdr->ts, sizeof(struct timeval));
		mem_package->next = NULL;

#ifdef STATISTICS_CAPTURE
		pthread_mutex_lock(&debug[bytes_and_packs->position].mutex_debug);
		RAISE_INFORMATION(debug[bytes_and_packs->position].packets_captured, debug[bytes_and_packs->position].packets_captured_g, 1, debug[bytes_and_packs->position].overflow_packets_captured);
		RAISE_INFORMATION(debug[bytes_and_packs->position].captured_b, debug[bytes_and_packs->position].captured_gb, ntohs(ip_header->ip_len), debug[bytes_and_packs->position].overflow_captured);
		RAISE_INFORMATION(debug[bytes_and_packs->position].queued_packets, debug[bytes_and_packs->position].queued_packets_g, 1, debug[bytes_and_packs->position].overflow_queued_packets);
		pthread_mutex_unlock(&debug[bytes_and_packs->position].mutex_debug);
#endif

		pthread_mutex_lock(&bytes_and_packs->mutual_exclusion_access);
		if (bytes_and_packs->last_process != NULL) {
			// Exist other packets
			bytes_and_packs->last_process->next = (memory_element *) mem_package;
		}
		else {
			// First packet
			bytes_and_packs->first_process = (memory_element *) mem_package;
		}
		bytes_and_packs->last_process = (memory_element *) mem_package;
		pthread_mutex_unlock(&bytes_and_packs->mutual_exclusion_access);
	}
	else {
		pthread_mutex_lock(&bytes_and_packs->mutual_exclusion_access);
		RAISE_INFORMATION(bytes_and_packs->dropped_packages, bytes_and_packs->dropped_packages_g, 1, debug[bytes_and_packs->position].overflow_dropped_packages);
		pthread_mutex_unlock(&bytes_and_packs->mutual_exclusion_access);
	}
}

void *init_capturer (void *args)
{
	bytes_and_packets *bytes_and_packs;
	char *errbuf = NULL;
	int pcap_result;

	bytes_and_packs = (bytes_and_packets *) args;

	bytes_and_packs->pcap_descriptor = pcap_open_live(bytes_and_packs->interface, MAX_BYTES_CAPTURE, 1, 100, errbuf);

	if (bytes_and_packs->pcap_descriptor == NULL) {
		sprintf(buffer_debug, "Error en pcap_open_live");
		save_debug (bytes_and_packs->interface, buffer_debug);
		exit(-1);
	}

	pcap_result = pcap_loop (bytes_and_packs->pcap_descriptor, -1, capturer_thread, (u_char *) bytes_and_packs);
	if (pcap_result == -1) {
		sprintf(buffer_debug, "Error en pcap_loop");
		save_debug (bytes_and_packs->interface, buffer_debug);
	}
	else if (pcap_result == -2) {
		// printf("Salimos ok de %s\n", bytes_and_packs->interface);
	}

	pcap_close(bytes_and_packs->pcap_descriptor);

	return NULL;
}

int save_to_disk (char *file_name, bytes_and_packets *bytes_and_packs)
{
	int fd, fd2;
	unsigned long i;
	unsigned int k;
	memory_element *package_aux = NULL;
	char line[512];
	char data_file[MAX_LINE];
	char debug_file[MAX_LINE];
	char source_address_str[40];
	char destination_address_str[40];
#ifdef STATISTICS_CAPTURE
	unsigned long connections[MAX_PROTOCOL];
	unsigned long zero_positions[MAX_PROTOCOL];
#endif

	/*
	 Estructura del fichero de salida
	 
	 (1) hora inicio
	 (2) hora fin
	 (3) protocolo
	 (4) host 1 ip
	 (5) host 2 ip
	 (6) host 1 port
	 (7) host 2 port
	 (8) host 1 bytes received
	 (9) host 2 bytes received
	 (10) host 1 packets received (no tendra sentido en flujos no tcp/udp)
	 (11) host 2 packets received (no tendra sentido en flujos no tcp/udp)
	*/

	sprintf(data_file, "%s.%s.tmp", file_name, bytes_and_packs->interface);
	sprintf(debug_file, "%s.%s.debug", file_name, bytes_and_packs->interface);
	fd = open (data_file, O_WRONLY|O_CREAT|O_APPEND);
	fd2 = open (debug_file, O_WRONLY|O_CREAT|O_APPEND);

#ifdef STATISTICS_CAPTURE
	i = 0;
	for (i=0; i<MAX_PROTOCOL; i++) zero_positions[i] = connections[i] = 0;
#endif

	i = 0;
	while ( i < max_hash ) {
		for ( k = 0 ; k < MAX_PROTOCOL ; k++ ) {
			package_aux = bytes_and_packs->array[k][i];
#ifdef STATISTICS_CAPTURE
			if (package_aux == NULL) zero_positions[k]++;
#endif
			while (package_aux != NULL) {
				//dist[k]++;

				strcpy(source_address_str, inet_ntoa(package_aux->processed.source_address));
				strcpy(destination_address_str, inet_ntoa(package_aux->processed.destination_address));

				// En el caso que no sea un paquete tcp, printeamos lo que nos interese...
				// De host1 a host2, por lo que se cuentan los bytes recibidos en el host2
				sprintf(line, "%lu.%lu %lu.%lu %d %s %s %u %u %llu %llu %llu %llu\n", \
							  // (1)   (2)   (3) (4)(5)(6) (7) (8)  (9)  (10) (11)
						(unsigned long)package_aux->processed.start_time.tv_sec, (unsigned long)package_aux->processed.start_time.tv_usec, \
						(unsigned long)package_aux->processed.end_time.tv_sec, (unsigned long)package_aux->processed.end_time.tv_usec, \
						(int)package_aux->processed.protocol, source_address_str, destination_address_str, \
						package_aux->processed.source_port, package_aux->processed.destination_port, \
						(unsigned long long) GIGA*package_aux->processed.source_gb + package_aux->processed.source_b, \
						(unsigned long long) GIGA*package_aux->processed.destination_gb + package_aux->processed.destination_b, \
						(unsigned long long) GIGA*package_aux->processed.source_packages_g + package_aux->processed.source_packages, \
						(unsigned long long) GIGA*package_aux->processed.destination_packages_g + package_aux->processed.destination_packages);
				write(fd, line, strlen(line));
				package_aux = package_aux->next;

#ifdef STATISTICS_CAPTURE
				connections[k]++;
#endif
				// used packet
				debug[bytes_and_packs->position].memory_packets_used++;
			}
		}
		i++;
	}

	// Salvamos la informacion de tmin y tmax
	sprintf(line, "%li\n%li\n", (long) bytes_and_packs->tmin, (long) bytes_and_packs->tmax);
	write(fd2, line, strlen(line));

	if (stopped_by_user) {
		sprintf(buffer_debug, "STOPPED by user");
		save_debug (bytes_and_packs->interface, buffer_debug);
	}

#ifdef STATISTICS_CAPTURE
	float bytes_1, dropped_pkgs;
	char overflows[300]="";

	dropped_pkgs = (float)GIGA * (float) bytes_and_packs->dropped_packages_g + (float) bytes_and_packs->dropped_packages;
	bytes_1 = (float) ((float) debug[bytes_and_packs->position].captured_gb * (float) GIGA + (float) debug[bytes_and_packs->position].captured_b) / (float) (1024 * 1024 * 60 * capture_time);
	sprintf(buffer_debug, "(CAPT) MB/s: %.4f\tPACKS/s: %lu\tDROP: %.1f", bytes_1, debug[bytes_and_packs->position].packets_captured/(60*capture_time), dropped_pkgs);
	save_debug (bytes_and_packs->interface, buffer_debug);

	bytes_1 = (float) ((float) debug[bytes_and_packs->position].processed_gb * (float) GIGA + (float) debug[bytes_and_packs->position].processed_b) / (float) (1024 * 1024 * 60 * capture_time);
	sprintf(buffer_debug, "(PROC) MB/s: %.4f\tPACKS/s: %lu\tDROP: %.1f", bytes_1, debug[bytes_and_packs->position].packets_processed/(60*capture_time), dropped_pkgs);
	save_debug (bytes_and_packs->interface, buffer_debug);

	sprintf(buffer_debug, "[CONNECTIONS] TCP: %lu, UDP: %lu, ICMP: %lu, OTHERS: %lu", connections[0], connections[1], connections[2], connections[3]);
	save_debug (bytes_and_packs->interface, buffer_debug);

	sprintf(buffer_debug, "[ZERO FROM %lu] TCP: %lu, UDP: %lu, ICMP: %lu, OTHERS: %lu", max_hash, zero_positions[0], zero_positions[1], zero_positions[2], zero_positions[3]);
	save_debug (bytes_and_packs->interface, buffer_debug);

	if (debug[bytes_and_packs->position].overflow_captured != 0) sprintf(overflows,"captured ov:%lu ", debug[bytes_and_packs->position].overflow_captured);
	if (debug[bytes_and_packs->position].overflow_processed != 0) sprintf(overflows,"processed ov:%lu ", debug[bytes_and_packs->position].overflow_processed);
	if (debug[bytes_and_packs->position].overflow_packets_captured != 0) sprintf(overflows,"packs captured ov:%lu ", debug[bytes_and_packs->position].overflow_packets_captured);
	if (debug[bytes_and_packs->position].overflow_packets_processed != 0) sprintf(overflows,"packs processed ov:%lu ", debug[bytes_and_packs->position].overflow_packets_processed);
	if (debug[bytes_and_packs->position].overflow_packets_recycled != 0) sprintf(overflows,"recycled ov:%lu ", debug[bytes_and_packs->position].overflow_packets_recycled);
	if (debug[bytes_and_packs->position].overflow_queued_packets != 0) sprintf(overflows,"queued packs ov:%lu ", debug[bytes_and_packs->position].overflow_queued_packets);
	if (debug[bytes_and_packs->position].overflow_queued_packets_highwatermark != 0) sprintf(overflows,"queued packs hwm ov:%lu ", debug[bytes_and_packs->position].overflow_queued_packets_highwatermark);
	if (debug[bytes_and_packs->position].overflow_source != 0) sprintf(overflows,"source ov:%lu ", debug[bytes_and_packs->position].overflow_source);
	if (debug[bytes_and_packs->position].overflow_destination != 0) sprintf(overflows,"destination ov:%lu ", debug[bytes_and_packs->position].overflow_destination);
	if (debug[bytes_and_packs->position].overflow_source_packages != 0) sprintf(overflows,"source pkgs ov:%lu ", debug[bytes_and_packs->position].overflow_source_packages);
	if (debug[bytes_and_packs->position].overflow_destination_packages != 0) sprintf(overflows,"dest pkgs ov:%lu ", debug[bytes_and_packs->position].overflow_destination_packages);
	if (debug[bytes_and_packs->position].overflow_dropped_packages != 0) sprintf(overflows,"drop pkgs ov:%lu ", debug[bytes_and_packs->position].overflow_dropped_packages);
	if (strlen(overflows)>0) {
		sprintf(overflows, "\n");
    	sprintf(buffer_debug, "OVERFLOW(S): %s",overflows);
    	save_debug (bytes_and_packs->interface, buffer_debug);
	}
#endif

	close(fd);
	close(fd2);

	char data_file2[MAX_LINE];
	sprintf(data_file2, "%s.%s.data", file_name, bytes_and_packs->interface);
	rename(data_file, data_file2);

	chown(data_file2, (uid_t) -1, file_gid);
	chmod(data_file2, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
	chown(debug_file, (uid_t) -1, file_gid);
	chmod(debug_file, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);

	return 0;
}
