#include <defines.h>
#include <listener.h>
#include <capturer_thread.h>
#include <common.h>

pthread_t *pthreads_capturer;
pthread_t *pthreads_processer;

int end_threads;
int end_threads_processer;
unsigned int stopped_by_user;

extern unsigned int max_interfaces;

extern char buffer_debug[MAX_BUFFER];

extern char fsalida[MAX_BUFFER];
extern bytes_and_packets *iface_information;
extern char *captured_directory;
extern int file_gid;

#ifdef STATISTICS_CAPTURE
extern debug_str *debug;
#endif

int listener_process (void)
{
	sigset_t set;
	int sig;
	unsigned int i;
	time_t end_time;

	SET_END_THREADS(0);
	SET_END_THREADS_PROCESSER(0);

	// Nos aseguramos que los signals no nos molesten (excepto los indispensables)
	sigemptyset( &set );
	sigaddset( &set, SIGUSR1 );	sigaddset( &set, SIGKILL ); sigaddset( &set, SIGTERM );
	if (pthread_sigmask( SIG_BLOCK, &set, NULL ) != 0) {
		sprintf(buffer_debug, "Error en pthread_sigmask");
		save_debug ("listener", buffer_debug);
	}

	pthreads_capturer = (pthread_t *) malloc (max_interfaces*sizeof(pthread_t));
	pthreads_processer = (pthread_t *) malloc (max_interfaces*sizeof(pthread_t));

	// Threads creation
	for (i=0; i < max_interfaces; i++) {
		if (pthread_create(&pthreads_capturer[i], NULL, init_capturer, (void *) &iface_information[i]) != 0) {
			sprintf(buffer_debug, "Error en la creacion del thread encargado de la captura de la interface");
			save_debug ("listener", buffer_debug);
		}

		if (pthread_create(&pthreads_processer[i], NULL, processer_thread, (void *) &iface_information[i]) != 0) {
			sprintf(buffer_debug, "Error en la creacion del thread encargado del procesado de la interface");
			save_debug ("listener", buffer_debug);
		}
	}

	sigwait( &set, &sig );

	if ((sig == SIGTERM) || (sig == SIGKILL)) stopped_by_user = 1;
	else stopped_by_user = 0;

	SET_END_THREADS(1);

	time(&end_time);

	for (i = 0; i < max_interfaces; i++) {
		pcap_breakloop(iface_information[i].pcap_descriptor);
	}

	// Para que no se quede bloqueado
	// system("ping -c 1 -q 127.0.0.1 >/dev/null");

	create_file_system(captured_directory, fsalida, TYPE_CAPTURER, 1, 1, end_time, (uid_t) -1, file_gid);

	// Y esperamos a los hijos
	for (i = 0; i < max_interfaces; i++) {
		pthread_join(pthreads_capturer[i], NULL);
	}

	SET_END_THREADS_PROCESSER(1);

	// Y ahora esperamos a los hijos que acaben de procesar...
	for (i = 0; i < max_interfaces; i++) {
		pthread_join(pthreads_processer[i], NULL);
		pthread_mutex_destroy(&iface_information[i].mutual_exclusion_access);
#ifdef STATISTICS_CAPTURE
		pthread_mutex_destroy(&debug[i].mutex_debug);
#endif
	}

	return 0;
}
