#include <defines.h>
#include <config_file.h>
#include <listener.h>
#include <capturer_thread.h>
#include <common.h>

unsigned int max_interfaces;
bytes_and_packets *iface_information;
char buffer_debug[MAX_BUFFER];

unsigned long max_packets;
unsigned long min_packets;
unsigned long max_hash;
unsigned int capture_time;
char *listener_pid;
char *captured_directory;
char *processed_directory;
char *listener_log_file;
int file_gid;

#ifdef STATISTICS_CAPTURE
debug_str *debug;
#endif

int parse_interfaces (char *interfaces)
{
	unsigned int i;
	int is_valid;
	char characters[2]=",";
	char *result = NULL;
	char *errbuf = NULL;
	char interfaces_copy[MAX_STRING_IFACES];
	pcap_if_t *alldevsp;
	pcap_if_t *alldevsp_copy;

	if (pcap_findalldevs(&alldevsp, errbuf) != 0) {
		sprintf(buffer_debug, "Error in pcap_findalldevs");
		save_debug ("main", buffer_debug);
		return -1;
	}

	strcpy(interfaces_copy, interfaces);

	result = strtok(interfaces_copy, characters);

	i = 0;

	while (result != NULL) {
		is_valid = 0;
		alldevsp_copy = alldevsp;
		while ((!is_valid) && (alldevsp_copy != NULL)) {
			if ( strcmp(alldevsp_copy->name,result) == 0 ) {
				i++;
				is_valid = 1;
			}
			else alldevsp_copy = alldevsp_copy->next;
		}
		result = strtok(NULL, characters);
	}
	max_interfaces = i;

	iface_information = (bytes_and_packets *) malloc(max_interfaces*sizeof(bytes_and_packets));

	strcpy(interfaces_copy, interfaces);

	result = strtok(interfaces_copy, characters);

	i = 0;

	while (result != NULL) {
		is_valid = 0;
		alldevsp_copy = alldevsp;
		while ((!is_valid) && (alldevsp_copy != NULL)) {
			if ( strcmp(alldevsp_copy->name,result) == 0 ) {
				strcpy(iface_information[i].interface, result);
				iface_information[i].position = i;
				i++;
				is_valid = 1;
			}
			else alldevsp_copy = alldevsp_copy->next;
		}
		result = strtok(NULL, characters);
	}

	// Free the device list
	pcap_freealldevs(alldevsp);

	if ( max_interfaces == 0 ) {
		sprintf(buffer_debug, "Error... No valid network interface found... exiting");
		save_debug ("main", buffer_debug);
		return -2;
	}

	return 0;
}

void initialize_bytes_and_packets (void)
{
	unsigned int i, k;
	unsigned long allocated = 0;

	debug = (debug_str *) malloc (max_interfaces*sizeof(debug_str));

	for (i = 0; i < max_interfaces; i++) {
#ifdef STATISTICS_CAPTURE
		debug[i].captured_gb = 0;
		debug[i].captured_b = 0;
		debug[i].packets_captured_g = 0;
		debug[i].packets_captured = 0;
		debug[i].processed_gb = 0;
		debug[i].processed_b = 0;
		debug[i].packets_processed_g = 0;
		debug[i].packets_processed = 0;
		debug[i].packets_recycled_g = 0;
		debug[i].packets_recycled = 0;
		debug[i].memory_packets_allocated = 0;
		debug[i].memory_packets_released = 0;
		debug[i].memory_packets_released_free = 0;
		debug[i].memory_packets_released_allocated = 0;
		debug[i].memory_packets_used = 0;
		pthread_mutex_init(&debug[i].mutex_debug, NULL);
		debug[i].queued_packets_g	= 0;
		debug[i].queued_packets	= 0;
		debug[i].queued_packets_highwatermark_g = 0;
		debug[i].queued_packets_highwatermark = 0;

		debug[i].overflow_captured = 0;
		debug[i].overflow_packets_captured = 0;
		debug[i].overflow_processed = 0;
		debug[i].overflow_packets_processed = 0;
		debug[i].overflow_packets_recycled = 0;
		debug[i].overflow_queued_packets = 0;	
		debug[i].overflow_queued_packets_highwatermark = 0;
		debug[i].overflow_source = 0;
		debug[i].overflow_destination = 0;
		debug[i].overflow_source_packages = 0;
		debug[i].overflow_destination_packages = 0;
		debug[i].overflow_dropped_packages = 0;
#endif

		pthread_mutex_init(&iface_information[i].mutual_exclusion_access, NULL);
		iface_information[i].free = NULL;
		iface_information[i].last_free = NULL;
		iface_information[i].first_process = NULL;
		iface_information[i].last_process = NULL;
		iface_information[i].cnt_free_mem = 0;
		iface_information[i].dropped_packages_g = 0;
		iface_information[i].dropped_packages = 0;
		iface_information[i].tmin = -1;
		iface_information[i].tmax = -1;

		for (k = 0 ; k < MAX_PROTOCOL ; k++ ) {
			iface_information[i].array[k] = (memory_element **) calloc(max_hash, sizeof(memory_element *));
		}
		allocated = allocate_memory(&iface_information[i], max_packets);
#ifdef STATISTICS_CAPTURE
		debug[i].memory_packets_allocated += allocated;
#endif
	}
}

int not_root (void) {
	int user_uid;
	
	user_uid = (int) geteuid();
	
	if (user_uid != 0) {
		printf("Required root privileges to work. Aborting...\n");
		return 1;
	}
	else
		return 0;
}

void save_debug (char *interface, char *message)
{
	int fd;
	time_t time_debug;
	struct tm *time_str_debug;
	char buffer_string[MAX_LINE];
	char output_file[MAX_LINE];

	strcpy(output_file, listener_log_file);
	fd = open (output_file, O_WRONLY|O_CREAT|O_APPEND);
	if (fd < 0) {
		printf("Error creating log file... aborting\n");
		exit (0);
	}
	else {
		time(&time_debug);
		time_str_debug = localtime(&time_debug);
		sprintf(buffer_string, "[  listener  - %-6s ] | %02d/%02d/%02d %02d:%02d:%02d | %s\n", interface, time_str_debug->tm_mday, time_str_debug->tm_mon, \
				time_str_debug->tm_year-100, time_str_debug->tm_hour, time_str_debug->tm_min, time_str_debug->tm_sec, message);
		write(fd, buffer_string, strlen(buffer_string));
		close(fd);
		chown(output_file, (uid_t) -1, file_gid);
		chmod(output_file, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
	}
}

int main(int argc, char *argv[])
{
	int fd, pid_hijo, status, exit_main = 0;
	int start_main, stop_main, args_counter, specified_filename, file_descriptor;
	unsigned int i;
	struct stat sb;
	// char command_create_dir[MAX_BUFFER];
	char config_file[MAX_LINE];
	char aux_line[MAX_LINE];
	unsigned long memory_size;

	if (not_root()) return -1;

	start_main = stop_main = 0;
	specified_filename = 0;
	args_counter = 1;
	strcpy(config_file, "./IPSS.conf");

	if ((argc < 2) || (argc > 4)) {
		printf("Error in number of arguments\n  Usage: %s [start|stop] [-c IPSS.conf]\n", argv[0]);
		return -2;
	}
	while (args_counter < argc) {
		if (strcmp(argv[args_counter], "start") == 0) {
			start_main = 1;
		}
		else if (strcmp(argv[args_counter], "stop") == 0) {
			stop_main = 1;
		}
		else if (strcmp(argv[args_counter], "-c") == 0) {
			args_counter++;
			if ((args_counter < argc) && (strlen(argv[args_counter]) < MAX_LINE)) {
					strcpy(config_file, argv[args_counter]);
					specified_filename = 1;
			}
		}
		else {
			printf("Error, unknown option \"%s\"\n", argv[args_counter]);
		}
		args_counter++;
	}
	if ((file_descriptor = open(config_file, O_RDONLY)) < 0) {
		if (specified_filename) {
			printf("Error, filename %s not found\n", config_file);
			return -1;
		}
		else {
			printf("Error, you must specify a configuration file with \"-c\"\n");
			return -1;
		}
	}
	close(file_descriptor);
	if ((start_main) && (stop_main)) {
		printf("Error, you must specify start or stop, not both\n");
		return -1;
	}
	else if ((!start_main) && (!stop_main)) {
		printf("Error, you must specify one action to attend...\n");
		return -1;
	}

	// Llegados a este punto, ya podemos acceder a los datos del fichero de configuracion...
	capture_time = get_capture_time(config_file);
	max_packets = get_max_block_number(config_file);
	min_packets = get_min_block_number(config_file);
	max_hash = get_hash_size_listener(config_file);
	listener_pid = get_listener_pid(config_file);
	captured_directory = get_captured_directory(config_file);
	processed_directory = get_processed_directory(config_file);
	listener_log_file = get_listener_log_file(config_file);
	file_gid = get_gid_group(config_file);
	if (file_gid < 0) {
		printf("Error, group %s inexisting\n", get_group(config_file));
		return -1;
	}

	if(stat(get_home_directory(config_file), &sb) < 0)
	{
		// The path element didn't exist
		mkdir(get_home_directory(config_file), 0770);
		if (stat(get_home_directory(config_file), &sb) < 0) {
			printf("Cann't create home directory %s\n", get_home_directory(config_file));
			return -1;
		}
		chown(get_home_directory(config_file), (uid_t) -1, file_gid);
		chmod(get_home_directory(config_file), S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
	}
	if(!S_ISDIR(sb.st_mode))
	{
		printf("home (%s) is not a directory\n", get_home_directory(config_file));
		return -1;
	}

	// Creamos el fichero de log
	fd = open(listener_log_file, O_RDWR|O_CREAT|O_APPEND);
	if (fd < 0) {
		printf("Cann't create log file %s.. exiting\n", listener_log_file);
		return -1;
	}
	close(fd);
	chown(listener_log_file, (uid_t) -1, file_gid);
	chmod(listener_log_file, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);

	if (stop_main) {
		file_descriptor = open(listener_pid, O_RDONLY);
		if (file_descriptor < 0) {
			sprintf(buffer_debug, "Error, pid file doesn't exist");
			save_debug("father", buffer_debug);
		}
		else {
			char pid_buffer[40], *pids, characters[2]=" ";

			read(file_descriptor, pid_buffer, 40);
			pids = strtok(pid_buffer, characters);
			kill(atoi(pids), SIGTERM);
			pids = strtok(NULL, characters);
			kill(atoi(pids), SIGTERM);
			close(file_descriptor);
			unlink(listener_pid);
		}
	}
	else if (start_main) {
		file_descriptor = open(listener_pid, O_RDONLY);
		if (file_descriptor >= 0) {
			sprintf(buffer_debug, "Error, another process running ? %s not started", argv[0]);
			save_debug ("father", buffer_debug);
			close(file_descriptor);
			return -1;
		}

		int detach_pid;

		detach_pid = fork();
		if (detach_pid == 0) {

			if (parse_interfaces(get_interfaces(config_file)) < 0) return -2;

			// We must save interfaces in a file located on $captured_directory/.interfaces
			//  (but before, we must create $captured_directory)
			sprintf(aux_line, "mkdir -p %s", captured_directory);
			system(aux_line);
			mkdir(captured_directory, 0770);
			if (stat(captured_directory, &sb) < 0) {
				printf("Can not create captured_directory %s\n", captured_directory);
				return -1;
			}
			chown(captured_directory, (uid_t) -1, file_gid);
			chmod(captured_directory, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
			// We must, also, correct processed_directory folder permissions
			sprintf(aux_line, "mkdir -p %s", processed_directory);
			system(aux_line);
			chown(processed_directory, (uid_t) -1, file_gid);
			chmod(processed_directory, S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
			// And now, we can save the interfaces_list file 
			sprintf(aux_line, "%s/.interfaces", captured_directory);
			file_descriptor = open(aux_line, O_WRONLY|O_CREAT|O_TRUNC);
			if (file_descriptor < 0) {
				sprintf(buffer_debug, "Error creating interfaces file on %s", captured_directory);
				save_debug ("father", buffer_debug);
				return -1;
			}
			else {
				for (i = 0; i < max_interfaces ; i++) {
					sprintf(aux_line, "%s\n", iface_information[i].interface);
					write(file_descriptor, aux_line, strlen(aux_line));
				}
				close(file_descriptor);
				sprintf(aux_line, "%s/.interfaces", captured_directory);
				chown(aux_line, (uid_t) -1, file_gid);
				chmod(aux_line, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
			}

			initialize_bytes_and_packets();

			sprintf(buffer_debug, "Starting listener (v%s) [", VERSION);
			for (i=0; i<max_interfaces; i++) {
				strcat(buffer_debug, iface_information[i].interface);
				if ((i+1) < max_interfaces) strcat(buffer_debug, ", ");
			}
			memory_size = ((max_packets * sizeof(memory_element)) + sizeof(bytes_and_packets) + \
						   (NUMBER_OF_HASHES * max_hash * sizeof (memory_element *)))/1024;
			sprintf(buffer_debug,"%s] x %luKB (capture time: %d minutes)", buffer_debug, memory_size, capture_time);
			save_debug ("father", buffer_debug);

			// Main while
			while (!exit_main) {
				pid_hijo = fork();
				if (pid_hijo > 0) {
					// We are on father code

					// We must save pid from father and child on file
					file_descriptor = open(listener_pid, O_WRONLY | O_CREAT | O_TRUNC);
					if (file_descriptor < 0) {
						sprintf(buffer_debug,"Cannot save pids on %s", listener_pid);
						save_debug("father", buffer_debug);
					}
					else {
						char pid_buffer[40];
						sprintf(pid_buffer, "%i %i", getpid(), pid_hijo);
						write(file_descriptor, pid_buffer, strlen(pid_buffer));
						close(file_descriptor);
						chown(listener_pid, (uid_t) -1, file_gid);
						chmod(listener_pid, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
					}

					// We must wait the capture time stablished on defines_lite.h
					sleep (45);
					// Wait for the child to finish without blocking...
					waitpid(-1, &status, WNOHANG);

					sleep ((capture_time-1)*60+15);

					// We must send SIGUSR1 signal to child to finish
					if ( kill (pid_hijo, SIGUSR1) < 0 ) {
						sprintf(buffer_debug, "Error sending SIGUSR1 kill signal");
						save_debug ("father", buffer_debug);
						exit (-1);
					}

					// Wait for the child to finish without blocking...
					waitpid(-1, &status, WNOHANG);
				}
				else if (pid_hijo == 0) {
					// We are on child code... we must call listener process and exit
					listener_process();
					exit_main = 1;
				}
				else {
					// Error in fork... we must exit
					exit_main = 1;
				}
			}
		}
	}

	return 0;
}
