#include <defines.h>
#include <config_file.h>
#include <viewer.h>
#include <common.h>

char buffer_debug[MAX_LINE]; 

int capture_time;
long unfinished_connections_seconds;
unsigned long max_packets;
unsigned long min_packets;
unsigned long max_hash;
int keep_days_captured;
char *captured_directory;
char *processed_directory;
int file_gid;
char *compress_bin_path;
char *uncompress_bin_path;
char *gnuplot_bin_path;
int top_n;
char *local_network;

int is_interface_correct (char *interface)
{
	int is_valid;

	int file_descriptor;
	char aux_line[MAX_LINE], rline[MAX_LINE];
	int fd_result;

	sprintf(aux_line, "%s/.interfaces", captured_directory);
	file_descriptor = open (aux_line, O_RDONLY);
	if (file_descriptor < 0) {
		printf("Information about interfaces not available on capture directory %s\n", aux_line);
		exit(1);
	}

	fd_result = read_line(file_descriptor, rline);

	is_valid = 0;
	while ((fd_result >= 0) || (strlen(rline) > 0)) {
		remove_blanks (rline);
		if (strcmp(interface, rline) == 0)
			is_valid = 1;
		fd_result = read_line(file_descriptor, rline);
	}
	close (fd_result);

	if (is_valid) return 1;
	else return 0;
}

int is_date_correct (char *date_string, int *day, int *month, int *year)
{
	// El formato de fecha correcto debe ser DD/MM/YYYY... lo comprobamos
	int aux_int;
	char aux_line[MAX_LINE], aux_line2[MAX_LINE];
	char *string_ptr;
	char slash_char_sep[2]="/";

	strcpy(aux_line, date_string);

	// Comprobamos el dia...
	string_ptr = strtok (aux_line, slash_char_sep);
	if (string_ptr == NULL) return 0;
	strcpy (aux_line2, string_ptr);
	if (strlen(aux_line2) > 2) return 0;
	aux_int = atoi(aux_line2);
	if ((aux_int < 1) || (aux_int > 31)) return 0;
	else *day = aux_int;

	// Comprobamos el mes...
	string_ptr = strtok (NULL, slash_char_sep);
	if (string_ptr == NULL) return 0;
	strcpy (aux_line2, string_ptr);
	if (strlen(aux_line2) > 2) return 0;
	aux_int = atoi(aux_line2);
	if ((aux_int < 1) || (aux_int > 12)) return 0;
	else *month = aux_int;

	// Comprobamos el anyo...
	string_ptr = strtok (NULL, slash_char_sep);
	if (string_ptr == NULL) return 0;
	strcpy (aux_line2, string_ptr);
	if (strlen(aux_line2) != 4) return 0;
	aux_int = atoi(aux_line2);
	if ((aux_int < 1900) || (aux_int > 2200)) return 0;
	else *year = aux_int;

	return 1;
}

int is_time_correct (char *time_string, int *hour, int *minute)
{
	// El formato de fecha correcto debe ser DD/MM/YYYY... lo comprobamos
	int aux_int;
	char aux_line[MAX_LINE], aux_line2[MAX_LINE];
	char *string_ptr;
	char double_dot_char_sep[2]=":";

	strcpy(aux_line, time_string);

	// Comprobamos la hora...
	string_ptr = strtok (aux_line, double_dot_char_sep);
	if (string_ptr == NULL) return 0;
	strcpy (aux_line2, string_ptr);
	if (strlen(aux_line2) > 2) return 0;
	aux_int = atoi(aux_line2);
	if ((aux_int < 0) || (aux_int > 24)) return 0;
	else *hour = aux_int;

	// Comprobamos los minutos...
	string_ptr = strtok (aux_line, double_dot_char_sep);
	if (string_ptr == NULL) return 0;
	strcpy (aux_line2, string_ptr);
	if (strlen(aux_line2) > 2) return 0;
	aux_int = atoi(aux_line2);
	if ((aux_int < 0) || (aux_int > 60)) return 0;
	else *minute = aux_int;

	return 1;
}

void save_debug (char *interface, char *message)
{
	time_t time_debug;
	struct tm *time_str_debug;

	time(&time_debug);
	time_str_debug = localtime(&time_debug);
	printf("%02d/%02d/%02d %02d:%02d:%02d | %s\n", time_str_debug->tm_mday, time_str_debug->tm_mon, \
			time_str_debug->tm_year-100, time_str_debug->tm_hour, time_str_debug->tm_min, time_str_debug->tm_sec, message);
}

int main(int argc, char *argv[])
{
	int statistics, processed, both, args_counter, file_descriptor;
	int specified_initial_day, specified_final_day, specificied_initial_time, specificied_final_time;
	int specified_config_filename, specified_output_filename, specified_interface;
	struct stat sb;
	char config_file[MAX_LINE];
	char initial_date[MAX_LINE], final_date[MAX_LINE];
	char initial_time[MAX_LINE], final_time[MAX_LINE];
	char interface[MAX_LINE];
	char output_filename[MAX_LINE];

	int initial_day, initial_month, initial_year;
	int final_day, final_month, final_year;

	int initial_hour, initial_minutes;
	int final_hour, final_minutes;

	struct tm initial_time_struct, final_time_struct;

	statistics = processed = both = specified_initial_day = specified_final_day = specified_output_filename = 0;
	specificied_initial_time = specificied_final_time = specified_config_filename = specified_interface = 0;
	args_counter = 1;
	strcpy(config_file, "./IPSS.conf");

	if ((argc < 6) || (argc > 12)) {
		printf("Error in number of arguments\n  Usage: %s [-c IPSS.conf] statistics|processed|both -interface iface_name [-o output_file] -initial_day DD/MM/YYYY [-initial_time HH:MM] -final_day DD/MM/YYYY [-final_time HH:MM]\n", argv[0]);
		return -2;
	}

	while (args_counter < argc) {
		if (strcmp(argv[args_counter], "statistics") == 0) {
			statistics = 1;
		}
		else if (strcmp(argv[args_counter], "processed") == 0) {
			processed = 1;
		}
		else if (strcmp(argv[args_counter], "both") == 0) {
			both = 1;
		}
		else if (strcmp(argv[args_counter], "-c") == 0) {
			args_counter++;
			if ((args_counter < argc) && (strlen(argv[args_counter]) < MAX_LINE)) {
					strcpy(config_file, argv[args_counter]);
					specified_config_filename = 1;
			}
		}
		else if (strcmp(argv[args_counter], "-o") == 0) {
			args_counter++;
			if ((args_counter < argc) && (strlen(argv[args_counter]) < MAX_LINE)) {
					strcpy(output_filename, argv[args_counter]);
					specified_output_filename = 1;
			}
		}
		else if (strcmp(argv[args_counter], "-initial_day") == 0) {
			args_counter++;
			if ((args_counter < argc) && (strlen(argv[args_counter]) < MAX_LINE)) {
					strcpy(initial_date, argv[args_counter]);
					specified_initial_day = 1;
			}
		}
		else if (strcmp(argv[args_counter], "-initial_time") == 0) {
			args_counter++;
			if ((args_counter < argc) && (strlen(argv[args_counter]) < MAX_LINE)) {
					strcpy(initial_time, argv[args_counter]);
					specificied_initial_time = 1;
			}
		}
		else if (strcmp(argv[args_counter], "-final_day") == 0) {
			args_counter++;
			if ((args_counter < argc) && (strlen(argv[args_counter]) < MAX_LINE)) {
					strcpy(final_date, argv[args_counter]);
					specified_final_day = 1;
			}
		}
		else if (strcmp(argv[args_counter], "-final_time") == 0) {
			args_counter++;
			if ((args_counter < argc) && (strlen(argv[args_counter]) < MAX_LINE)) {
					strcpy(final_time, argv[args_counter]);
					specificied_final_time = 1;
			}
		}
		else if (strcmp(argv[args_counter], "-interface") == 0) {
			args_counter++;
			if ((args_counter < argc) && (strlen(argv[args_counter]) < MAX_LINE)) {
					strcpy(interface, argv[args_counter]);
					specified_interface = 1;
			}
		}
		else {
			printf("Error, unknown option \"%s\"\n", argv[args_counter]);
		}
		args_counter++;
	}

	if ((file_descriptor = open(config_file, O_RDONLY)) < 0) {
		if (specified_config_filename) {
			printf("Error, filename %s not found\n", config_file);
			return -1;
		}
		else {
			printf("Error, you must specify a configuration file with \"-c\"\n");
			return -1;
		}
	}
	close(file_descriptor);

	captured_directory = get_captured_directory(config_file);
	processed_directory = get_processed_directory(config_file);
	compress_bin_path = get_compress_path(config_file);
	uncompress_bin_path = get_uncompress_path(config_file);
	gnuplot_bin_path = get_gnuplot_path(config_file);
	top_n = get_top_n (config_file);
	local_network = get_local_network (config_file);

	if (((statistics) && (processed)) || ((statistics) && (both)) || \
		((processed) && (both))) {
		printf("Error, you must specify \"statistics\", \"processed\" or \"both\"... not any combination of them\n");
		return -1;
	}
	else if ((!statistics) && (!processed) && (!both)) {
		printf("Error, you must specify one action to attend...\n");
		return -1;
	}
	if ((!specified_initial_day) || (!specified_final_day)) {
		printf("Error, you must specify both initial_day and final_day\n");
		return -1;
	}
	if (!specified_interface) {
		printf("Error, you must specify an interface to process\n");
		return -1;
	}
	// Comprobamos la validez de ciertos campos...
	// Fechas...
	if (!is_date_correct (initial_date, &initial_day, &initial_month, &initial_year)) {
		printf("Error, initial date format not correct\n");
		return -1;
	}
	if (!is_date_correct (final_date, &final_day, &final_month, &final_year)) {
		printf("Error, final date format not correct\n");
		return -1;
	}
	// Horas...
	if ((specificied_initial_time) && (!is_time_correct (initial_time, &initial_hour, &initial_minutes))) {
		printf("Error, initial time format not correct\n");
		return -1;
	}
	if ((specificied_final_time) && (!is_time_correct (final_time, &final_hour, &final_minutes))) {
		printf("Error, final time format not correct\n");
		return -1;
	}
	// Interficie de red...
	if (!is_interface_correct (interface)) {
		printf("Error, interface not available\n");
		return -1;
	}

	// Y acabamos de rellenar los valores optativos
	if (!specificied_initial_time) {
		initial_hour = 0;
		initial_minutes = 0;
	}
	if (!specificied_final_time) {
		final_hour = 23;
		final_minutes = 59;
	}
	if (!specified_output_filename) {
		// si no especificado, creamos el nombre del fichero de salida con una combinacion de fechas/horas
		sprintf(output_filename,"From_%02d-%02d-%04d_%02d:%02d_To_%02d-%02d-%04d_%02d:%02d", initial_day, initial_month, initial_year, \
				initial_hour, initial_minutes, final_day, final_month, final_year, final_hour, final_minutes);
	}

	// Y tambien que las fechas sean coherentes...
	if ((initial_year > final_year) || \
		((initial_year == final_year) && (initial_month > final_month)) || \
		((initial_year == final_year) && (initial_month == final_month) && (initial_day > final_day)) || \
		((initial_year == final_year) && (initial_month == final_month) && (initial_day == final_day) && (initial_hour > final_hour)) || \
		((initial_year == final_year) && (initial_month == final_month) && (initial_day == final_day) && (initial_hour == final_hour) && (initial_minutes >= final_minutes)))
	{
		printf("Error... initial time %02d/%02d/%04d %02d:%02d is greater than final %02d/%02d/%04d %02d:%02d\n", \
			   initial_day, initial_month, initial_year, initial_hour, initial_minutes, \
			   final_day, final_month, final_year, final_hour, final_minutes);
		return -1;
	}

	file_gid = get_gid_group(config_file);
	if (file_gid < 0) {
		printf("Error, group %s inexisting\n", get_group(config_file));
		return -1;
	}

	if(stat(get_home_directory(config_file), &sb) < 0)
	{
		// The path element didn't exist
		mkdir(get_home_directory(config_file), 0770);
		if (stat(get_home_directory(config_file), &sb) < 0) {
			printf("Cann't create home directory %s\n", get_home_directory(config_file));
			return -1;
		}
		chown(get_home_directory(config_file), (uid_t) -1, file_gid);
		chmod(get_home_directory(config_file), S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IWGRP|S_IXGRP);
	}
	if(!S_ISDIR(sb.st_mode))
	{
		printf("home (%s) is not a directory\n", get_home_directory(config_file));
		return -1;
	}

	capture_time = get_capture_time(config_file);
	unfinished_connections_seconds = get_unfinished_connections_time (config_file) * 60;
	max_packets = get_max_block_number(config_file);
	min_packets = get_min_block_number(config_file);
	max_hash = get_hash_size_flow_adder(config_file);
	keep_days_captured = get_keep_days (config_file);

	// Tiempos...
	memset(&initial_time_struct, '\0', sizeof(struct tm));
	initial_time_struct.tm_year = initial_year - 1900;
	initial_time_struct.tm_mon = initial_month - 1;
	initial_time_struct.tm_mday = initial_day;
	initial_time_struct.tm_hour = initial_hour;
	initial_time_struct.tm_min = initial_minutes;
	initial_time_struct.tm_sec = 0;

	memset(&final_time_struct, '\0', sizeof(struct tm));
	final_time_struct.tm_year = final_year - 1900;
	final_time_struct.tm_mon = final_month - 1;
	final_time_struct.tm_mday = final_day;
	final_time_struct.tm_hour = final_hour;
	final_time_struct.tm_min = final_minutes;
	final_time_struct.tm_sec = 59;

	if (statistics) {
		// Generaremos las estadisticas, el fichero para el gnuplot y el fichero de graficas
		generate_statistics_viewer (output_filename, interface, initial_time_struct, final_time_struct);
	}
	else if (processed) {
	}
	else if (both) {
	}

	return 0;
}
