#include <defines.h>
#include <viewer.h>
#include <flow_adder.h>
#include <statistics.h>
#include <common.h>

#include <sys/types.h>
#include <dirent.h>

extern char buffer_debug[MAX_LINE];

extern int capture_time;
extern long unfinished_connections_seconds;
extern unsigned long max_packets;
extern unsigned long min_packets;
extern unsigned long max_hash;
extern int keep_days_captured;
extern char *captured_directory;
extern char *processed_directory;
extern int file_gid;
extern char *compress_bin_path;
extern char *uncompress_bin_path;
extern char *gnuplot_bin_path;
extern int top_n;
extern char *local_network;

extern bytes_and_packets *iface_information;

// bytes_and_packets *iface_information;
// int use_gzip_or_bzip;	// 0 -> gzip  |  1 -> bzip2

viewer_files_struct *add_viewer_element (viewer_files_struct *list, viewer_files_struct *element)
{
	viewer_files_struct *tmp_elem, *first;

	first = tmp_elem = list;
	if (tmp_elem == NULL) {
		tmp_elem = element;
		element->next = NULL;
		return tmp_elem;
	}

	while (tmp_elem->next != NULL)
		tmp_elem = tmp_elem->next;

	tmp_elem->next = element;
	element->next = NULL;
	return first;
}

int path_is_between (char *substring_to_verify, char *initial_string, char *final_string, time_t initial_time, time_t final_time)
{
	// En esta funcion, miraremos si la ruta que nos han pasado est dentro de nuestro intervalo...
	// Formatos de "initial_string" y "final_string": YYYY/MM/WW/DD
	// Formato de "substring_to_verify": YYYY/MM/WW/DD/xxxxxx.??.statistics
	char *string_ptr, aux_line[MAX_LINE], aux_line2[MAX_LINE];
	int initial_year, final_year, actual_year;
	int initial_month, final_month, actual_month;
	int initial_week, final_week, actual_week;
	int initial_day, final_day, actual_day;
	time_t actual_time;
	char slash_char[2]="/";
	char dot_char[2]=".";

	int good_time_begin, good_time_end;

	// **** Parseamos la fecha de substring_to_verify
	strcpy (aux_line, substring_to_verify);

	// ANYO
	string_ptr = strtok(aux_line, slash_char);
	if (string_ptr == NULL) return 0;
	actual_year = atoi(string_ptr);

	// MES
	string_ptr = strtok(NULL, slash_char);
	if (string_ptr == NULL) return 0;
	actual_month = atoi(string_ptr);

	// SEMANA
	string_ptr = strtok(NULL, slash_char);
	if (string_ptr == NULL) return 0;
	actual_week = atoi(string_ptr);

	// DIA
	string_ptr = strtok(NULL, slash_char);
	if (string_ptr == NULL) return 0;
	actual_day = atoi(string_ptr);

	// TIEMPO...
	string_ptr = strtok(NULL, slash_char);
	if (string_ptr == NULL) return 0;
	strcpy(aux_line2, string_ptr);
	string_ptr = strtok(aux_line2, dot_char);
	if (string_ptr == NULL) return 0;
	actual_time = (time_t) atoi(string_ptr);


	// **** Parseamos la fecha de initial_string
	strcpy (aux_line, initial_string);

	// ANYO
	string_ptr = strtok(aux_line, slash_char);
	if (string_ptr == NULL) return 0;
	initial_year = atoi(string_ptr);

	// MES
	string_ptr = strtok(NULL, slash_char);
	if (string_ptr == NULL) return 0;
	initial_month = atoi(string_ptr);

	// SEMANA
	string_ptr = strtok(NULL, slash_char);
	if (string_ptr == NULL) return 0;
	initial_week = atoi(string_ptr);

	// DIA
	string_ptr = strtok(NULL, slash_char);
	if (string_ptr == NULL) return 0;
	initial_day = atoi(string_ptr);

	// TIEMPO...
	// no hace falta... lo tenemos en "initial_time"


	// **** Parseamos la fecha de final_string
	strcpy (aux_line, final_string);

	// ANYO
	string_ptr = strtok(aux_line, slash_char);
	if (string_ptr == NULL) return 0;
	final_year = atoi(string_ptr);

	// MES
	string_ptr = strtok(NULL, slash_char);
	if (string_ptr == NULL) return 0;
	final_month = atoi(string_ptr);

	// SEMANA
	string_ptr = strtok(NULL, slash_char);
	if (string_ptr == NULL) return 0;
	final_week = atoi(string_ptr);

	// DIA
	string_ptr = strtok(NULL, slash_char);
	if (string_ptr == NULL) return 0;
	final_day = atoi(string_ptr);

	// TIEMPO...
	// no hace falta... lo tenemos en "final_time"


	// Y, finalmente, comparamos...
	// El tiempo inicial < tiempo_actual ?
	if (actual_year > initial_year) {
		good_time_begin = 1;
	}
	else if (actual_year < initial_year) {
		good_time_begin = 0;
	}
	else {
		// Comparamos meses
		if (actual_month > initial_month) {
			good_time_begin = 1;
		}
		else if (actual_month < initial_month) {
			good_time_begin = 0;
		}
		else {
			// Comparamos semanas
			if (actual_week > initial_week) {
				good_time_begin = 1;
			}
			else if (actual_week < initial_week) {
				good_time_begin = 0;
			}
			else {
				// Comparamos dias
				if (actual_day > initial_day) {
					good_time_begin = 1;
				}
				else if (actual_day < initial_day) {
					good_time_begin = 0;
				}
				else {
					// Comparamos horas
					if (actual_time >= initial_time) {
						good_time_begin = 1;
					}
					else {
						good_time_begin = 0;
					}
				}
			}
		}
	}

	// El tiempo final > tiempo_actual ?
	if (actual_year < final_year) {
		good_time_end = 1;
	}
	else if (actual_year > final_year) {
		good_time_end = 0;
	}
	else {
		// Comparamos meses
		if (actual_month < final_month) {
			good_time_end = 1;
		}
		else if (actual_month > final_month) {
			good_time_end = 0;
		}
		else {
			// Comparamos semanas
			if (actual_week < final_week) {
				good_time_end = 1;
			}
			else if (actual_week > final_week) {
				good_time_end = 0;
			}
			else {
				// Comparamos dias
				if (actual_day < final_day) {
					good_time_end = 1;
				}
				else if (actual_day > final_day) {
					good_time_end = 0;
				}
				else {
					// Comparamos horas
					if (actual_time <= final_time) {
						good_time_end = 1;
					}
					else {
						good_time_end = 0;
					}
				}
			}
		}
	}

	if ((good_time_begin) && (good_time_end)) return 1;
	else return 0;
}

int generate_statistics_viewer (char *output_file, char *interface, struct tm initial_time_struct, struct tm final_time_struct)
{
	// Haremos un recorrido por los directorios de salida... agregando los .statistics comprendidos dentro del intervalo...
	interface_str interface_in;
	int level;
	long diff_seconds;

	int num_entries_year_p, num_entries_month_p, num_entries_week_p, num_entries_day_p, num_entries_file_p;
	int count_year_p, count_month_p, count_week_p, count_day_p, count_file_p;
    struct dirent **year_list_p, **month_list_p, **week_list_p, **day_list_p, **file_list_p;
    char year_string_p[MAX_LINE], month_string_p[MAX_LINE], week_string_p[MAX_LINE], day_string_p[MAX_LINE], file_string_p[MAX_LINE];
    char aux_line_year_p[MAX_LINE], aux_line_month_p[MAX_LINE], aux_line_week_p[MAX_LINE], aux_line_day_p[MAX_LINE], aux_line_file_p[MAX_LINE];

	time_t initial_time_seconds, final_time_seconds;
	char initial_time_string[MAX_LINE], final_time_string[MAX_LINE];

	struct tm time_struct;
	time_t aux_time;

	char file_name_data[MAX_LINE];
	char network_interface[MAX_LINE];
	char dot_char[2]=".";

	char aux_line[MAX_LINE], aux_line2[MAX_LINE];
	char *string_ptr;

	int fd;

	viewer_files_struct *files_struct;
	viewer_files_struct *aux_element, *aux_element2;
	int interval_valid = 0;
	int level_interval_valid = 0;

	initial_time_seconds = timegm (&initial_time_struct);
	final_time_seconds = timegm (&final_time_struct);

	// Y convertimos este tiempo, en un string parseable y comparable con la estructura de directorios
	// que vamos a recorrer
	create_file_system ("", initial_time_string, TYPE_PROCESSER, 0, 1, initial_time_seconds, (uid_t) -1, file_gid);
	create_file_system ("", final_time_string, TYPE_PROCESSER, 0, 1, final_time_seconds, (uid_t) -1, file_gid);

	strcpy (interface_in.name, interface);

	initialize_bytes_and_packets (1, &interface_in);

	diff_seconds = final_time_seconds - initial_time_seconds;
	if (diff_seconds <= DAY_SECONDS)
		level = LEVEL_SUBDAY;
	else if ((diff_seconds > DAY_SECONDS) && (diff_seconds <= WEEK_SECONDS))
		level = LEVEL_SUBDAY;
	else if ((diff_seconds > WEEK_SECONDS) && (diff_seconds <= MONTH_SECONDS))
		level = LEVEL_DAY;
	else if ((diff_seconds > MONTH_SECONDS) && (diff_seconds <= (6*MONTH_SECONDS)))
		level = LEVEL_WEEK;
	else
		level = LEVEL_MONTH;

	files_struct = NULL;

	if ((num_entries_year_p = scandir(processed_directory, &year_list_p, NULL, alphasort)) < 0) {
		sprintf(buffer_debug, "No files to process...");
		save_debug(interface, buffer_debug);
		return 1;
	}
	if (num_entries_year_p) {
		count_year_p = 0;
		while (count_year_p < num_entries_year_p) {
			strcpy(aux_line_year_p, year_list_p[count_year_p]->d_name);
			sprintf(year_string_p, "%s/%s", processed_directory, aux_line_year_p);
			if ((strcmp(year_list_p[count_year_p]->d_name, ".") != 0) && \
				(strcmp(year_list_p[count_year_p]->d_name, "..") != 0) && \
				(is_a_directory(year_string_p))) {

				if ((num_entries_month_p = scandir(year_string_p, &month_list_p, NULL, alphasort)) < 0) {
					return 1;
				}
				if (num_entries_month_p) {
					count_month_p = 0;
					while (count_month_p < num_entries_month_p) {
						strcpy(aux_line_month_p, month_list_p[count_month_p]->d_name);
						sprintf(month_string_p, "%s/%s", year_string_p, aux_line_month_p);
						if ((strcmp(month_list_p[count_month_p]->d_name, ".") != 0) && \
							(strcmp(month_list_p[count_month_p]->d_name, "..") != 0) && \
							(is_a_directory(month_string_p))) {

							if (level == LEVEL_MONTH)
								level_interval_valid = 0;

							if ((num_entries_week_p = scandir(month_string_p, &week_list_p, NULL, alphasort)) < 0) {
								return 1;
							}
							if (num_entries_week_p) {
								count_week_p = 0;
								while (count_week_p < num_entries_week_p) {
									strcpy(aux_line_week_p, week_list_p[count_week_p]->d_name);
									sprintf(week_string_p, "%s/%s", month_string_p, aux_line_week_p);
									if ((strcmp(week_list_p[count_week_p]->d_name, ".") != 0) && \
										(strcmp(week_list_p[count_week_p]->d_name, "..") != 0) && \
										(is_a_directory(week_string_p))) {

										if (level == LEVEL_WEEK)
											level_interval_valid = 0;

										if ((num_entries_day_p = scandir(week_string_p, &day_list_p, NULL, alphasort)) < 0) {
											return 1;
										}
										if (num_entries_day_p) {
											count_day_p = 0;
											while (count_day_p < num_entries_day_p) {
												strcpy(aux_line_day_p, day_list_p[count_day_p]->d_name);
												sprintf(day_string_p, "%s/%s", week_string_p, aux_line_day_p);
												if ((strcmp(day_list_p[count_day_p]->d_name, ".") != 0) && \
													(strcmp(day_list_p[count_day_p]->d_name, "..") != 0) && \
													(is_a_directory(day_string_p))) {

													if (level == LEVEL_DAY)
														level_interval_valid = 0;

													if ((num_entries_file_p = scandir(day_string_p, &file_list_p, NULL, alphasort)) < 0) {
														return 1;
													}
													if (num_entries_file_p) {
														count_file_p = 0;
														while (count_file_p < num_entries_file_p) {
															strcpy(aux_line_file_p, file_list_p[count_file_p]->d_name);
															sprintf(file_string_p, "%s/%s", day_string_p, aux_line_file_p);
															if ((strcmp(file_list_p[count_file_p]->d_name, ".") != 0) && \
																(strcmp(file_list_p[count_file_p]->d_name, "..") != 0) && \
																(!is_a_directory(file_string_p)) && \
																(strstr(file_list_p[count_file_p]->d_name, "statistics") != NULL)) {

																if (level == LEVEL_SUBDAY)
																	level_interval_valid = 0;

																strcpy(aux_line, aux_line_file_p);
																string_ptr = strtok(aux_line, dot_char);
																strcpy(file_name_data, string_ptr);
																string_ptr = strtok(NULL, dot_char);
																strcpy(network_interface, string_ptr);
																if (strcmp(network_interface, interface) == 0) {
																	// Comprobamos si el intervalo es bueno
																	strcpy(aux_line, (file_string_p+strlen(processed_directory)+1));
																	interval_valid = path_is_between (aux_line, initial_time_string, final_time_string, initial_time_seconds, final_time_seconds);
																	if ((!level_interval_valid) && (interval_valid)) {
																		level_interval_valid = 1;
																	}
																	if (interval_valid) {
																		aux_time = (time_t) atol (file_name_data);
																		gmtime_r (&aux_time, &time_struct);
																	}
																	if ((level_interval_valid) && (level == LEVEL_SUBDAY)) {
																		// En la estructura, guardamos la ruta absoluta
																		aux_element = (viewer_files_struct *) malloc(sizeof(viewer_files_struct));
																		if (aux_element != NULL) {
																			strcpy (aux_element->filename, file_string_p);
																			sprintf(aux_element->time, "%02d/%02d/%04d-%02d:%02d", time_struct.tm_mday, \
																					time_struct.tm_mon+1, time_struct.tm_year+1900, time_struct.tm_hour, \
																					time_struct.tm_min);
																			files_struct = add_viewer_element (files_struct, aux_element);
																		}
																		else {
																			sprintf (buffer_debug, "Error allocating files memory");
																			save_debug (interface, buffer_debug);
																		}
																	}
																}
															}
															free(file_list_p[count_file_p]);
															count_file_p++;
														}
														free(file_list_p);
													}
													if ((level_interval_valid) && (level == LEVEL_DAY)) {
														aux_element = (viewer_files_struct *) malloc(sizeof(viewer_files_struct));
														if (aux_element != NULL) {
															sprintf(aux_line, "%s.%s.statistics", day_string_p, interface);
															strcpy (aux_element->filename, aux_line);
															sprintf(aux_element->time, "%02d/%02d/%04d", time_struct.tm_mday, \
																	time_struct.tm_mon+1, time_struct.tm_year+1900);

															files_struct = add_viewer_element (files_struct, aux_element);
														}
														else {
															sprintf (buffer_debug, "Error allocating files memory");
															save_debug (interface, buffer_debug);
														}
													}
												}
												free(day_list_p[count_day_p]);
												count_day_p++;
											}
											free(day_list_p);
										}
										if ((level_interval_valid) && (level == LEVEL_WEEK)) {
											aux_element = (viewer_files_struct *) malloc(sizeof(viewer_files_struct));
											if (aux_element != NULL) {
												sprintf(aux_line, "%s.%s.statistics", week_string_p, interface);
												strcpy (aux_element->filename, aux_line);
												sprintf(aux_element->time, "%02d/%02d/%04d", time_struct.tm_mday, time_struct.tm_mon+1, time_struct.tm_year+1900);
												files_struct = add_viewer_element (files_struct, aux_element);
											}
											else {
												sprintf (buffer_debug, "Error allocating files memory");
												save_debug (interface, buffer_debug);
											}
										}
									}
									free(week_list_p[count_week_p]);
									count_week_p++;
								}
								free(week_list_p);
							}
							if ((level_interval_valid) && (level == LEVEL_MONTH)) {
								aux_element = (viewer_files_struct *) malloc(sizeof(viewer_files_struct));
								if (aux_element != NULL) {
									sprintf(aux_line, "%s.%s.statistics", month_string_p, interface);
									strcpy (aux_element->filename, aux_line);
									sprintf(aux_element->time, "%02d/%02d", time_struct.tm_mon+1, time_struct.tm_year+1900);
									files_struct = add_viewer_element (files_struct, aux_element);
								}
								else {
									sprintf (buffer_debug, "Error allocating files memory");
									save_debug (interface, buffer_debug);
								}
							}

						}
						free(month_list_p[count_month_p]);
						count_month_p++;
					}
					free(month_list_p);
				}
			}
			free(year_list_p[count_year_p]);
			count_year_p++;
		}
		free(year_list_p);
	}

	aux_element = files_struct;
	int at_least_one = 0;
	while (aux_element != NULL) {
		printf("Processing file %s...\n", aux_element->filename);
		if (at_least_one == 0) {
			// Es el primer elemento que encontramos... creamos los ficheros para el gnuplot...

			// *** CONEXIONES TOTALES
			FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_connections.gnuplot", "# Total connections\n", "#Time\tTotal\tTCP\tUDP\tICMP\tOTHERS\n");
			
			// *** BYTES TOTALES
			FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_bytes.gnuplot", "# Total bytes\n", "#Time\tTotal\tTCP\tUDP\tICMP\tOTHERS\n");
		}
		read_and_process_statistics_file (aux_element->filename, &iface_information[0], MODE_DOUBLE, output_file, aux_element->time);
		aux_element2 = aux_element;
		aux_element = aux_element->next;
		free(aux_element2);
		at_least_one++;
	}

	// Creamos los ficheros para los TOP_N

	// *** TOTAL TOP RECEIVED (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_bytes_received.gnuplot", "# Global IPs with more bandwidth received\n", "#IP\tBandwidth\n");

	// *** TOTAL TOP TCP RECEIVED (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_tcp_bytes_received.gnuplot", "# Global IPs with more bandwidth TCP received\n", "#IP\tBandwidth\n");

	// *** TOTAL TOP UDP RECEIVED (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_udp_bytes_received.gnuplot", "# Global IPs with more bandwidth UDP received\n", "#IP\tBandwidth\n");

	// *** TOTAL TOP ICMP RECEIVED (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_icmp_bytes_received.gnuplot", "# Global IPs with more bandwidth ICMP received\n", "#IP\tBandwidth\n");

	// *** TOTAL TOP OTHERS RECEIVED (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_others_bytes_received.gnuplot", "# Global IPs with more bandwidth OTHERS received\n", "#IP\tBandwidth\n");


	// *** LOCAL TOP RECEIVED (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_bytes_received.gnuplot", "# Local IPs with more bandwidth received\n", "#IP\tBandwidth\n");
	
	// *** LOCAL TOP TCP RECEIVED (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_tcp_bytes_received.gnuplot", "# Local IPs with more bandwidth TCP received\n", "#IP\tBandwidth\n");
	
	// *** LOCAL TOP UDP RECEIVED (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_udp_bytes_received.gnuplot", "# Local IPs with more bandwidth UDP received\n", "#IP\tBandwidth\n");
	
	// *** LOCAL TOP ICMP RECEIVED (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_icmp_bytes_received.gnuplot", "# Local IPs with more bandwidth ICMP received\n", "#IP\tBandwidth\n");
	
	// *** LOCAL TOP OTHERS RECEIVED (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_others_bytes_received.gnuplot", "# Local IPs with more bandwidth OTHERS received\n", "#IP\tBandwidth\n");
	
	
	// *** TOTAL TOP SENT (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_bytes_sent.gnuplot", "# Global IPs with more bandwidth sent\n", "#IP\tBandwidth\n");
	
	// *** TOTAL TOP TCP SENT (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_tcp_bytes_sent.gnuplot", "# Global IPs with more bandwidth TCP sent\n", "#IP\tBandwidth\n");
	
	// *** TOTAL TOP UDP SENT (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_udp_bytes_sent.gnuplot", "# Global IPs with more bandwidth UDP sent\n", "#IP\tBandwidth\n");
	
	// *** TOTAL TOP ICMP SENT (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_icmp_bytes_sent.gnuplot", "# Global IPs with more bandwidth ICMP sent\n", "#IP\tBandwidth\n");
	
	// *** TOTAL TOP OTHERS SENT (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_others_bytes_sent.gnuplot", "# Global IPs with more bandwidth OTHERS sent\n", "#IP\tBandwidth\n");


	// *** LOCAL TOP SENT (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_bytes_sent.gnuplot", "# Local IPs with more bandwidth sent\n", "#IP\tBandwidth\n");
	
	// *** LOCAL TOP TCP SENT (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_tcp_bytes_sent.gnuplot", "# Local IPs with more bandwidth TCP sent\n", "#IP\tBandwidth\n");
	
	// *** LOCAL TOP UDP SENT (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_udp_bytes_sent.gnuplot", "# Local IPs with more bandwidth UDP sent\n", "#IP\tBandwidth\n");
	
	// *** LOCAL TOP ICMP SENT (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_icmp_bytes_sent.gnuplot", "# Local IPs with more bandwidth ICMP sent\n", "#IP\tBandwidth\n");
	
	// *** LOCAL TOP OTHERS SENT (BYTES)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_others_bytes_sent.gnuplot", "# Local IPs with more bandwidth OTHERS sent\n", "#IP\tBandwidth\n");
	
	
	
	
	// *** TOTAL TOP RECEIVED (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_connections_received.gnuplot", "# Global IPs with more connections received\n", "#IP\tConnections\n");
	
	// *** TOTAL TOP TCP RECEIVED (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_tcp_connections_received.gnuplot", "# Global IPs with more connections TCP received\n", "#IP\tConnections\n");
	
	// *** TOTAL TOP UDP RECEIVED (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_udp_connections_received.gnuplot", "# Global IPs with more connections UDP received\n", "#IP\tConnections\n");
	
	// *** TOTAL TOP ICMP RECEIVED (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_icmp_connections_received.gnuplot", "# Global IPs with more connections ICMP received\n", "#IP\tConnections\n");
	
	// *** TOTAL TOP OTHERS RECEIVED (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_others_connections_received.gnuplot", "# Global IPs with more connections OTHERS received\n", "#IP\tConnections\n");
	
	
	// *** LOCAL TOP RECEIVED (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_connections_received.gnuplot", "# Local IPs with more connections received\n", "#IP\tConnections\n");
	
	// *** LOCAL TOP TCP RECEIVED (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_tcp_connections_received.gnuplot", "# Local IPs with more connections TCP received\n", "#IP\tConnections\n");
	
	// *** LOCAL TOP UDP RECEIVED (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_udp_connections_received.gnuplot", "# Local IPs with more connections UDP received\n", "#IP\tConnections\n");
	
	// *** LOCAL TOP ICMP RECEIVED (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_icmp_connections_received.gnuplot", "# Local IPs with more connections ICMP received\n", "#IP\tConnections\n");
	
	// *** LOCAL TOP OTHERS RECEIVED (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_others_connections_received.gnuplot", "# Local IPs with more connections OTHERS received\n", "#IP\tConnections\n");
	
	
	// *** TOTAL TOP SENT (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_connections_sent.gnuplot", "# Global IPs with more connections sent\n", "#IP\tConnections\n");
	
	// *** TOTAL TOP TCP SENT (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_tcp_connections_sent.gnuplot", "# Global IPs with more connections TCP sent\n", "#IP\tConnections\n");
	
	// *** TOTAL TOP UDP SENT (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_udp_connections_sent.gnuplot", "# Global IPs with more connections UDP sent\n", "#IP\tConnections\n");
	
	// *** TOTAL TOP ICMP SENT (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_icmp_connections_sent.gnuplot", "# Global IPs with more connections ICMP sent\n", "#IP\tConnections\n");
	
	// *** TOTAL TOP OTHERS SENT (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_others_connections_sent.gnuplot", "# Global IPs with more connections OTHERS sent\n", "#IP\tConnections\n");
	
	
	// *** LOCAL TOP SENT (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_connections_sent.gnuplot", "# Local IPs with more connections sent\n", "#IP\tConnections\n");
	
	// *** LOCAL TOP TCP SENT (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_tcp_connections_sent.gnuplot", "# Local IPs with more connections TCP sent\n", "#IP\tConnections\n");
	
	// *** LOCAL TOP UDP SENT (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_udp_connections_sent.gnuplot", "# Local IPs with more connections UDP sent\n", "#IP\tConnections\n");
	
	// *** LOCAL TOP ICMP SENT (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_icmp_connections_sent.gnuplot", "# Local IPs with more connections ICMP sent\n", "#IP\tConnections\n");
	
	// *** LOCAL TOP OTHERS SENT (CONNECTIONS)
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_others_connections_sent.gnuplot", "# Local IPs with more connections OTHERS sent\n", "#IP\tConnections\n");
	
	
	
	// *** TOTAL TOP TCP PORTS MORE ACCESSED
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_tcp_ports.gnuplot", "# Total TCP PORTs more accessed\n", "#Port\tTimes\n");
	
	// *** TOTAL TOP UDP PORTS MORE ACCESSED
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "total_top_udp_ports.gnuplot", "# Total UDP PORTs more accessed\n", "#Port\tTimes\n");
	
	// *** LOCAL TOP TCP PORTS MORE ACCESSED
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_tcp_ports.gnuplot", "# Local TCP PORTs more accessed\n", "#Port\tTimes\n");
	
	// *** LOCAL TOP UDP PORTS MORE ACCESSED
	FILE_HEAD_INITIALISE(fd, file_gid, aux_line2, output_file, interface, "local_top_udp_ports.gnuplot", "# Local UDP PORTs more accessed\n", "#Port\tTimes\n");


	if (at_least_one) save_statistics (output_file, &iface_information[0], MODE_DOUBLE, output_file);
	else printf("No files on this interval\n");


	char graphic_string[MAX_LINE];
	char tmp_graphic_file[MAX_LINE];
	char filename_gnuplot[MAX_LINE];
	int fd_graph;

	// Generamos la grafica para las conexiones totales
	sprintf(filename_gnuplot, "%s.%s.total_connections", output_file, interface);
	GNUPLOT_STATISTICS_ACCUMULATIVE(fd_graph, tmp_graphic_file, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
									initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, \
									initial_time_struct.tm_min, final_time_struct.tm_mday, final_time_struct.tm_mon+1, \
									final_time_struct.tm_year+1900, final_time_struct.tm_hour, final_time_struct.tm_min, \
									aux_line, aux_line2, gnuplot_bin_path, "connections", "Connections");

	// Generamos la grafica para los bytes totales
	sprintf(filename_gnuplot, "%s.%s.total_bytes", output_file, interface);
	GNUPLOT_STATISTICS_ACCUMULATIVE(fd_graph, tmp_graphic_file, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
									initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, \
									initial_time_struct.tm_min, final_time_struct.tm_mday, final_time_struct.tm_mon+1, \
									final_time_struct.tm_year+1900, final_time_struct.tm_hour, final_time_struct.tm_min, \
									aux_line, aux_line2, gnuplot_bin_path, "bytes", "Bytes");

	// Generamos la grafica para los bytes totales recibidos
	sprintf(filename_gnuplot, "%s.%s.total_top_bytes_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total", "Bytes", "bytes", "received", "IP\'s");

	// Generamos la grafica para los bytes TCP totales recibidos
	sprintf(filename_gnuplot, "%s.%s.total_top_tcp_bytes_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total TCP", "Bytes", "bytes", "received", "IP\'s");

	// Generamos la grafica para los bytes UDP totales recibidos
	sprintf(filename_gnuplot, "%s.%s.total_top_udp_bytes_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total UDP", "Bytes", "bytes", "received", "IP\'s");

	// Generamos la grafica para los bytes ICMP totales recibidos
	sprintf(filename_gnuplot, "%s.%s.total_top_icmp_bytes_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total ICMP", "Bytes", "bytes", "received", "IP\'s");

	// Generamos la grafica para los bytes OTHERS totales recibidos
	sprintf(filename_gnuplot, "%s.%s.total_top_others_bytes_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total OTHERS", "Bytes", "bytes", "received", "IP\'s");

	// Generamos la grafica para los bytes locales recibidos
	sprintf(filename_gnuplot, "%s.%s.local_top_bytes_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local", "Bytes", "bytes", "received", "IP\'s");

	// Generamos la grafica para los bytes TCP locales recibidos
	sprintf(filename_gnuplot, "%s.%s.local_top_tcp_bytes_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local TCP", "Bytes", "bytes", "received", "IP\'s");

	// Generamos la grafica para los bytes UDP locales recibidos
	sprintf(filename_gnuplot, "%s.%s.local_top_udp_bytes_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local UDP", "Bytes", "bytes", "received", "IP\'s");

	// Generamos la grafica para los bytes ICMP locales recibidos
	sprintf(filename_gnuplot, "%s.%s.local_top_icmp_bytes_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local ICMP", "Bytes", "bytes", "received", "IP\'s");

	// Generamos la grafica para los bytes OTHERS locales recibidos
	sprintf(filename_gnuplot, "%s.%s.local_top_others_bytes_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local OTHERS", "Bytes", "bytes", "received", "IP\'s");


	// Generamos la grafica para los bytes totales enviados
	sprintf(filename_gnuplot, "%s.%s.total_top_bytes_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total", "Bytes", "bytes", "sent", "IP\'s");

	// Generamos la grafica para los bytes TCP totales enviados
	sprintf(filename_gnuplot, "%s.%s.total_top_tcp_bytes_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total TCP", "Bytes", "bytes", "sent", "IP\'s");

	// Generamos la grafica para los bytes UDP totales enviados
	sprintf(filename_gnuplot, "%s.%s.total_top_udp_bytes_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total UDP", "Bytes", "bytes", "sent", "IP\'s");

	// Generamos la grafica para los bytes ICMP totales enviados
	sprintf(filename_gnuplot, "%s.%s.total_top_icmp_bytes_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total ICMP", "Bytes", "bytes", "sent", "IP\'s");

	// Generamos la grafica para los bytes OTHERS totales enviados
	sprintf(filename_gnuplot, "%s.%s.total_top_others_bytes_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total OTHERS", "Bytes", "bytes", "sent", "IP\'s");

	// Generamos la grafica para los bytes locales enviados
	sprintf(filename_gnuplot, "%s.%s.local_top_bytes_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local", "Bytes", "bytes", "sent", "IP\'s");

	// Generamos la grafica para los bytes TCP locales enviados
	sprintf(filename_gnuplot, "%s.%s.local_top_tcp_bytes_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local TCP", "Bytes", "bytes", "sent", "IP\'s");

	// Generamos la grafica para los bytes UDP locales enviados
	sprintf(filename_gnuplot, "%s.%s.local_top_udp_bytes_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local UDP", "Bytes", "bytes", "sent", "IP\'s");

	// Generamos la grafica para los bytes ICMP locales enviados
	sprintf(filename_gnuplot, "%s.%s.local_top_icmp_bytes_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local ICMP", "Bytes", "bytes", "sent", "IP\'s");

	// Generamos la grafica para los bytes OTHERS locales enviados
	sprintf(filename_gnuplot, "%s.%s.local_top_others_bytes_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local OTHERS", "Bytes", "bytes", "sent", "IP\'s");


	// Generamos la grafica para las conexiones totales recibidos
	sprintf(filename_gnuplot, "%s.%s.total_top_connections_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total", "Connections", "connections", "received", "IP\'s");

	// Generamos la grafica para las conexiones TCP totales recibidos
	sprintf(filename_gnuplot, "%s.%s.total_top_tcp_connections_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total TCP", "Connections", "connections", "received", "IP\'s");

	// Generamos la grafica para las conexiones UDP totales recibidos
	sprintf(filename_gnuplot, "%s.%s.total_top_udp_connections_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total UDP", "Connections", "connections", "received", "IP\'s");

	// Generamos la grafica para las conexiones ICMP totales recibidos
	sprintf(filename_gnuplot, "%s.%s.total_top_icmp_connections_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total ICMP", "Connections", "connections", "received", "IP\'s");

	// Generamos la grafica para las conexiones OTHERS totales recibidos
	sprintf(filename_gnuplot, "%s.%s.total_top_others_connections_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total OTHERS", "Connections", "connections", "received", "IP\'s");

	// Generamos la grafica para las conexiones locales recibidos
	sprintf(filename_gnuplot, "%s.%s.local_top_connections_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local", "Connections", "connections", "received", "IP\'s");

	// Generamos la grafica para las conexiones TCP locales recibidos
	sprintf(filename_gnuplot, "%s.%s.local_top_tcp_connections_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local TCP", "Connections", "connections", "received", "IP\'s");

	// Generamos la grafica para las conexiones UDP locales recibidos
	sprintf(filename_gnuplot, "%s.%s.local_top_udp_connections_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local UDP", "Connections", "connections", "received", "IP\'s");

	// Generamos la grafica para las conexiones ICMP locales recibidos
	sprintf(filename_gnuplot, "%s.%s.local_top_icmp_connections_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local ICMP", "Connections", "connections", "received", "IP\'s");

	// Generamos la grafica para las conexiones OTHERS locales recibidos
	sprintf(filename_gnuplot, "%s.%s.local_top_others_connections_received", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local OTHERS", "Connections", "connections", "received", "IP\'s");


	// Generamos la grafica para las conexiones totales enviadas
	sprintf(filename_gnuplot, "%s.%s.total_top_connections_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total", "Connections", "connections", "sent", "IP\'s");

	// Generamos la grafica para las conexiones TCP totales enviadas
	sprintf(filename_gnuplot, "%s.%s.total_top_tcp_connections_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total TCP", "Connections", "connections", "sent", "IP\'s");

	// Generamos la grafica para las conexiones UDP totales enviadas
	sprintf(filename_gnuplot, "%s.%s.total_top_udp_connections_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total UDP", "Connections", "connections", "sent", "IP\'s");

	// Generamos la grafica para las conexiones ICMP totales enviadas
	sprintf(filename_gnuplot, "%s.%s.total_top_icmp_connections_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total ICMP", "Connections", "connections", "sent", "IP\'s");

	// Generamos la grafica para las conexiones OTHERS totales enviadas
	sprintf(filename_gnuplot, "%s.%s.total_top_others_connections_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total OTHERS", "Connections", "connections", "sent", "IP\'s");

	// Generamos la grafica para las conexiones locales enviadas
	sprintf(filename_gnuplot, "%s.%s.local_top_connections_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local", "Connections", "connections", "sent", "IP\'s");

	// Generamos la grafica para las conexiones TCP locales enviadas
	sprintf(filename_gnuplot, "%s.%s.local_top_tcp_connections_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local TCP", "Connections", "connections", "sent", "IP\'s");

	// Generamos la grafica para las conexiones UDP locales enviadas
	sprintf(filename_gnuplot, "%s.%s.local_top_udp_connections_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local UDP", "Connections", "connections", "sent", "IP\'s");

	// Generamos la grafica para las conexiones ICMP locales enviadas
	sprintf(filename_gnuplot, "%s.%s.local_top_icmp_connections_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local ICMP", "Connections", "connections", "sent", "IP\'s");

	// Generamos la grafica para las conexiones OTHERS locales enviadas
	sprintf(filename_gnuplot, "%s.%s.local_top_others_connections_sent", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local OTHERS", "Connections", "connections", "sent", "IP\'s");


	// Generamos la grafica para los puertos TCP totales
	sprintf(filename_gnuplot, "%s.%s.total_top_tcp_ports", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total TCP", "Ports", "ports", "more accessed", "Ports");

	// Generamos la grafica para los puertos UDP totales
	sprintf(filename_gnuplot, "%s.%s.total_top_udp_ports", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Total UDP", "Ports", "ports", "more accessed", "Ports");

	// Generamos la grafica para los puertos TCP locales
	sprintf(filename_gnuplot, "%s.%s.local_top_tcp_ports", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local TCP", "Ports", "ports", "more accessed", "Ports");

	// Generamos la grafica para los puertos UDP locales
	sprintf(filename_gnuplot, "%s.%s.local_top_udp_ports", output_file, interface);
	GNUPLOT_STATISTICS_TOP(fd_graph, tmp_graphic_file, top_n, filename_gnuplot, interface, graphic_string, initial_time_struct.tm_mday, \
			initial_time_struct.tm_mon+1, initial_time_struct.tm_year+1900, initial_time_struct.tm_hour, initial_time_struct.tm_min, \
			final_time_struct.tm_mday, final_time_struct.tm_mon+1, final_time_struct.tm_year+1900, final_time_struct.tm_hour, \
			final_time_struct.tm_min, aux_line, aux_line2, gnuplot_bin_path, "Local UDP", "Ports", "ports", "more accessed", "Ports");

	return 0;
}
